@@ -1,7 +1,6 @@
 #!/usr/bin/env python
 
 import copy
-                
 import numarray as NA
 from tables import *
 import random
@@ -13,23 +12,23 @@ class Small(IsDescription):
     values will become their types. The IsDescription class will take care
     the user will not add any new variables and that its type is
     correct."""
-    
-    var1 = StringCol(length=4, dflt="")
-    var2 = IntCol(0)
-    var3 = FloatCol(0)
+
+    var1 = StringCol(length=4, dflt="",pos=2)
+    var2 = IntCol(0,pos=1)
+    var3 = FloatCol(0,pos=0)
 
 # Define a user record to characterize some kind of particles
 class Medium(IsDescription):
-    name        = StringCol(length=16, dflt="")  # 16-character String
-    float1      = Col("Float64", 2, NA.arange(2))
+    name        = StringCol(length=16, dflt="",pos=0)  # 16-character String
+    float1      = Col("Float64", 2, NA.arange(2),pos=1)
     #float1      = Col("Float64", 1, 2.3)
     #float2      = Col("Float64", 1, 2.3)
     #zADCcount    = Col("Int16", 1, 0)    # signed short integer
-    ADCcount    = Col("Int32", 1, 0)    # signed short integer
-    grid_i      = Col("Int32", 1, 0)    # integer
-    grid_j      = Col("Int32", 1, 0)    # integer
-    pressure    = Col("Float32", 1, 0)    # float  (single-precision)
-    energy      = Col("Float64", 1, 0)    # double (double-precision)
+    ADCcount    = Col("Int32", 1, 0,pos=6)    # signed short integer
+    grid_i      = Col("Int32", 1, 0,pos=7)    # integer
+    grid_j      = Col("Int32", 1, 0,pos=8)    # integer
+    pressure    = Col("Float32", 1, 0,pos=9)    # float  (single-precision)
+    energy      = Col("Float64", 1, 0,pos=2)    # double (double-precision)
 
 # Define a user record to characterize some kind of particles
 class Big(IsDescription):
@@ -52,7 +51,7 @@ def createFile(filename, totalrows, filters, recsize):
 
     # Table title
     title = "This is the table title"
-    
+
     # Create a Table instance
     group = fileh.root
     rowswritten = 0
@@ -90,12 +89,12 @@ def createFile(filename, totalrows, filters, recsize):
                 #d['float1'][0] = float(i)
                 #d['float2'][0] = float(i*2)
                 # Common part with medium
-                d['grid_i'] = i 
+                d['grid_i'] = i
                 d['grid_j'] = 10 - i
                 d['pressure'] = float(i*i)
                 # d['energy'] = float(d['pressure'] ** 4)
                 d['energy'] = d['pressure']
-                # d['idnumber'] = i * (2 ** 34) 
+                # d['idnumber'] = i * (2 ** 34)
                 d.append()
         elif recsize == "medium":
             for i in xrange(totalrows):
@@ -105,7 +104,7 @@ def createFile(filename, totalrows, filters, recsize):
                 #d['float1'] = i
                 #d['float2'] = float(i)
                 # Common part with big:
-                d['grid_i'] = i 
+                d['grid_i'] = i
                 d['grid_j'] = 10 - i
                 d['pressure'] = i*2
                 # d['energy'] = float(d['pressure'] ** 4)
@@ -121,7 +120,6 @@ def createFile(filename, totalrows, filters, recsize):
                 #d['var3'] = 12.1e10
                 d['var3'] = totalrows-i
                 d.append()  # This is a 10% faster than table.append()
-		    
         rowswritten += totalrows
 
         if recsize == "small":
@@ -134,10 +132,9 @@ def createFile(filename, totalrows, filters, recsize):
         group2 = fileh.createGroup(group, 'group'+str(j))
         # Iterate over this new group (group2)
         group = group2
-    
+
     # Close the file (eventually destroy the extended type)
     fileh.close()
-    
     return (rowswritten, rowsize)
 
 def readFile(filename, recsize, verbose):
@@ -158,25 +155,25 @@ def readFile(filename, recsize, verbose):
                 print "MaxTuples:", table._v_maxTuples
 
             if recsize == "big" or recsize == "medium":
-                #e = [ p.float1 for p in table.iterrows() 
+                #e = [ p.float1 for p in table.iterrows()
                 #      if p.grid_i < 2 ]
                 #e = [ str(p) for p in table.iterrows() ]
                 #      if p.grid_i < 2 ]
-#                 e = [ p['grid_i'] for p in table.iterrows() 
+#                 e = [ p['grid_i'] for p in table.iterrows()
 #                       if p['grid_j'] == 20 and p['grid_i'] < 20 ]
 #                 e = [ p['grid_i'] for p in table
 #                       if p['grid_i'] <= 2 ]
 #                e = [ p['grid_i'] for p in table.where("grid_i<=20")]
                 e = [ p['grid_i'] for p in
                       table.where(table.cols.grid_i<20)]
-#                 e = [ p['grid_i'] for p in table.iterrows() 
+#                 e = [ p['grid_i'] for p in table.iterrows()
 #                       if p.nrow() == 20 ]
-#                 e = [ table.delrow(p.nrow()) for p in table.iterrows() 
+#                 e = [ table.delrow(p.nrow()) for p in table.iterrows()
 #                       if p.nrow() == 20 ]
                 # The version with a for loop is only 1% better than
                 # comprenhension list
                 #e = []
-                #for p in table.iterrows(): 
+                #for p in table.iterrows():
                 #    if p.grid_i < 20:
                 #        e.append(p.grid_j)
             else:  # small record case
@@ -208,7 +205,7 @@ def readFile(filename, recsize, verbose):
 #                 e = [ p['var3'] for p in table.where(table.cols.var2 <=20)
                 e = [ p['var3'] for p in table
                        if p['var2'] <= 10 ]
-#                  e = [ p['var3'] for p in table.whereInRange(table.cols.var2 <=20)]
+#                  e = [ p['var3'] for p in table._whereInRange(table.cols.var2 <=20)]
                 #e = [ p['var3'] for p in table.iterrows(0,21) ]
 #                  e = [ p['var3'] for p in table.iterrows()
 #                       if p.nrow() <= 20 ]
@@ -228,7 +225,7 @@ def readFile(filename, recsize, verbose):
             row += 1
             if verbose:
                 print "Total selected records ==> ", len(e)
-        
+
     # Close the file (eventually destroy the extended type)
     fileh.close()
 
@@ -259,7 +256,7 @@ def readField(filename, field, rng, verbose):
             if verbose:
                 print "Selected rows ==> ", e
                 print "Total selected rows ==> ", len(e)
-        
+
     # Close the file (eventually destroy the extended type)
     fileh.close()
     return (rowsread, rowsize)
@@ -272,9 +269,9 @@ if __name__=="__main__":
         psyco_imported = 1
     except:
         psyco_imported = 0
-    
+
     import time
-    
+
     usage = """usage: %s [-v] [-p] [-R range] [-r] [-w] [-s recsize] [-f field] [-c level] [-l complib] [-i iterations] [-S] [-F] file
             -v verbose
 	    -p use "psyco" if available
@@ -286,7 +283,7 @@ if __name__=="__main__":
             -c sets a compression level (do not set it or 0 for no compression)
             -S activate shuffling filter
             -F activate fletcher32 filter
-            -l sets the compression library to be used ("zlib", "lzo", "ucl")
+            -l sets the compression library to be used ("zlib", "lzo", "ucl", "bzip2")
             -i sets the number of rows in each table\n""" % sys.argv[0]
 
     try:
@@ -296,7 +293,7 @@ if __name__=="__main__":
         sys.exit(0)
 
     # if we pass too much parameters, abort
-    if len(pargs) <> 1: 
+    if len(pargs) <> 1:
         sys.stderr.write(usage)
         sys.exit(0)
 
@@ -343,7 +340,7 @@ if __name__=="__main__":
             complib = option[1]
         elif option[0] == '-i':
             iterations = int(option[1])
-            
+
     # Build the Filters instance
     filters = Filters(complevel=complevel, complib=complib,
                       shuffle=shuffle, fletcher32=fletcher32)
@@ -351,6 +348,9 @@ if __name__=="__main__":
     # Catch the hdf5 file passed as the last argument
     file = pargs[0]
 
+    if verbose:
+	print "numarray version:", NA.__version__
+
     if testwrite:
         print "Compression level:", complevel
         if complevel > 0:
@@ -372,13 +372,13 @@ if __name__=="__main__":
               (tapprows, cpuapprows, tpercent)
 	print "Write rows/sec: ", int(rowsw / float(tapprows))
 	print "Write KB/s :", int(rowsw * rowsz / (tapprows * 1024))
-	
+
     if testread:
 	t1 = time.time()
         cpu1 = time.clock()
         if psyco_imported and usepsyco:
             psyco.bind(readFile)
-            psyco.bind(readField)
+            #psyco.bind(readField)
             pass
         if rng or fieldName:
             (rowsr, rowsz) = readField(file, fieldName, rng, verbose)
@@ -396,4 +396,4 @@ if __name__=="__main__":
               (treadrows, cpureadrows, tpercent)
 	print "Read rows/sec: ", int(rowsr / float(treadrows))
 	print "Read KB/s :", int(rowsr * rowsz / (treadrows * 1024))
-    
+
