@@ -17,3 +17,5 @@ python tutorial1-2.py
 #python tutorial2.py   # This should always fail at the beginning
 python vlarray1.py
 python vlarray2.py
+python nested1.py
+python nested-tut.py
