@@ -18,11 +18,11 @@ class Particle(IsDescription):
 
 # Another way to describe the columns of a table
 Event = {
-    "name"    : Col('CharType', 16),    # 16-character String
-    "TDCcount": Col("UInt8", 1),        # unsigned byte
-    "ADCcount": Col("UInt16", 1),       # Unsigned short integer
-    "xcoord"  : Col("Float32", 1),      # integer
-    "ycoord"  : Col("Float32", 1),      # integer
+    "name"        : StringCol(length=16),
+    "lati"        : IntCol(),
+    "longi"       : IntCol(),
+    "pressure"    : Float32Col(shape=(2,3)),
+    "temperature" : FloatCol(shape=(2,3)),
     }
 
 # Open a file in "w"rite mode
