@@ -1,9 +1,10 @@
 #include "H5LT.h"
 
 #include "tables.h"
-#include "H5Zlzo.h"
-#include "H5Zucl.h"
 #include "utils.h"
+#include "H5Zlzo.h"  		       /* Import FILTER_LZO */
+#include "H5Zucl.h"  		       /* Import FILTER_UCL */
+#include "H5Zbzip2.h"  		       /* Import FILTER_BZIP2 */
 
 #include <string.h>
 #include <stdlib.h>
@@ -37,7 +38,7 @@
 
 herr_t H5ARRAYmake( hid_t loc_id, 
 		    const char *dset_name,
-		    const char *klass,
+		    const char *class_,
 		    const char *title,
 		    const char *flavor,
 		    const char *obversion,    /* The Array VERSION number */
@@ -62,12 +63,14 @@ herr_t H5ARRAYmake( hid_t loc_id,
  int     i;
 
  /* Check if the array has to be chunked or not */
- if (extdim >= 0) {
+ /*if (extdim >= 0) {*/
+ if (dims_chunk) {
    chunked = 1;
  }
 
- if (chunked) {
+ if(chunked) {
    maxdims = malloc(rank*sizeof(hsize_t));
+   if(!maxdims) return -1;
 
    for(i=0;i<rank;i++) {
 /*      printf("dims_chunk[%d]: %d\n", i, dims_chunk[i]); */
@@ -75,7 +78,7 @@ herr_t H5ARRAYmake( hid_t loc_id,
        maxdims[i] = H5S_UNLIMITED;
      }
      else {
-       maxdims[i] = dims[i];
+       maxdims[i] = dims[i] < dims_chunk[i] ? dims_chunk[i] : dims[i];
      }
    }
  }
@@ -120,7 +123,7 @@ herr_t H5ARRAYmake( hid_t loc_id,
      cd_values[0] = compress;
      cd_values[1] = (int)(atof(obversion) * 10);
      if (extdim <0) 
-       cd_values[2] = Array;
+       cd_values[2] = CArray;
      else
        cd_values[2] = EArray;
 
@@ -139,6 +142,11 @@ herr_t H5ARRAYmake( hid_t loc_id,
        if ( H5Pset_filter( plist_id, FILTER_UCL, H5Z_FLAG_OPTIONAL, 3, cd_values) < 0 )
 	 return -1;
      }
+     /* The bzip2 compress does accept parameters */
+     else if (strcmp(complib, "bzip2") == 0) {
+       if ( H5Pset_filter( plist_id, FILTER_BZIP2, H5Z_FLAG_OPTIONAL, 3, cd_values) < 0 )
+	 return -1;
+     }
      else {
        /* Compression library not supported */
        fprintf(stderr, "Compression library not supported\n");
@@ -150,7 +158,7 @@ herr_t H5ARRAYmake( hid_t loc_id,
    if ((dataset_id = H5Dcreate(loc_id, dset_name, datatype,
 			       space_id, plist_id )) < 0 )
      goto out;
- }  
+ }
  else {  			/* Not chunked case */
    /* Create the dataset. */
    if ((dataset_id = H5Dcreate(loc_id, dset_name, datatype,
@@ -185,11 +193,12 @@ herr_t H5ARRAYmake( hid_t loc_id,
  */
     
  /* Attach the CLASS attribute */
- if ( H5LTset_attribute_string( loc_id, dset_name, "CLASS", klass ) < 0 )
+ if ( H5LTset_attribute_string( loc_id, dset_name, "CLASS", class_ ) < 0 )
      goto out;
  if (extdim >= 0) {
    /* Attach the EXTDIM attribute in case of enlargeable arrays */
-   if ( H5LTset_attribute_int( loc_id, dset_name, "EXTDIM", &extdim, 1 ) < 0 )
+   if (H5LT_set_attribute_numerical(loc_id,dset_name,"EXTDIM",
+                                    H5T_NATIVE_INT,&extdim)<0)
      goto out;
  }
    
@@ -344,7 +353,8 @@ out:
  *
  * Comments: Uses memory offsets
  *
- * Modifications: 
+ * Modifications: Norbert Nemec for dealing with arrays of zero dimensions
+ *                Date: Wed, 15 Dec 2004 18:48:07 +0100
  *
  *
  *-------------------------------------------------------------------------
@@ -364,51 +374,52 @@ herr_t H5ARRAYwrite_records( hid_t loc_id,
  hid_t    type_id;
  hid_t    space_id;
  hid_t    mem_space_id;
+ herr_t   retval;
 
  /* Open the dataset. */
  if ( (dataset_id = H5Dopen( loc_id, dset_name )) < 0 )
-  goto out;
+  return -1;
 
  /* Get the datatype */
- if ( (type_id = H5Dget_type( dataset_id )) < 0 )
-  goto out;
+ else if ( (type_id = H5Dget_type( dataset_id )) < 0 )
+  retval = -2;
 
  /* Create a simple memory data space */
- if ( (mem_space_id = H5Screate_simple( rank, count, NULL )) < 0 )
-  return -1;
+ else if ( (mem_space_id = H5Screate_simple( rank, count, NULL )) < 0 )
+  retval = -3;
 
  /* Get the file data space */
- if ( (space_id = H5Dget_space( dataset_id )) < 0 )
-  return -1;
+ else if ( (space_id = H5Dget_space( dataset_id )) < 0 )
+  retval = -4;
 
  /* Define a hyperslab in the dataset */
- if ( H5Sselect_hyperslab( space_id, H5S_SELECT_SET, start, step, count, NULL) < 0 )
-   goto out;
+ else if ( rank != 0 && H5Sselect_hyperslab( space_id, H5S_SELECT_SET, start,
+					     step, count, NULL) < 0 )
+  retval = -5;
 
- if ( H5Dwrite( dataset_id, type_id, mem_space_id, space_id, H5P_DEFAULT, data ) < 0 )
-     goto out;
+ else if ( H5Dwrite( dataset_id, type_id, mem_space_id, space_id, H5P_DEFAULT,
+		     data ) < 0 )
+  retval = -6;
 
  /* Terminate access to the dataspace */
- if ( H5Sclose( mem_space_id ) < 0 )
-  goto out;
+ else if ( H5Sclose( mem_space_id ) < 0 )
+  retval = -7;
  
- if ( H5Sclose( space_id ) < 0 )
-  goto out;
+ else if ( H5Sclose( space_id ) < 0 )
+  retval = -8;
  
  /* Release the datatype. */
- if ( H5Tclose( type_id ) < 0 )
-  goto out;
+ else if ( H5Tclose( type_id ) < 0 )
+  retval = -9;
+
+ else
+  retval = 0;
 
  /* End access to the dataset */
  if ( H5Dclose( dataset_id ) < 0 )
-  goto out;
-
-return 0;
-
-out:
- H5Dclose( dataset_id );
- return -1;
-
+  return retval - 10;
+ else
+  return retval;
 }
 
 
@@ -491,7 +502,7 @@ herr_t H5ARRAYtruncate( hid_t loc_id,
  if ( H5Dclose( dataset_id ) < 0 )
   return -1;
 
- return size;
+ return 0;
 
 out:
  H5Dclose( dataset_id );
@@ -508,7 +519,7 @@ out:
  *
  * Return: Success: 0, Failure: -1
  *
- * Programmer: Francesc Altet, falted@pytables.org
+ * Programmer: Francesc Altet, faltet@carabos.com
  *
  * Date: October 22, 2002
  *
@@ -644,7 +655,7 @@ out:
  *
  * Return: Success: 0, Failure: -1
  *
- * Programmer: Francesc Altet, falted@pytables.org
+ * Programmer: Francesc Altet, faltet@carabos.com
  *
  * Date: December 16, 2003
  *
@@ -769,7 +780,7 @@ out:
  *
  * Return: Success: 0, Failure: -1
  *
- * Programmer: Francesc Altet, falted@pytables.org
+ * Programmer: Francesc Altet, faltet@carabos.com
  *
  * Date: June 21, 2004
  *
@@ -983,7 +994,8 @@ out:
 /*-------------------------------------------------------------------------
  * Function: H5ARRAYget_chunksize
  *
- * Purpose: Gets the chunksize along the extensible dimension of an earray.
+ * Purpose: Gets the chunksize along the extensible dimension of a 
+ *          carray/earray.
  *
  * Return: Success: chunksize, Failure: -1
  *
@@ -999,8 +1011,9 @@ herr_t H5ARRAYget_chunksize( hid_t loc_id,
 			     int rank,
 			     hsize_t *dims_chunk)
 {
-  hid_t       dataset_id;  
-  hid_t       plist_id;
+  hid_t        dataset_id;  
+  hid_t        plist_id;
+  H5D_layout_t layout;
 
   /* Open the dataset. */
   if ( (dataset_id = H5Dopen( loc_id, dset_name )) < 0 )
@@ -1010,6 +1023,11 @@ herr_t H5ARRAYget_chunksize( hid_t loc_id,
   if ( (plist_id = H5Dget_create_plist( dataset_id )) < 0 )
     goto out;
 
+  /* Get the daraset layout */
+  layout = H5Pget_layout(plist_id);
+  if (layout != H5D_CHUNKED)
+    goto out;
+
   /* Get the chunksize for all dimensions */
   if (H5Pget_chunk(plist_id, rank, dims_chunk ) < 0 )
     goto out;
@@ -1038,15 +1056,16 @@ out:
 /* Addition: Now, this routine can deal with both array and
    atomic datatypes. 2003-01-29 */
 
-hid_t H5ARRAYget_info( hid_t loc_id, 
+hid_t H5ARRAYget_info( hid_t loc_id,
 		       const char *dset_name,
 		       hsize_t *dims,
+		       hsize_t *maxdims,
 		       hid_t *super_type_id,
 		       H5T_class_t *super_class_id,
 		       char *byteorder)
 {
-  hid_t       dataset_id;  
-  hid_t       space_id; 
+  hid_t       dataset_id;
+  hid_t       space_id;
   H5T_class_t class_arr_id;
   H5T_order_t order;
   hid_t       type_id;
@@ -1061,12 +1080,12 @@ hid_t H5ARRAYget_info( hid_t loc_id,
   /* Get the class. */
   class_arr_id = H5Tget_class( type_id );
 
-  /* Check if this is an array class object*/ 
+  /* Check if this is an array class object*/
   if ( class_arr_id == H5T_ARRAY) {
 
     /* Get the array base component */
     *super_type_id = H5Tget_super( type_id );
- 
+
     /* Get the class of base component. */
     *super_class_id = H5Tget_class( *super_type_id );
 
@@ -1082,34 +1101,27 @@ hid_t H5ARRAYget_info( hid_t loc_id,
   else {
     *super_class_id = class_arr_id;
     *super_type_id = type_id;
-   
+
     /* Get the dataspace handle */
     if ( (space_id = H5Dget_space( dataset_id )) < 0 )
       goto out;
 
     /* Get dimensions */
-    if ( H5Sget_simple_extent_dims( space_id, dims, NULL) < 0 )
+    if ( H5Sget_simple_extent_dims( space_id, dims, maxdims) < 0 )
       goto out;
 
     /* Terminate access to the dataspace */
     if ( H5Sclose( space_id ) < 0 )
       goto out;
- 
+
   }
 
   /* Get the byteorder */
-  /* Only class integer and float can be byteordered */
-  if ( (*super_class_id == H5T_INTEGER) || (*super_class_id == H5T_FLOAT)
-       || (*super_class_id == H5T_BITFIELD) || (*super_class_id == H5T_COMPOUND)) {
-    order = get_order( *super_type_id );
-    if (order == H5T_ORDER_LE) 
-      strcpy(byteorder, "little");
-    else if (order == H5T_ORDER_BE)
-      strcpy(byteorder, "big");
-    else {
-      fprintf(stderr, "Error: unsupported byteorder: %d\n", order);
-      goto out;
-    }
+  /* Only integer, float, time and enumerate classes can be byteordered */
+  if ((*super_class_id == H5T_INTEGER) || (*super_class_id == H5T_FLOAT)
+      || (*super_class_id == H5T_BITFIELD) || (*super_class_id == H5T_COMPOUND)
+      || (*super_class_id == H5T_TIME) || (*super_class_id == H5T_ENUM)) {
+    get_order(*super_type_id, byteorder);
   }
   else {
     strcpy(byteorder, "non-relevant");
