@@ -18,14 +18,14 @@
 
 
 /*-------------------------------------------------------------------------
- * 
+ *
  * Private functions
- * 
+ *
  *-------------------------------------------------------------------------
  */
 
 
-herr_t H5LT_open_id( hid_t loc_id, 
+herr_t H5LT_open_id( hid_t loc_id,
                      const char *obj_name, 
                      int obj_type );
 
@@ -1078,29 +1078,111 @@ out:
  */
 
 
-herr_t H5LT_set_attribute_numerical( hid_t loc_id, 
-                                     const char *obj_name, 
+herr_t H5LT_set_attribute_numerical( hid_t loc_id,
+                                     const char *obj_name,
                                      const char *attr_name,
-                                     size_t size,
                                      hid_t type_id,
-                                     const void *data ) 
+                                     const void *data )
 {
 
- hid_t      obj_id, space_id, attr_id;  
- hsize_t    dim_size=size;
+ hid_t      obj_id, space_id, attr_id;
  int        has_attr;
  H5G_stat_t statbuf;
- 
+
  /* Get the type of object */
  if (H5Gget_objinfo(loc_id, obj_name, 1, &statbuf)<0)
   return -1;
- 
+
  /* Open the object */
  if ((obj_id = H5LT_open_id( loc_id, obj_name, statbuf.type )) < 0)
   return -1;
 
  /* Create the data space for the attribute. */
- if ( (space_id = H5Screate_simple( 1, &dim_size, NULL )) < 0 )
+/*  if ( (space_id = H5Screate_simple( 1, &dim_size, NULL )) < 0 ) */
+ /* For File version 1.4 on, the numerical attributes will be saved as
+    regular scalars, as they should */
+ if ( (space_id = H5Screate( H5S_SCALAR )) < 0 )
+  goto out;
+
+  /* Verify if the attribute already exists */
+ has_attr = H5LT_find_attribute( obj_id, attr_name );
+
+ /* The attribute already exists, delete it */
+ if ( has_attr == 1 )
+ {
+  if ( H5Adelete( obj_id, attr_name ) < 0 )
+    goto out;
+ }
+
+ /* Create the attribute. */
+ if ( (attr_id = H5Acreate( obj_id, attr_name, type_id, space_id, H5P_DEFAULT )) < 0 )
+  goto out;
+
+ /* Write the attribute data. */
+ if ( H5Awrite( attr_id, type_id, data ) < 0 )
+  goto out;
+
+ /* Close the attribute. */
+ if ( H5Aclose( attr_id ) < 0 )
+  goto out;
+
+ /* Close the dataspace. */
+ if ( H5Sclose( space_id ) < 0 )
+  goto out;
+
+ /* Close the object */
+ if ( H5LT_close_id( obj_id, statbuf.type ) < 0 )
+  return -1;
+
+ return 0;
+
+out:
+ H5LT_close_id( obj_id, statbuf.type );
+ return -1;
+}
+
+
+
+/*-------------------------------------------------------------------------
+ * Function: H5LTset_attribute_numerical_NAarray
+ *
+ * Purpose: write an array attribute
+ *
+ * Return: Success: 0, Failure: -1
+ *
+ * Programmer:
+ *
+ * Date: July 25, 2001
+ *
+ * Comments:
+ *
+ *-------------------------------------------------------------------------
+ */
+
+
+herr_t H5LTset_attribute_numerical_NAarray( hid_t loc_id,
+                                            const char *obj_name,
+                                            const char *attr_name,
+                                            size_t rank,
+                                            hsize_t *dims,
+                                            hid_t type_id,
+                                            const void *data )
+{
+
+ hid_t      obj_id, space_id, attr_id;
+ int        has_attr;
+ H5G_stat_t statbuf;
+
+ /* Get the type of object */
+ if (H5Gget_objinfo(loc_id, obj_name, 1, &statbuf)<0)
+  return -1;
+
+ /* Open the object */
+ if ((obj_id = H5LT_open_id( loc_id, obj_name, statbuf.type )) < 0)
+  return -1;
+
+ /* Create the data space for the attribute. */
+ if ( (space_id = H5Screate_simple( rank, dims, NULL )) < 0 )
   goto out;
 
   /* Verify if the attribute already exists */
@@ -1257,11 +1339,10 @@ herr_t H5LT_close_id( hid_t obj_id,
 herr_t H5LTset_attribute_char( hid_t loc_id, 
                                const char *obj_name, 
                                const char *attr_name,
-                               const char *data,
-                               size_t size ) 
+                               const char *data ) 
 {
  
- if ( H5LT_set_attribute_numerical( loc_id, obj_name, attr_name, size, 
+ if ( H5LT_set_attribute_numerical( loc_id, obj_name, attr_name,
       H5T_NATIVE_CHAR, data ) < 0 )
   return -1;
 
@@ -1287,11 +1368,10 @@ herr_t H5LTset_attribute_char( hid_t loc_id,
 herr_t H5LTset_attribute_uchar( hid_t loc_id, 
                                 const char *obj_name, 
                                 const char *attr_name,
-                                const unsigned char *data,
-                                size_t size ) 
+                                const unsigned char *data ) 
 {
  
- if ( H5LT_set_attribute_numerical( loc_id, obj_name, attr_name, size, 
+ if ( H5LT_set_attribute_numerical( loc_id, obj_name, attr_name,
       H5T_NATIVE_UCHAR, data ) < 0 )
   return -1;
 
@@ -1318,11 +1398,10 @@ herr_t H5LTset_attribute_uchar( hid_t loc_id,
 herr_t H5LTset_attribute_short( hid_t loc_id, 
                                 const char *obj_name, 
                                 const char *attr_name,
-                                const short *data,
-                                size_t size ) 
+                                const short *data ) 
 {
  
- if ( H5LT_set_attribute_numerical( loc_id, obj_name, attr_name, size, 
+ if ( H5LT_set_attribute_numerical( loc_id, obj_name, attr_name,
       H5T_NATIVE_SHORT, data ) < 0 )
   return -1;
 
@@ -1349,11 +1428,10 @@ herr_t H5LTset_attribute_short( hid_t loc_id,
 herr_t H5LTset_attribute_ushort( hid_t loc_id, 
                                 const char *obj_name, 
                                 const char *attr_name,
-                                const unsigned short *data,
-                                size_t size ) 
+                                const unsigned short *data ) 
 {
  
- if ( H5LT_set_attribute_numerical( loc_id, obj_name, attr_name, size, 
+ if ( H5LT_set_attribute_numerical( loc_id, obj_name, attr_name,
       H5T_NATIVE_USHORT, data ) < 0 )
   return -1;
 
@@ -1380,11 +1458,10 @@ herr_t H5LTset_attribute_ushort( hid_t loc_id,
 herr_t H5LTset_attribute_int( hid_t loc_id, 
                               const char *obj_name, 
                               const char *attr_name,
-                              const int *data,
-                              size_t size ) 
+                              const int *data ) 
 {
  
- if ( H5LT_set_attribute_numerical( loc_id, obj_name, attr_name, size, 
+ if ( H5LT_set_attribute_numerical( loc_id, obj_name, attr_name,
       H5T_NATIVE_INT, data ) < 0 )
   return -1;
 
@@ -1411,11 +1488,10 @@ herr_t H5LTset_attribute_int( hid_t loc_id,
 herr_t H5LTset_attribute_uint( hid_t loc_id, 
                               const char *obj_name, 
                               const char *attr_name,
-                              const unsigned int *data,
-                              size_t size ) 
+                              const unsigned int *data ) 
 {
  
- if ( H5LT_set_attribute_numerical( loc_id, obj_name, attr_name, size, 
+ if ( H5LT_set_attribute_numerical( loc_id, obj_name, attr_name,
       H5T_NATIVE_UINT, data ) < 0 )
   return -1;
 
@@ -1443,11 +1519,10 @@ herr_t H5LTset_attribute_uint( hid_t loc_id,
 herr_t H5LTset_attribute_long( hid_t loc_id, 
                                const char *obj_name, 
                                const char *attr_name,
-                               const long *data,
-                               size_t size ) 
+                               const long *data ) 
 {
  
- if ( H5LT_set_attribute_numerical( loc_id, obj_name, attr_name, size, 
+ if ( H5LT_set_attribute_numerical( loc_id, obj_name, attr_name,
       H5T_NATIVE_LONG, data ) < 0 )
   return -1;
 
@@ -1474,11 +1549,10 @@ herr_t H5LTset_attribute_long( hid_t loc_id,
 herr_t H5LTset_attribute_ulong( hid_t loc_id, 
                                const char *obj_name, 
                                const char *attr_name,
-                               const unsigned long *data,
-                               size_t size ) 
+                               const unsigned long *data ) 
 {
  
- if ( H5LT_set_attribute_numerical( loc_id, obj_name, attr_name, size, 
+ if ( H5LT_set_attribute_numerical( loc_id, obj_name, attr_name,
       H5T_NATIVE_ULONG, data ) < 0 )
   return -1;
 
@@ -1507,11 +1581,10 @@ herr_t H5LTset_attribute_ulong( hid_t loc_id,
 herr_t H5LTset_attribute_float( hid_t loc_id, 
                                 const char *obj_name, 
                                 const char *attr_name,
-                                const float *data,
-                                size_t size )
+                                const float *data )
 {
 
- if ( H5LT_set_attribute_numerical( loc_id, obj_name, attr_name, size, 
+ if ( H5LT_set_attribute_numerical( loc_id, obj_name, attr_name,
       H5T_NATIVE_FLOAT, data ) < 0 )
   return -1;
 
@@ -1539,11 +1612,10 @@ herr_t H5LTset_attribute_float( hid_t loc_id,
 herr_t H5LTset_attribute_double( hid_t loc_id, 
                                  const char *obj_name, 
                                  const char *attr_name,
-                                 const double *data,
-                                 size_t size ) 
+                                 const double *data ) 
 {
  
- if ( H5LT_set_attribute_numerical( loc_id, obj_name, attr_name, size, 
+ if ( H5LT_set_attribute_numerical( loc_id, obj_name, attr_name,
       H5T_NATIVE_DOUBLE, data ) < 0 )
   return -1;
 
@@ -1722,10 +1794,10 @@ herr_t H5LTget_attribute_info( hid_t loc_id,
                                const char *attr_name,
                                hsize_t *dims,
                                H5T_class_t *type_class,
-                               size_t *type_size )
+                               size_t *type_size,
+                               hid_t *type_id)
 {
  hid_t       attr_id;  
- hid_t       type_id;
  hid_t       space_id; 
  H5G_stat_t  statbuf;
  hid_t       obj_id;  
@@ -1746,13 +1818,13 @@ herr_t H5LTget_attribute_info( hid_t loc_id,
  }
 
  /* Get an identifier for the datatype. */
- type_id = H5Aget_type( attr_id );
+ *type_id = H5Aget_type( attr_id );
 
  /* Get the class. */
-  *type_class = H5Tget_class( type_id );
+  *type_class = H5Tget_class( *type_id );
 
  /* Get the size. */
-  *type_size = H5Tget_size( type_id );
+  *type_size = H5Tget_size( *type_id );
 
   /* Get the dataspace handle */
  if ( (space_id = H5Aget_space( attr_id )) < 0 )
@@ -1766,10 +1838,6 @@ herr_t H5LTget_attribute_info( hid_t loc_id,
  if ( H5Sclose( space_id ) < 0 )
   goto out;
 
-  /* Release the datatype. */
- if ( H5Tclose( type_id ) )
-  goto out;
-
   /* End access to the attribute */
  if ( H5Aclose( attr_id ) )
   goto out;
@@ -1781,7 +1849,7 @@ herr_t H5LTget_attribute_info( hid_t loc_id,
  return 0;
 
 out:
- H5Tclose( type_id );
+ H5Tclose( *type_id );
  H5Aclose( attr_id );
  H5LT_close_id( obj_id, statbuf.type );
  return -1;
@@ -2236,10 +2304,10 @@ herr_t H5LTget_attribute_int( hid_t loc_id,
  *
  *-------------------------------------------------------------------------
  */
-herr_t H5LTget_attribute_uint( hid_t loc_id, 
-                              const char *obj_name, 
+herr_t H5LTget_attribute_uint( hid_t loc_id,
+                              const char *obj_name,
                               const char *attr_name,
-                              unsigned int *data ) 
+                              unsigned int *data )
 {
 
  /* identifiers */
@@ -2249,7 +2317,7 @@ herr_t H5LTget_attribute_uint( hid_t loc_id,
  /* Get the type of object */
  if (H5Gget_objinfo(loc_id, obj_name, 1, &statbuf)<0)
   return -1;
- 
+
  /* Open the object */
  if ((obj_id = H5LT_open_id( loc_id, obj_name, statbuf.type )) < 0)
   return -1;
@@ -2285,10 +2353,10 @@ herr_t H5LTget_attribute_uint( hid_t loc_id,
  *
  *-------------------------------------------------------------------------
  */
-herr_t H5LTget_attribute_long( hid_t loc_id, 
-                              const char *obj_name, 
+herr_t H5LTget_attribute_long( hid_t loc_id,
+                              const char *obj_name,
                               const char *attr_name,
-                              long *data ) 
+                              long *data )
 {
 
  /* identifiers */
@@ -2298,7 +2366,7 @@ herr_t H5LTget_attribute_long( hid_t loc_id,
  /* Get the type of object */
  if (H5Gget_objinfo(loc_id, obj_name, 1, &statbuf)<0)
   return -1;
- 
+
  /* Open the object */
  if ((obj_id = H5LT_open_id( loc_id, obj_name, statbuf.type )) < 0)
   return -1;
@@ -2332,10 +2400,10 @@ herr_t H5LTget_attribute_long( hid_t loc_id,
  *
  *-------------------------------------------------------------------------
  */
-herr_t H5LTget_attribute_ulong( hid_t loc_id, 
-                              const char *obj_name, 
+herr_t H5LTget_attribute_ulong( hid_t loc_id,
+                              const char *obj_name,
                               const char *attr_name,
-                              unsigned long *data ) 
+                              unsigned long *data )
 {
 
  /* identifiers */
@@ -2345,7 +2413,7 @@ herr_t H5LTget_attribute_ulong( hid_t loc_id,
  /* Get the type of object */
  if (H5Gget_objinfo(loc_id, obj_name, 1, &statbuf)<0)
   return -1;
- 
+
  /* Open the object */
  if ((obj_id = H5LT_open_id( loc_id, obj_name, statbuf.type )) < 0)
   return -1;
@@ -2382,10 +2450,10 @@ herr_t H5LTget_attribute_ulong( hid_t loc_id,
  */
 
 
-herr_t H5LTget_attribute_float( hid_t loc_id, 
-                                const char *obj_name, 
+herr_t H5LTget_attribute_float( hid_t loc_id,
+                                const char *obj_name,
                                 const char *attr_name,
-                                float *data ) 
+                                float *data )
 {
 
  /* identifiers */
@@ -2395,7 +2463,7 @@ herr_t H5LTget_attribute_float( hid_t loc_id,
  /* Get the type of object */
  if (H5Gget_objinfo(loc_id, obj_name, 1, &statbuf)<0)
   return -1;
- 
+
  /* Open the object */
  if ((obj_id = H5LT_open_id( loc_id, obj_name, statbuf.type )) < 0)
   return -1;
@@ -2432,10 +2500,10 @@ herr_t H5LTget_attribute_float( hid_t loc_id,
  */
 
 
-herr_t H5LTget_attribute_double( hid_t loc_id, 
-                                 const char *obj_name, 
+herr_t H5LTget_attribute_double( hid_t loc_id,
+                                 const char *obj_name,
                                  const char *attr_name,
-                                 double *data ) 
+                                 double *data )
 {
 
  /* identifiers */
@@ -2445,7 +2513,7 @@ herr_t H5LTget_attribute_double( hid_t loc_id,
  /* Get the type of object */
  if (H5Gget_objinfo(loc_id, obj_name, 1, &statbuf)<0)
   return -1;
- 
+
  /* Open the object */
  if ((obj_id = H5LT_open_id( loc_id, obj_name, statbuf.type )) < 0)
   return -1;
@@ -2482,11 +2550,11 @@ herr_t H5LTget_attribute_double( hid_t loc_id,
  */
 
 
-herr_t H5LTget_attribute( hid_t loc_id, 
-                          const char *obj_name, 
+herr_t H5LTget_attribute( hid_t loc_id,
+                          const char *obj_name,
                           const char *attr_name,
                           hid_t mem_type_id,
-                          void *data ) 
+                          void *data )
 {
 
  /* identifiers */
@@ -2496,7 +2564,7 @@ herr_t H5LTget_attribute( hid_t loc_id,
  /* Get the type of object */
  if (H5Gget_objinfo(loc_id, obj_name, 1, &statbuf)<0)
   return -1;
- 
+
  /* Open the object */
  if ((obj_id = H5LT_open_id( loc_id, obj_name, statbuf.type )) < 0)
   return -1;
@@ -2535,10 +2603,10 @@ herr_t H5LTget_attribute( hid_t loc_id,
  */
 
 
-herr_t H5LT_get_attribute_mem( hid_t obj_id, 
+herr_t H5LT_get_attribute_mem( hid_t obj_id,
                            const char *attr_name,
                            hid_t mem_type_id,
-                           void *data ) 
+                           void *data )
 {
 
  /* identifiers */
@@ -2581,9 +2649,9 @@ out:
  */
 
 
-herr_t H5LT_get_attribute_disk( hid_t loc_id, 
+herr_t H5LT_get_attribute_disk( hid_t loc_id,
                           const char *attr_name,
-                          void *attr_out ) 
+                          void *attr_out )
 {
 
  /* identifiers */
@@ -2598,7 +2666,7 @@ herr_t H5LT_get_attribute_disk( hid_t loc_id,
 
  if ( H5Aread( attr_id, attr_type, attr_out ) < 0 )
   goto out;
- 
+
  if ( H5Tclose( attr_type )  < 0 )
   goto out;
 
