@@ -154,62 +154,60 @@ herr_t H5LTset_attribute_string( hid_t loc_id,
 herr_t H5LTset_attribute_char( hid_t loc_id, 
                                const char *obj_name, 
                                const char *attr_name,
-                               const char *buffer,
-                               size_t size );
+                               const char *buffer );
 
 herr_t H5LTset_attribute_uchar( hid_t loc_id, 
                                const char *obj_name, 
                                const char *attr_name,
-                               const unsigned char *buffer,
-                               size_t size );
+                               const unsigned char *buffer );
 
 herr_t H5LTset_attribute_short( hid_t loc_id, 
                               const char *obj_name, 
                               const char *attr_name,
-                              const short *buffer,
-                              size_t size );
+                              const short *buffer );
 
 herr_t H5LTset_attribute_ushort( hid_t loc_id, 
                               const char *obj_name, 
                               const char *attr_name,
-                              const unsigned short *buffer,
-                              size_t size );
+                              const unsigned short *buffer );
 
 herr_t H5LTset_attribute_int( hid_t loc_id, 
                               const char *obj_name, 
                               const char *attr_name,
-                              const int *buffer,
-                              size_t size );
+                              const int *buffer );
 
 herr_t H5LTset_attribute_uint( hid_t loc_id, 
                               const char *obj_name, 
                               const char *attr_name,
-                              const unsigned int *buffer,
-                              size_t size );
+                              const unsigned int *buffer );
 
 herr_t H5LTset_attribute_long( hid_t loc_id, 
                                const char *obj_name, 
                                const char *attr_name,
-                               const long *buffer,
-                               size_t size );
+                               const long *buffer );
 
 herr_t H5LTset_attribute_ulong( hid_t loc_id, 
                                const char *obj_name, 
                                const char *attr_name,
-                               const unsigned long *buffer,
-                               size_t size );
+                               const unsigned long *buffer );
 
 herr_t H5LTset_attribute_float( hid_t loc_id, 
                                 const char *obj_name, 
                                 const char *attr_name,
-                                const float *buffer,
-                                size_t size );
+                                const float *buffer );
 
 herr_t H5LTset_attribute_double( hid_t loc_id, 
                                  const char *obj_name, 
                                  const char *attr_name,
-                                 const double *buffer,
-                                 size_t size );
+                                 const double *buffer );
+
+herr_t H5LTset_attribute_numerical_NAarray( hid_t loc_id,
+                                            const char *obj_name,
+                                            const char *attr_name,
+                                            size_t rank,
+                                            hsize_t *dims,
+                                            hid_t type_id,
+                                            const void *data );
 
 /*-------------------------------------------------------------------------
  *
@@ -298,7 +296,8 @@ herr_t H5LTget_attribute_info( hid_t loc_id,
                                const char *attr_name,
                                hsize_t *dims,
                                H5T_class_t *type_class,
-                               size_t *type_size );
+                               size_t *type_size,
+                               hid_t *type_id );
 
 
 
@@ -349,10 +348,9 @@ herr_t H5LT_get_attribute_disk( hid_t obj_id,
 herr_t H5LT_find_attribute( hid_t loc_id, const char *name ); 
 
 
-herr_t H5LT_set_attribute_numerical( hid_t loc_id, 
-                                     const char *obj_name, 
+herr_t H5LT_set_attribute_numerical( hid_t loc_id,
+                                     const char *obj_name,
                                      const char *attr_name,
-                                     size_t size,
                                      hid_t type_id,
                                      const void *data );
 
