@@ -6,17 +6,17 @@
 /*-------------------------------------------------------------------------
  * Function: H5TBOopen_read
  *
- * Purpose: Prepare a table to be read incrementally
+ * Purpose: Open a table to be read
  *
  * Return: Success: 0, Failure: -1
  *
- * Programmer: Francesc Alted, falted@pytables.org
+ * Programmer: Francesc Altet, faltet@carabos.com
  *
  * Date: April 19, 2003
  *
- * Comments: 
+ * Comments:
  *
- * Modifications: 
+ * Modifications:
  *
  *
  *-------------------------------------------------------------------------
@@ -24,56 +24,18 @@
 
 herr_t H5TBOopen_read( hid_t *dataset_id,
 		       hid_t *space_id,
-		       hid_t *mem_type_id, 
-		       hid_t loc_id, 
-		       const char *dset_name,
-		       hsize_t nfields,
-		       char **field_names,
-		       size_t type_size,
-		       size_t *field_offset)
+		       hid_t loc_id,
+		       const char *dset_name)
 {
- hid_t    type_id;    
- hid_t    member_type_id;
- hsize_t  i;
 
  /* Open the dataset. */
  if ( (*dataset_id = H5Dopen( loc_id, dset_name )) < 0 )
   return -1;
 
- /* Get the datatype */
- if ( (type_id = H5Dget_type( *dataset_id )) < 0 )
-  goto out;
-
- /* Create the memory data type. */
- if ((*mem_type_id = H5Tcreate (H5T_COMPOUND, type_size )) < 0 )
-  return -1;
-
- /* Insert fields on the memory data type. We use the same types as in
-    disk */
- for ( i = 0; i < nfields; i++) 
- {
-
-  /* Get the member type */
-  if ( ( member_type_id = H5Tget_member_type( type_id, (int) i )) < 0 )
-   goto out;
-
-  if ( H5Tinsert(*mem_type_id, field_names[i], field_offset[i], member_type_id ) < 0 )
-   goto out;
-
-  /* Release the datatype */
-  if ( H5Tclose( member_type_id ) < 0 )
-   goto out; 
-
- }
-
- /* Release the type */
- if ( H5Tclose( type_id ) < 0 )
-  return -1;
-
  /* Get the dataspace handle */
  if ( (*space_id = H5Dget_space( *dataset_id )) < 0 )
   goto out;
- 
+
 return 0;
 
 out:
@@ -89,13 +51,13 @@ out:
  *
  * Return: Success: 0, Failure: -1
  *
- * Programmer: Francesc Alted, falted@pytables.org
+ * Programmer: Francesc Altet, faltet@carabos.com
  *
  * Date: April 19, 2003
  *
- * Comments: 
+ * Comments:
  *
- * Modifications: 
+ * Modifications:
  *
  *
  *-------------------------------------------------------------------------
@@ -103,16 +65,15 @@ out:
 
 herr_t H5TBOread_records( hid_t *dataset_id,
 			  hid_t *space_id,
-			  hid_t *mem_type_id,
+			  hid_t mem_type_id,
 			  hsize_t start,
 			  hsize_t nrecords,
 			  void *data )
 {
 
- hsize_t  count[1];    
+ hsize_t  count[1];
  hssize_t offset[1];
  hid_t    mem_space_id;
- hsize_t  mem_size[1];
 
  /* Define a hyperslab in the dataset of the size of the records */
  offset[0] = start;
@@ -121,11 +82,10 @@ herr_t H5TBOread_records( hid_t *dataset_id,
   goto out;
 
  /* Create a memory dataspace handle */
- mem_size[0] = count[0];
- if ( (mem_space_id = H5Screate_simple( 1, mem_size, NULL )) < 0 )
+ if ( (mem_space_id = H5Screate_simple( 1, count, NULL )) < 0 )
   goto out;
 
- if ( H5Dread( *dataset_id, *mem_type_id, mem_space_id, *space_id, H5P_DEFAULT, data ) < 0 )
+ if ( H5Dread( *dataset_id, mem_type_id, mem_space_id, *space_id, H5P_DEFAULT, data ) < 0 )
   goto out;
 
  /* Terminate access to the memory dataspace */
@@ -140,29 +100,48 @@ out:
 
 }
 
+/*-------------------------------------------------------------------------
+ * Function: H5TBOread_records
+ *
+ * Purpose: Read selected records from an opened table
+ *
+ * Return: Success: 0, Failure: -1
+ *
+ * Programmer: Francesc Altet, faltet@carabos.com
+ *
+ * Date: April 19, 2003
+ *
+ * Comments:
+ *
+ * Modifications:
+ *
+ *
+ *-------------------------------------------------------------------------
+ */
+
 herr_t H5TBOread_elements( hid_t *dataset_id,
 			   hid_t *space_id,
-			   hid_t *mem_type_id,
+			   hid_t mem_type_id,
 			   hsize_t nrecords,
 			   void *coords,
 			   void *data )
 {
 
  hid_t    mem_space_id;
- hsize_t  mem_size[1];
+ hsize_t  count[1];
 
  /* Define a selection of points in the dataset */
  H5Sselect_none(*space_id); 	/* Delete the previous selection */
 
- if ( H5Sselect_elements(*space_id, H5S_SELECT_SET, (size_t)nrecords, (const hssize_t **)coords) < 0 )
+ if ( H5Sselect_elements(*space_id, H5S_SELECT_SET, (size_t)nrecords, (const hsize_t **)coords) < 0 )
   goto out;
 
  /* Create a memory dataspace handle */
- mem_size[0] = nrecords;
- if ( (mem_space_id = H5Screate_simple( 1, mem_size, NULL )) < 0 )
+ count[0] = nrecords;
+ if ( (mem_space_id = H5Screate_simple( 1, count, NULL )) < 0 )
   goto out;
 
- if ( H5Dread( *dataset_id, *mem_type_id, mem_space_id, *space_id, H5P_DEFAULT, data ) < 0 )
+ if ( H5Dread( *dataset_id, mem_type_id, mem_space_id, *space_id, H5P_DEFAULT, data ) < 0 )
   goto out;
 
  /* Terminate access to the memory dataspace */
@@ -184,30 +163,25 @@ out:
  *
  * Return: Success: 0, Failure: -1
  *
- * Programmer: Francesc Alted, falted@pytables.org
+ * Programmer: Francesc Altet, faltet@carabos.com
  *
  * Date: April 19, 2003
  *
- * Comments: 
+ * Comments:
  *
- * Modifications: 
+ * Modifications:
  *
  *
  *-------------------------------------------------------------------------
  */
 
 herr_t H5TBOclose_read(hid_t *dataset_id,
-		       hid_t *space_id,
-		       hid_t *mem_type_id)
+		       hid_t *space_id)
 {
 
  /* Terminate access to the dataspace */
  if ( H5Sclose( *space_id ) < 0 )
   goto out;
- 
-  /* Release the datatype. */
- if ( H5Tclose( *mem_type_id ) < 0 )
-  goto out;
 
  /* End access to the dataset */
  if ( H5Dclose( *dataset_id ) < 0 )
@@ -231,82 +205,28 @@ out:
  *
  * Return: Success: 0, Failure: -1
  *
- * Programmer: Francesc Alted, falted@pytables.org
+ * Programmer: Francesc Altet, faltet@carabos.com
  *
  * Date: April 20, 2003
  *
- * Comments: 
+ * Comments:
  *
- * Modifications: 
+ * Modifications:
  *
  *
  *-------------------------------------------------------------------------
  */
 
 herr_t H5TBOopen_append( hid_t *dataset_id,
-			 hid_t *mem_type_id,
-			 hid_t loc_id, 
-			 const char *dset_name,
-			 hsize_t nfields,
-			 size_t type_size,
-			 const size_t *field_offset)
+			 hid_t loc_id,
+			 const char *dset_name)
 {
- hid_t    type_id;    
- char     **field_names;
- hid_t    member_type_id;
- hsize_t  i;
-
-  /* Alocate space */
- field_names = malloc( sizeof(char*) * (size_t)nfields );
- for ( i = 0; i < nfields; i++) 
- {
-  field_names[i] = malloc( sizeof(char) * HLTB_MAX_FIELD_LEN );
- }
-
- /* Get field info */
- if ( H5TBget_field_info( loc_id, dset_name, field_names, NULL, NULL, NULL ) < 0 )
-  return -1;
 
  /* Open the dataset. */
  if ( (*dataset_id = H5Dopen( loc_id, dset_name )) < 0 )
   goto out;
 
-  /* Get the datatype */
- if ( (type_id = H5Dget_type( *dataset_id )) < 0 )
-  goto out;
-
- /* Create the memory data type. */
- if ((*mem_type_id = H5Tcreate (H5T_COMPOUND, type_size )) < 0 )
-  return -1;
-
- /* Insert fields on the memory data type */
- for ( i = 0; i < nfields; i++) 
- {
-
-  /* Get the member type */
-  if ( ( member_type_id = H5Tget_member_type( type_id,(int) i )) < 0 )
-   goto out;
-
-  if ( H5Tinsert(*mem_type_id, field_names[i], field_offset[i], member_type_id ) < 0 )
-   goto out;
-
-  /* Close the member type */
-  if ( H5Tclose( member_type_id ) < 0 )
-   goto out;
-
- /* Release resources. */
-  free ( field_names[i] );
-
- }
-
- /* Release resources. */
- free ( field_names );
-
- /* Release the datatype. */
- if ( H5Tclose( type_id ) < 0 )
-  return -1;
-
-return 0;
+ return 0;
 
 out:
  H5Dclose( *dataset_id );
@@ -322,14 +242,14 @@ out:
  *
  * Return: Success: 0, Failure: -1
  *
- * Programmers: 
- *  Francesc Alted, falted@pytables.org
+ * Programmers:
+ *  Francesc Altet, faltet@carabos.com
  *
  * Date: April 20, 2003
  *
  * Comments: Uses memory offsets
  *
- * Modifications: 
+ * Modifications:
  *
  *
  *-------------------------------------------------------------------------
@@ -337,18 +257,16 @@ out:
 
 
 herr_t H5TBOappend_records( hid_t *dataset_id,
-			    hid_t *mem_type_id,
+			    hid_t mem_type_id,
 			    hsize_t nrecords,
 			    hsize_t nrecords_orig,
-			    const void *data )  
+			    const void *data )
 {
  hid_t    space_id = -1; 	/* Shut up the compiler */
  hsize_t  count[1];
  hssize_t offset[1];
  hid_t    mem_space_id = -1;    /* Shut up the compiler */
- int      rank;
  hsize_t  dims[1];
- hsize_t  mem_dims[1];
 
 
  /* Extend the dataset */
@@ -358,25 +276,20 @@ herr_t H5TBOappend_records( hid_t *dataset_id,
   goto out;
 
  /* Create a simple memory data space */
- mem_dims[0]=nrecords;
- if ( (mem_space_id = H5Screate_simple( 1, mem_dims, NULL )) < 0 )
+ count[0]=nrecords;
+ if ( (mem_space_id = H5Screate_simple( 1, count, NULL )) < 0 )
   return -1;
 
  /* Get the file data space */
  if ( (space_id = H5Dget_space( *dataset_id )) < 0 )
   return -1;
 
- /* Get the dimensions */
- if ( (rank = H5Sget_simple_extent_dims( space_id, dims, NULL )) != 1 )
-  goto out;
-
  /* Define a hyperslab in the dataset */
  offset[0] = nrecords_orig;
- count[0]  = nrecords;
  if ( H5Sselect_hyperslab( space_id, H5S_SELECT_SET, offset, NULL, count, NULL) < 0 )
   goto out;
 
- if ( H5Dwrite( *dataset_id, *mem_type_id, mem_space_id, space_id, H5P_DEFAULT, data ) < 0 )
+ if ( H5Dwrite( *dataset_id, mem_type_id, mem_space_id, space_id, H5P_DEFAULT, data ) < 0 )
   goto out;
 
  /* Terminate access to the dataspace */
@@ -385,13 +298,12 @@ herr_t H5TBOappend_records( hid_t *dataset_id,
 
  if ( H5Sclose( space_id ) < 0 )
   goto out;
- 
+
 return 0;
 
 out:
  H5E_BEGIN_TRY {
   H5Dclose(*dataset_id);
-  H5Tclose(*mem_type_id);
   H5Sclose(mem_space_id);
   H5Sclose(space_id);
  } H5E_END_TRY;
@@ -406,29 +318,23 @@ out:
  *
  * Return: Success: 0, Failure: -1
  *
- * Programmer: Francesc Alted, falted@pytables.org
+ * Programmer: Francesc Altet, faltet@carabos.com
  *
  * Date: April 20, 2003
  *
- * Comments: 
+ * Comments:
  *
- * Modifications: 
+ * Modifications:
  *
  *
  *-------------------------------------------------------------------------
  */
 
 herr_t H5TBOclose_append(hid_t *dataset_id,
-			 hid_t *mem_type_id,
 			 hsize_t ntotal_records,
 			 const char *dset_name,
 			 hid_t parent_id)
 {
- hsize_t nrows;
-  
-  /* Release the datatype. */
- if ( H5Tclose( *mem_type_id ) < 0 )
-  goto out;
 
  /* End access to the dataset */
  if ( H5Dclose( *dataset_id ) < 0 )
@@ -439,11 +345,9 @@ herr_t H5TBOclose_append(hid_t *dataset_id,
  *-------------------------------------------------------------------------
  */
 
- nrows = ntotal_records;
  /* Set the attribute */
- if (H5LT_set_attribute_numerical(parent_id,dset_name,"NROWS",1, 
-				  H5T_NATIVE_LLONG,&nrows)<0)
-/*  if ( H5LTset_attribute_int(parent_id, dset_name, "NROWS", &nrows, 1 ) < 0 ) */
+ if (H5LT_set_attribute_numerical(parent_id, dset_name, "NROWS",
+				  H5T_NATIVE_LLONG, &ntotal_records)<0)
    return -1;
 
 return 0;
@@ -467,83 +371,37 @@ out:
  *
  * Comments: Uses memory offsets
  *
- * Modifications: 
+ * Modifications:
  * -  Added a step parameter in order to support strided writing.
- *    Francesc Alted, falted@pytables.org. 2004-08-12
+ *    Francesc Altet, faltet@carabos.com. 2004-08-12
  *
  *
  *-------------------------------------------------------------------------
  */
 
 
-herr_t H5TBOwrite_records( hid_t loc_id, 
+herr_t H5TBOwrite_records( hid_t loc_id,
 			   const char *dset_name,
 			   hsize_t start,
 			   hsize_t nrecords,
 			   hsize_t step,
 			   size_t type_size,
-			   const size_t *field_offset,
-			   const void *data )  
+			   hid_t mem_type_id,
+			   const void *data )
 {
 
  hid_t    dataset_id;
- hid_t    type_id=-1;    
- hsize_t  count[1];    
- hsize_t  stride[1];    
+ hsize_t  count[1];
+ hsize_t  stride[1];
  hssize_t offset[1];
  hid_t    space_id;
  hid_t    mem_space_id;
- hsize_t  mem_size[1];
  hsize_t  dims[1];
- hid_t    mem_type_id=-1;
- hsize_t  nrecords_orig;
- hsize_t  nfields;
- char     **field_names;
- hid_t    member_type_id;
- hsize_t  i;
-
-  /* Get the number of records and fields  */
- if ( H5TBget_table_info ( loc_id, dset_name, &nfields, &nrecords_orig ) < 0 )
-  return -1;
-
-  /* Alocate space */
- field_names = malloc( sizeof(char*) * (size_t)nfields );
- for ( i = 0; i < nfields; i++) 
- {
-  field_names[i] = malloc( sizeof(char) * HLTB_MAX_FIELD_LEN );
- }
-
- /* Get field info */
- if ( H5TBget_field_info( loc_id, dset_name, field_names, NULL, NULL, NULL ) < 0 )
-  return -1;
 
  /* Open the dataset. */
  if ( (dataset_id = H5Dopen( loc_id, dset_name )) < 0 )
   return -1;
 
-  /* Get the datatype */
- if ( (type_id = H5Dget_type( dataset_id )) < 0 )
-  goto out;
-
- /* Create the memory data type. */
- if ((mem_type_id = H5Tcreate (H5T_COMPOUND, type_size )) < 0 )
-  return -1;
-
- /* Insert fields on the memory data type. We use the types from disk */
- for ( i = 0; i < nfields; i++) 
- {
-  /* Get the member type */
-  if ( ( member_type_id = H5Tget_member_type( type_id,(int) i )) < 0 )
-   goto out;
-
-  if ( H5Tinsert(mem_type_id, field_names[i], field_offset[i], member_type_id ) < 0 )
-   goto out;
-
-  /* Close the member type */
-  if ( H5Tclose( member_type_id ) < 0 )
-   goto out;
- }
-
  /* Get the dataspace handle */
  if ( (space_id = H5Dget_space( dataset_id )) < 0 )
   goto out;
@@ -555,7 +413,7 @@ herr_t H5TBOwrite_records( hid_t loc_id,
 /*  if ( start + nrecords > dims[0] ) */
  if ( start + (nrecords-1) * step + 1 > dims[0] )
   goto out;
-  
+
  /* Define a hyperslab in the dataset of the size of the records */
  offset[0] = start;
  stride[0] = step;
@@ -564,8 +422,7 @@ herr_t H5TBOwrite_records( hid_t loc_id,
   goto out;
 
  /* Create a memory dataspace handle */
- mem_size[0] = count[0];
- if ( (mem_space_id = H5Screate_simple( 1, mem_size, NULL )) < 0 )
+ if ( (mem_space_id = H5Screate_simple( 1, count, NULL )) < 0 )
   goto out;
 
  if ( H5Dwrite( dataset_id, mem_type_id, mem_space_id, space_id, H5P_DEFAULT, data ) < 0 )
@@ -578,26 +435,11 @@ herr_t H5TBOwrite_records( hid_t loc_id,
  /* Terminate access to the dataspace */
  if ( H5Sclose( space_id ) < 0 )
   goto out;
- 
- /* Release the datatype. */
- if ( H5Tclose( type_id ) < 0 )
-  goto out;
-
- /* Release the datatype. */
- if ( H5Tclose( mem_type_id ) < 0 )
-  return -1;
 
  /* End access to the dataset */
  if ( H5Dclose( dataset_id ) < 0 )
   return -1;
 
- /* Release resources. */
- for ( i = 0; i < nfields; i++) 
- {
-  free ( field_names[i] );
- }
- free ( field_names );
- 
 return 0;
 
 out:
