@@ -1,8 +1,9 @@
 #include "H5LT.h"
 #include "tables.h"
-#include "H5Zlzo.h"
-#include "H5Zucl.h"
 #include "utils.h"
+#include "H5Zlzo.h"  		       /* Import FILTER_LZO */
+#include "H5Zucl.h"  		       /* Import FILTER_UCL */
+#include "H5Zbzip2.h"  		       /* Import FILTER_BZIP2 */
 #include <string.h>
 #include <stdlib.h>
 
@@ -120,7 +121,7 @@ void test_vltypes_free_custom_orig(void *_mem, void *info)
  *
  * Return: Success: 0, Failure: -1
  *
- * Programmer: F. Alted
+ * Programmer: F. Altet
  *
  * Date: November 08, 2003
  *-------------------------------------------------------------------------
@@ -128,6 +129,7 @@ void test_vltypes_free_custom_orig(void *_mem, void *info)
 
 herr_t H5VLARRAYmake( hid_t loc_id, 
 		      const char *dset_name,
+		      const char *class_,
 		      const char *title,
 		      const char *flavor,
 		      const char *obversion,    /* The Array VERSION number */
@@ -218,6 +220,11 @@ herr_t H5VLARRAYmake( hid_t loc_id,
      if ( H5Pset_filter( plist_id, FILTER_UCL, H5Z_FLAG_OPTIONAL, 3, cd_values) < 0 )
        return -1;
    }
+   /* The bzip2 compress does accept parameters */
+   else if (strcmp(complib, "bzip2") == 0) {
+     if ( H5Pset_filter( plist_id, FILTER_BZIP2, H5Z_FLAG_OPTIONAL, 3, cd_values) < 0 )
+       return -1;
+   }
    else {
      /* Compression library not supported */
      fprintf(stderr, "Compression library not supported\n");
@@ -256,7 +263,7 @@ herr_t H5VLARRAYmake( hid_t loc_id,
  */
     
  /* Attach the CLASS attribute */
- if ( H5LTset_attribute_string( loc_id, dset_name, "CLASS", "VLARRAY" ) < 0 )
+ if ( H5LTset_attribute_string( loc_id, dset_name, "CLASS", class_ ) < 0 )
   goto out;
    
  /* Attach the CLASS attribute */
@@ -289,7 +296,7 @@ out:
  * Return: Success: 0, Failure: -1
  *
  * Programmers: 
- *  Francesc Alted
+ *  Francesc Altet
  *
  * Date: October 30, 2003
  *
@@ -385,7 +392,7 @@ out:
  * Return: Success: 0, Failure: -1
  *
  * Programmers: 
- *  Francesc Alted
+ *  Francesc Altet
  *
  * Date: October 28, 2004
  *
@@ -471,7 +478,7 @@ out:
  *
  * Return: Success: 0, Failure: -1
  *
- * Programmer: Francesc Alted, falted@pytables.org
+ * Programmer: Francesc Altet, faltet@carabos.com
  *
  * Date: November 19, 2003
  *
@@ -586,7 +593,7 @@ out:
  *
  * Return: Success: 0, Failure: -1
  *
- * Programmer: Francesc Alted
+ * Programmer: Francesc Altet
  *
  * Date: November 19, 2003
  *
@@ -655,7 +662,7 @@ out:
  *
  * Return: Success: 0, Failure: -1
  *
- * Programmer: Francesc Alted
+ * Programmer: Francesc Altet
  *
  * Date: November 19, 2003
  *
@@ -722,18 +729,11 @@ hid_t H5VLARRAYget_info( hid_t   loc_id,
   }
 
   /* Get the byteorder */
-  /* Only class integer and float can be byteordered */
+  /* Only integer, float and time classes can be byteordered */
   if ((base_class_id == H5T_INTEGER) || (base_class_id == H5T_FLOAT)
-      || (base_class_id == H5T_BITFIELD) || (base_class_id == H5T_COMPOUND)) {
-    order = get_order( *base_type_id );
-    if (order == H5T_ORDER_LE) 
-      strcpy(base_byteorder, "little");
-    else if (order == H5T_ORDER_BE)
-      strcpy(base_byteorder, "big");
-    else {
-      fprintf(stderr, "Error: unsupported byteorder: %d\n", order);
-      goto out;
-    }
+      || (base_class_id == H5T_BITFIELD) || (base_class_id == H5T_COMPOUND)
+      || (base_class_id == H5T_TIME)) {
+    get_order(*base_type_id, base_byteorder);
   }
   else {
     strcpy(base_byteorder, "non-relevant");
