@@ -19,10 +19,10 @@
    special scenarios (like transmitting through a net with no error
    correction capabilities).  Anyway, this takes only a 1% more of
    space and a 2% more of CPU, which is almost negligible.
-   F. Alted 2003/07/22
+   F. Altet 2003/07/22
 
    Added code for pytables 0.5 backward compatibility.
-   F. Alted 2003/07/28
+   F. Altet 2003/07/28
 
    Now, it seems that CHECKSUM makes the code to crash when running the 
    test_all.py script. De-activate it until more investigations is done.
@@ -32,16 +32,16 @@
 /* Ok. from pytables 0.8 on I decided to let the user select the
    fletcher32 checksum provided in HDF5 1.6 or higher. So, even though
    the CHECKSUM support here seems pretty stable it will be disabled.
-   F. Alted 2004/01/12 */
-#undef CHECKSUM  		       
+   F. Altet 2004/01/12 */
+#undef CHECKSUM
 
 /* Adding more memory to the nrve seems to make it more resistant to
  seg faults. But I don't fully understand were is exactly the problem,
- anyway.  F. Alted 2003/07/24 
- * 
+ anyway.  F. Altet 2003/07/24
+ *
  * I'm suspucious now about a possible interaction between psyco and
  * ucl nrve that makes the checksum to fail. 2003/07/28
- * 
+ *
  */
 /* Adding a combination of the zlib method and ucl to the output buffer */
 /* #define H5Z_UCL_SIZE_ADJUST(s) (ceil((double)((s)*1.001))+((s)/8)+256+12) */
@@ -51,7 +51,7 @@
    This should be further analyzed. */
 /* With the nrv2d enabled by default this seems to work just fine */
 /* #define H5Z_UCL_SIZE_ADJUST(s) ((s)+((s)/8)+256) /\* Correct value *\/ */
-/* Added this for a bit more of safety! */
+/* Use this for a bit more of safety! */
 /* #define H5Z_UCL_SIZE_ADJUST(s) (ceil((double)((s)*1.001))+((s)/8)+256+12) */
 /* After returning always the compressed buffer, and even with the
    current value, UCL nrv2e seems to work just fine. Great!. 2003/12/09
@@ -61,32 +61,35 @@
    H5Z_FLAG_OPTIONAL to the pproperty list when creating chunks. So,
    this should not pose more problems anymore. 2003/12/21  */
 
-#define H5Z_UCL_SIZE_ADJUST(s) ((s)+((s)/8)+256) /* Correct value */
+/* #define H5Z_UCL_SIZE_ADJUST(s) ((s)+((s)/8)+256) /\* Correct value *\/ */
+#define H5Z_UCL_SIZE_ADJUST(s) (ceil((double)((s)*1.001))+((s)/8)+256+12)
 
 int register_ucl(void) {
 
 #ifdef HAVE_UCL_LIB
-
    int status;
-  /* Feed the filter_class data structure */
-   /* 1.6.2 */
+
+#if H5_VERS_MAJOR == 1 && H5_VERS_MINOR < 7
+   /* 1.6.x */
   H5Z_class_t filter_class = {
-    (H5Z_filter_t)FILTER_UCL,	/* filter_id */
-    "ucl", 			/* comment */
-    NULL,                       /* can_apply_func */
-    NULL,                       /* set_local_func */
-    (H5Z_func_t)ucl_deflate     /* filter_func */
+    (H5Z_filter_t)(FILTER_UCL),    /* filter_id */
+    "ucl",                         /* comment */
+    NULL,                          /* can_apply_func */
+    NULL,                          /* set_local_func */
+    (H5Z_func_t)(ucl_deflate)      /* filter_func */
   };
+#else
    /* 1.7.x */
-/*   H5Z_class_t filter_class = { */
-/*     H5Z_CLASS_T_VERS,           /\* H5Z_class_t version *\/ */
-/*     (H5Z_filter_t)FILTER_UCL,	/\* filter_id *\/ */
-/*     1, 1,                       /\* Encoding and decoding enabled *\/ */
-/*     "ucl",	 		/\* comment *\/ */
-/*     NULL,                       /\* can_apply_func *\/ */
-/*     NULL,                       /\* set_local_func *\/ */
-/*     (H5Z_func_t)ucl_deflate     /\* filter_func *\/ */
-/*   }; */
+  H5Z_class_t filter_class = {
+    H5Z_CLASS_T_VERS,             /* H5Z_class_t version */
+    (H5Z_filter_t)(FILTER_UCL),   /* filter_id */
+    1, 1,                         /* Encoding and decoding enabled */
+    "ucl",	 		  /* comment */
+    NULL,                         /* can_apply_func */
+    NULL,                         /* set_local_func */
+    (H5Z_func_t)(ucl_deflate)     /* filter_func */
+  };
+#endif /* if H5_VERSION < "1.7" */
 
   /* Init the ucl library */
   if (ucl_init()!=UCL_E_OK)
@@ -94,7 +97,6 @@ int register_ucl(void) {
 
   /* Register the lzo compressor */
   status = H5Zregister(&filter_class);
-  
   return UCL_VERSION; /* lib is available */
 
 #else
@@ -244,7 +246,7 @@ size_t ucl_deflate(unsigned int flags, size_t cd_nelmts,
       checksum=ucl_adler32(ucl_adler32(0,NULL,0), outbuf, out_len);
   
       /* Compare */
-      if (memcmp(&checksum, (char*)(*buf)+nbytes, 4)) {
+      if (memcmp(&checksum, (unsigned char*)(*buf)+nbytes, 4)) {
 	ret_value = 0; /*fail*/
 	fprintf(stderr,"Checksum failed!.\n");
 	goto done;
@@ -293,14 +295,14 @@ size_t ucl_deflate(unsigned int flags, size_t cd_nelmts,
        
        The nrv2b and nrv2d seems to give no problems. I'm adopting the
        nrv2d which is slightly better (more compression) than
-       nrv2b. The best is, though, the nrv2e. F. Alted 2003/07/22 
+       nrv2b. The best is, though, the nrv2e. F. Altet 2003/07/22 
        
        Note: I've discovered that adding some more space to the
        nrv2e compressor, it seems to work fine. I'm pretty sure that
        this does not solve the problem, it just makes the seg faults
        harder to appear. I'm adopting this strategy in order to keep
        backward compatibility with the existing files writen with
-       pytables 0.5.x.  F. Alted 2003/07/24
+       pytables 0.5.x.  F. Altet 2003/07/24
 
        Note: Finally, I think I've found what was causing problems
        with nrv2e.  It seems that after forcing returning the
@@ -308,7 +310,7 @@ size_t ucl_deflate(unsigned int flags, size_t cd_nelmts,
        crashes due to the UCL compressor has disappeared. I upgraded
        both the Table and Array version to 2.1 in order to get back to
        use nrv2e, that gives a little better compression ratio.
-       F. Alted 2003/12/09
+       F. Altet 2003/12/09
 
     */
 
@@ -328,7 +330,7 @@ size_t ucl_deflate(unsigned int flags, size_t cd_nelmts,
 #endif
       /* Append checksum of *uncompressed* data at the end */
       checksum = ucl_adler32(ucl_adler32(0,NULL,0), *buf, nbytes);
-      memcpy((char*)(z_dst)+z_dst_nbytes, &checksum, 4);
+      memcpy((unsigned char*)(z_dst)+z_dst_nbytes, &checksum, 4);
       z_dst_nbytes += (ucl_uint)4;
       nbytes += 4; 
     }
