@@ -32,9 +32,6 @@
 #   define TRUE (!FALSE)
 #endif
 
-/* Maximum dimensions in an array */
-/* This number has been taken from numarray/numarray.h */ 
-#define MAXDIM  40
 
 /* Use %ld to print the value because long should cover most cases. */
 /* Used to make certain a return value _is_not_ a value */
@@ -47,6 +44,8 @@
     H5Eclear();                                                               \
 } while(0)
 
+herr_t set_cache_size(hid_t file_id, size_t cache_size);
+
 PyObject *_getTablesVersion(void);
 
 /* PyObject *getZLIBVersionInfo(void); */
@@ -63,14 +62,16 @@ PyObject *Aiterate(hid_t loc_id);
 
 H5T_class_t getHDF5ClassID(hid_t loc_id,
 			   const char *name,
-			   H5D_layout_t *layout);
+			   H5D_layout_t *layout,
+			   hid_t *type_id);
 
-PyObject *H5UIget_info( hid_t loc_id, 
+PyObject *H5UIget_info( hid_t loc_id,
 			const char *dset_name,
 			char *byteorder);
 
-int GetIndicesEx(PyObject *s, int length,
-		 int *start, int *stop, int *step, int *slicelength);
+hsize_t getIndicesExt(PyObject *s, hsize_t length,
+		      hsize_t *start, hsize_t *stop, hsize_t *step,
+		      hsize_t *slicelength);
 
 PyObject *get_attribute_string_sys( hid_t loc_id,
 				    const char *obj_name,
@@ -83,7 +84,7 @@ int is_complex(hid_t type_id);
 
 size_t get_complex_precision(hid_t type_id);
 
-H5T_order_t get_order(hid_t type_id);
+herr_t get_order(hid_t type_id, char *byteorder);
 
 hid_t create_native_complex64(const char *byteorder);
 
