@@ -2,10 +2,10 @@
 #
 #       License: BSD
 #       Created: October 10, 2002
-#       Author:  Francesc Alted - falted@pytables.org
+#       Author:  Francesc Altet - faltet@carabos.com
 #
-#       $Source: /cvsroot/pytables/pytables/tables/Array.py,v $
-#       $Id: Array.py,v 1.79.2.1 2004/11/11 10:47:22 falted Exp $
+#       $Source: /home/ivan/_/programari/pytables/svn/cvs/pytables/pytables/tables/Array.py,v $
+#       $Id: Array.py 1181 2005-09-12 17:49:13Z faltet $
 #
 ########################################################################
 
@@ -16,6 +16,7 @@ See Array class docstring for more info.
 Classes:
 
     Array
+    ImageArray
 
 Functions:
 
@@ -27,29 +28,36 @@ Misc variables:
 
 """
 
-__version__ = "$Revision: 1.79.2.1 $"
+import types, warnings, sys
+
+import numarray
+import numarray.strings as strings
+
+try:
+    import Numeric
+    Numeric_imported = True
+except ImportError:
+    Numeric_imported = False
+
+import tables.hdf5Extension as hdf5Extension
+from tables.utils import calcBufferSize, processRange, processRangeRead, \
+                         convToFlavor, CHUNKTIMES
+from tables.Leaf import Leaf, Filters
+
+
+__version__ = "$Revision: 1181 $"
+
 
 # default version for ARRAY objects
 #obversion = "1.0"    # initial version
 #obversion = "2.0"    # Added an optional EXTDIM attribute
-obversion = "2.1"    # Added support for complex datatypes
+#obversion = "2.1"    # Added support for complex datatypes
+#obversion = "2.2"    # This adds support for time datatypes.
+obversion = "2.3"    # This adds support for enumerated datatypes.
 
 
-import types, warnings, sys
-from Leaf import Leaf, Filters
-from utils import calcBufferSize, processRange, processRangeRead
-import hdf5Extension
-import numarray
-import numarray.strings as strings
-import numarray.records as records
 
-try:
-    import Numeric
-    Numeric_imported = 1
-except:
-    Numeric_imported = 0
-
-class Array(Leaf, hdf5Extension.Array, object):
+class Array(hdf5Extension.Array, Leaf):
     """Represent an homogeneous dataset in HDF5 file.
 
     It enables to create new datasets on-disk from Numeric, numarray,
@@ -74,7 +82,65 @@ class Array(Leaf, hdf5Extension.Array, object):
         nrow -- On iterators, this is the index of the current row.
 
     """
-    
+
+    # Class identifier.
+    _c_classId = 'ARRAY'
+
+
+    # <undo-redo support>
+    _c_canUndoCreate = True  # Can creation/copying be undone and redone?
+    _c_canUndoRemove = True  # Can removal be undone and redone?
+    _c_canUndoMove   = True  # Can movement/renaming be undone and redone?
+    # </undo-redo support>
+
+
+    def _g_calcBufferSize(self, expectedfsizeinKb):
+        """Compute and optimum buffer size.
+
+        The logic to do that is based purely in experiments playing
+        with different buffer sizes, chunksize and compression
+        flag. It is obvious that using big buffers optimize the I/O
+        speed. This might (should) be further optimized doing more
+        experiments.
+
+        This only is important for CArray and EArray classes, and it
+        is keeped here just because it's an easy way to share it.
+
+        """
+
+        # Increasing the bufmultfactor would enable a good compression
+        # ratio (up to an extend), but it would affect to reading
+        # performance. Be careful when touching this
+        # F. Altet 2004-11-10
+        #bufmultfactor = int(1000 * 5) # Conservative value
+        bufmultfactor = int(1000 * 10) # Medium value
+        #bufmultfactor = int(1000 * 20)  # Agressive value
+        #bufmultfactor = int(1000 * 50) # Very Aggresive value
+
+        if expectedfsizeinKb <= 100:
+            # Values for files less than 100 KB of size
+            buffersize = 5 * bufmultfactor
+        elif (expectedfsizeinKb > 100 and
+            expectedfsizeinKb <= 1000):
+            # Values for files less than 1 MB of size
+            buffersize = 10 * bufmultfactor
+        elif (expectedfsizeinKb > 1000 and
+              expectedfsizeinKb <= 20 * 1000):
+            # Values for sizes between 1 MB and 20 MB
+            buffersize = 20  * bufmultfactor
+        elif (expectedfsizeinKb > 20 * 1000 and
+              expectedfsizeinKb <= 200 * 1000):
+            # Values for sizes between 20 MB and 200 MB
+            buffersize = 40 * bufmultfactor
+        elif (expectedfsizeinKb > 200 * 1000 and
+              expectedfsizeinKb <= 2000 * 1000):
+            # Values for sizes between 200 MB and 2 GB
+            buffersize = 50 * bufmultfactor
+        else:  # Greater than 2 GB
+            buffersize = 60 * bufmultfactor
+
+        return buffersize
+
     def __init__(self, object = None, title = ""):
         """Create the instance Array.
 
@@ -107,20 +173,18 @@ class Array(Leaf, hdf5Extension.Array, object):
         self._v_version = obversion
         try:
             naarr, self.flavor = self._convertIntoNA(self.object)
-        except:
-            # Problems converting data. Close this node.
+        except:  #XXX
+            # Problems converting data. Close the node and re-raise exception.
             #print "Problems converting input object:", str(self.object)
             self.close(flush=0)
-            (type, value, traceback) = sys.exc_info()
-            # Re-raise the exception
-            raise type, value
+            raise
 
         if naarr.shape:
             self._v_expectedrows = naarr.shape[0]
         else:
             self._v_expectedrows = 1  # Scalar case
         if (isinstance(naarr, strings.CharArray)):
-            self.byteorder = "non-relevant" 
+            self.byteorder = "non-relevant"
         else:
             self.byteorder  = naarr._byteorder
 
@@ -131,12 +195,11 @@ class Array(Leaf, hdf5Extension.Array, object):
             if i>0:
                 self.rowsize *= i
             else:
-                raise ValueError, "An Array object cannot be empty."
+                raise ValueError, "An Array object cannot have zero-dimensions."
 
         # Compute the optimal chunksize
         (self._v_maxTuples, self._v_chunksize) = \
-                            calcBufferSize(self.rowsize, self._v_expectedrows,
-                                           self.filters.complevel)
+                            calcBufferSize(self.rowsize, self._v_expectedrows)
 
         self.shape = naarr.shape
         if naarr.shape:
@@ -145,13 +208,11 @@ class Array(Leaf, hdf5Extension.Array, object):
             self.nrows = 1    # Scalar case
         self.itemsize = naarr.itemsize()
         try:
-            self.type = self._createArray(naarr, self._v_new_title)
-        except:
-            # Problems creating the Array on disk. Close this node
+            self.type, self.stype = self._createArray(naarr, self._v_new_title)
+        except:  #XXX
+            # Problems creating the Array on disk. Close node and re-raise.
             self.close(flush=0)
-            (type, value, traceback) = sys.exc_info()
-            # Re-raise the exception
-            raise type, value
+            raise
 
     def _convertIntoNA(self, object):
         "Convert a generic object into a numarray object"
@@ -163,7 +224,7 @@ class Array(Leaf, hdf5Extension.Array, object):
         elif isinstance(arr, strings.CharArray):
             flavor = "CharArray"
             naarr = arr
-            self.byteorder = "non-relevant" 
+            self.byteorder = "non-relevant"
         elif (Numeric_imported and type(arr) == type(Numeric.array(1))):
             flavor = "Numeric"
             if arr.typecode() == "c":
@@ -196,10 +257,10 @@ class Array(Leaf, hdf5Extension.Array, object):
                     # the tolist() method, but this should be tested.
                     naarr = numarray.array(buffer(arr.copy()),
                                            type=arr.typecode(),
-                                           shape=arr.shape)                    
+                                           shape=arr.shape)
 
-        elif (isinstance(arr, types.TupleType) or
-              isinstance(arr, types.ListType)):
+        elif (isinstance(arr, tuple) or
+              isinstance(arr, list)):
             # Test if can convert to numarray object
             try:
                 naarr = numarray.array(arr)
@@ -208,25 +269,25 @@ class Array(Leaf, hdf5Extension.Array, object):
                 try:
                     naarr = strings.array(arr)
                 # If still doesn't, issues an error
-                except:
-                    raise ValueError, \
+                except:  #XXX
+                    raise TypeError, \
 """The object '%s' can't be converted into a numerical or character array.
-  Sorry, but this object is not supported.""" % (arr)
-            if isinstance(arr, types.TupleType):
+Sorry, but this object is not supported.""" % (arr)
+            if isinstance(arr, tuple):
                 flavor = "Tuple"
             else:
                 flavor = "List"
-        elif isinstance(arr, types.IntType):
+        elif isinstance(arr, int):
             naarr = numarray.array(arr)
             flavor = "Int"
-        elif isinstance(arr, types.FloatType):
+        elif isinstance(arr, float):
             naarr = numarray.array(arr)
             flavor = "Float"
-        elif isinstance(arr, types.StringType):
+        elif isinstance(arr, str):
             naarr = strings.array(arr)
             flavor = "String"
         else:
-            raise ValueError, \
+            raise TypeError, \
 """The object '%s' is not in the list of supported objects (NumArray, CharArray, Numeric, homogeneous list or homogeneous tuple, int, float or str). Sorry, but this object is not supported.""" % (arr)
 
         # We always want a contiguous buffer
@@ -239,9 +300,18 @@ class Array(Leaf, hdf5Extension.Array, object):
 
     def _open(self):
         """Get the metadata info for an array in file."""
-        (self.type, self.shape, self.itemsize, self.byteorder,
+
+        (self.type, self.stype, self.shape, self.itemsize, self.byteorder,
          self._v_chunksize) = self._openArray()
-        
+        # Compute the maxTuples
+        self._v_maxTuples = self._v_chunksize * CHUNKTIMES
+
+        # Get enumeration from disk.
+        if self.stype == 'Enum':
+            (self._enum, self.type) = self._loadEnum()
+        else:
+            self._enum = None
+
         # Compute the rowsize for each element
         self.rowsize = self.itemsize
         for i in xrange(len(self.shape)):
@@ -250,16 +320,27 @@ class Array(Leaf, hdf5Extension.Array, object):
         if self.shape:
             self.nrows = self.shape[0]
         else:
-            self.nrows = 1   # Scalar case
-        # Compute the optimal chunksize
-        (self._v_maxTuples, self._v_chunksize) = \
-                            calcBufferSize(self.rowsize, self.nrows,
-                                           self.filters.complevel)
-            
+            self.nrows = 1L   # Scalar case
+
+    def getEnum(self):
+        """
+        Get the enumerated type associated with this array.
+
+        If this array is of an enumerated type, the corresponding `Enum`
+        instance is returned.  If it is not of an enumerated type, a
+        ``TypeError`` is raised.
+        """
+
+        if self.stype != 'Enum':
+            raise TypeError("array ``%s`` is not of an enumerated type"
+                            % self._v_pathname)
+
+        return self._enum
+
 
     def iterrows(self, start=None, stop=None, step=None):
         """Iterate over all the rows or a range.
-        
+
         """
 
         try:
@@ -270,7 +351,7 @@ class Array(Leaf, hdf5Extension.Array, object):
             return ()
         self._initLoop()
         return self
-        
+
     def __iter__(self):
         """Iterate over all the rows."""
 
@@ -319,7 +400,8 @@ class Array(Leaf, hdf5Extension.Array, object):
                             self.listarr = swapaxes(self.listarr,
                                                     self.extdim, 0)
                     else:
-                        self.listarr.swapaxes(self.extdim, 0)
+                        self.listarr = numarray.swapaxes(self.listarr,
+			                                 self.extdim, 0)
                 self._row = -1
                 self._startb = self._stopb
             self._row += 1
@@ -332,21 +414,8 @@ class Array(Leaf, hdf5Extension.Array, object):
             else:
                 return self.listarr    # Scalar case
 
-    def __getitem__(self, keys):
-        """Returns an Array element, row or extended slice.
-
-        It takes different actions depending on the type of the "key"
-        parameter:
-
-        If "key" is an integer, the corresponding row is returned. If
-        "key" is a slice, the row slice determined by key is returned.
-
-        """
-
-#	The following restriction is unnecessary: it should work fine for keys == () ...
-#        if self.shape == ():
-#            # Scalar case
-#            raise IndexError, "You cannot read scalar Arrays through indexing. Try using the read() method better."
+    def _interpret_indexing(self, keys):
+        """Internal routine used by __getitem__ and __setitem__"""
 
         maxlen = len(self.shape)
         shape = (maxlen,)
@@ -354,7 +423,7 @@ class Array(Leaf, hdf5Extension.Array, object):
         stopl = numarray.array(None, shape=shape, type=numarray.Int64)
         stepl = numarray.array(None, shape=shape, type=numarray.Int64)
         stop_None = numarray.zeros(shape=shape, type=numarray.Int64)
-        if not isinstance(keys, types.TupleType):
+        if not isinstance(keys, tuple):
             keys = (keys,)
         nkeys = len(keys)
         dim = 0
@@ -362,9 +431,6 @@ class Array(Leaf, hdf5Extension.Array, object):
         # but this is a bit weird way to pass parameters anyway
         for key in keys:
             ellipsis = 0  # Sentinel
-            if dim >= maxlen:
-                raise IndexError, "Too many indices for object '%s'" % \
-                      self._v_pathname
             if isinstance(key, types.EllipsisType):
                 ellipsis = 1
                 for diml in xrange(dim, len(self.shape) - (nkeys - dim) + 1):
@@ -372,30 +438,31 @@ class Array(Leaf, hdf5Extension.Array, object):
                     stopl[dim] = self.shape[diml]
                     stepl[dim] = 1
                     dim += 1
-            elif isinstance(key, types.IntType):
+            elif dim >= maxlen:
+                raise IndexError, "Too many indices for object '%s'" % \
+                      self._v_pathname
+            elif type(key) in (int, long):
                 # Index out of range protection
                 if key >= self.shape[dim]:
                     raise IndexError, "Index out of range"
                 if key < 0:
                     # To support negative values (Fixes bug #968149)
                     key += self.shape[dim]
-#                 start, stop, step = processRangeRead(self.shape[dim],
-#                                                      key, key+1, 1)
                 start, stop, step = processRange(self.shape[dim],
                                                  key, key+1, 1)
                 stop_None[dim] = 1
-            elif isinstance(key, types.SliceType):
+            elif isinstance(key, slice):
                 start, stop, step = processRange(self.shape[dim],
                                                  key.start, key.stop, key.step)
             else:
-                raise ValueError, "Non-valid index or slice: %s" % \
+                raise TypeError, "Non-valid index or slice: %s" % \
                       key
             if not ellipsis:
                 startl[dim] = start
                 stopl[dim] = stop
                 stepl[dim] = step
                 dim += 1
-            
+
         # Complete the other dimensions, if needed
         if dim < len(self.shape):
             for diml in xrange(dim, len(self.shape)):
@@ -404,10 +471,33 @@ class Array(Leaf, hdf5Extension.Array, object):
                 stepl[dim] = 1
                 dim += 1
 
-        return self._readSlice(startl, stopl, stepl, stop_None)
+        # Compute the shape for the container properly. Fixes #1288792
+        shape = []
+        for dim in xrange(len(self.shape)):
+            new_dim = ((stopl[dim] - startl[dim] - 1) / stepl[dim]) + 1
+            if not (new_dim == 1 and stop_None[dim]):
+                # Append dimension
+                shape.append(new_dim)
+
+        return startl, stopl, stepl, shape
+
+    def __getitem__(self, keys):
+        """Returns an Array element, row or extended slice.
+
+        It takes different actions depending on the type of the "keys"
+        parameter:
+
+        If "keys" is an integer, the corresponding row is returned. If
+        "keys" is a slice, the row slice determined by key is returned.
+
+        """
+
+        startl, stopl, stepl, shape = self._interpret_indexing(keys)
+
+        return self._readSlice(startl, stopl, stepl, shape)
 
 
-    def __setitem__(self, key, value):
+    def __setitem__(self, keys, value):
         """Sets an Array element, row or extended slice.
 
         It takes different actions depending on the type of the "key"
@@ -415,7 +505,7 @@ class Array(Leaf, hdf5Extension.Array, object):
 
         If "key" is an integer, the corresponding row is assigned to
         value.
-        
+
         If "key" is a slice, the row slice determined by it is
         assigned to "value". If needed, this "value" is broadcasted to
         fit in the desired range. If the slice to be updated exceeds
@@ -427,113 +517,41 @@ class Array(Leaf, hdf5Extension.Array, object):
 
         """
 
-        if self.shape == ():
-            # Scalar case
-            raise IndexError, "You cannot read scalar Arrays through indexing. Try using the read() method better."
-
-        maxlen = len(self.shape)
-        shape = (maxlen,)
-        startl = numarray.array(None, shape=shape, type=numarray.Int64)
-        stopl = numarray.array(None, shape=shape, type=numarray.Int64)
-        stepl = numarray.array(None, shape=shape, type=numarray.Int64)
-        stop_None = numarray.zeros(shape=shape, type=numarray.Int64)
-        if not isinstance(key, types.TupleType):
-            keys = (key,)
-        else:
-            keys = key
-        nkeys = len(keys)
-        dim = 0
-        # Here is some problem when dealing with [...,...] params
-        # but this is a bit weird way to pass parameters anyway
-        for key in keys:
-            ellipsis = 0  # Sentinel
-            if dim >= maxlen:
-                raise IndexError, "Too many indices for object '%s'" % \
-                      self._v_pathname
-            if isinstance(key, types.EllipsisType):
-                ellipsis = 1
-                for diml in xrange(dim, len(self.shape) - (nkeys - dim) + 1):
-                    
-                    startl[dim] = 0
-                    stopl[dim] = self.shape[diml]
-                    stepl[dim] = 1
-                    dim += 1
-            elif isinstance(key, types.IntType):
-                # Index out of range protection
-                if key >= self.shape[dim]:
-                    raise IndexError, "Index out of range"
-                if key < 0:
-                    # To support negative values (Fixes bug #968149)
-                    key += self.shape[dim]
-                start, stop, step = processRange(self.shape[dim],
-                                                 key, key+1, 1)
-                stop_None[dim] = 1
-            elif isinstance(key, types.SliceType):
-                start, stop, step = processRange(self.shape[dim],
-                                                 key.start, key.stop, key.step)
-            else:
-                raise ValueError, "Non-valid index or slice: %s" % \
-                      key
-            if not ellipsis:
-                startl[dim] = start
-                stopl[dim] = stop
-                stepl[dim] = step
-                dim += 1
-            
-        # Complete the other dimensions, if needed
-        if dim < len(self.shape):
-            for diml in xrange(dim, len(self.shape)):
-                startl[dim] = 0
-                stopl[dim] = self.shape[diml]
-                stepl[dim] = 1
-                dim += 1
-
-        # Create an array compliant with the specified slice
+        startl, stopl, stepl, shape = self._interpret_indexing(keys)
         countl = ((stopl - startl - 1) / stepl) + 1
+        # Create an array compliant with the specified slice
         if str(self.type) == "CharType":
-            if shape <> []:
-                narr = strings.array(None, itemsize=self.itemsize,
-                                     shape=countl)
-            else:
-                narr = strings.array([""], itemsize=self.itemsize,
-                                     shape=countl)
+            narr = strings.array(None, itemsize=self.itemsize,
+            # Here shape=shape should be enough, but it makes some
+            # tests to fail. This should be analyzed more carefully.
+            # F. Altet 2005-09-12
+                                 shape=countl)
         else:
-            narr = numarray.array(None, shape=countl, type=self.type)
+            narr = numarray.array(None, shape=shape, type=self.type)
+
+        shape = numarray.array(shape, shape=len(shape), type=numarray.Int64)
 
         # Assign the value to it
         try:
-            narr[:] = value
-        except:
-            (type, value2, traceback) = sys.exc_info()
+            narr[...] = value
+        except:  #XXX
+            (typerr, value2, traceback) = sys.exc_info()
             raise ValueError, \
-"value parameter '%s' cannot be converted into an array object compliant with earray: \n'%r'\nThe error was: <%s>" % \
-        (value, self, value2)
+"""value parameter '%s' cannot be converted into an array object compliant with %s:
+'%r'
+The error was: <%s>""" % (value, self.__class__.__name__, self, value2)
 
         if narr.size():
             self._modify(startl, stepl, countl, narr)
 
     # Accessor for the _readArray method in superclass
-    def _readSlice(self, startl, stopl, stepl, stop_None):
+    def _readSlice(self, startl, stopl, stepl, shape):
 
         if self.extdim < 0:
             extdim = 0
         else:
             extdim = self.extdim
 
-        shape = []
-        for dim in xrange(len(self.shape)):
-            new_dim = ((stopl[dim] - startl[dim] - 1) / stepl[dim]) + 1
-            if not (new_dim == 1 and stop_None[dim]):
-                # Append dimension
-                shape.append(new_dim)
-
-        # The next solution isn't appropriate as a scalar array is
-        # meant as a way to return a Python value
-#         if shape == []:
-#             # In case of scalar elements, make them equivalent to 1dim 
-#             # This is much better for dealing with single-element objects
-#             shape = [1]  
-            
         if repr(self.type) == "CharType":
             # Workaround for numarray bug #997997
             if shape <> []:
@@ -548,12 +566,7 @@ class Array(Leaf, hdf5Extension.Array, object):
             arr._byteorder = self.byteorder
 
         # Protection against reading empty arrays
-        zerodim = 0
-        for i in xrange(len(shape)):
-            if shape[i] == 0:
-                zerodim = 1
-
-        if not zerodim:
+        if 0 not in shape:
             # Arrays that have non-zero dimensionality
             self._g_readSlice(startl, stopl, stepl, arr)
 
@@ -562,7 +575,7 @@ class Array(Leaf, hdf5Extension.Array, object):
 
         if self.flavor in ["NumArray", "CharArray"]:
             if arr.shape == ():  # Scalar case
-                return arr[()] 
+                return arr[()]
             else:             # No conversion needed
                 return arr
         # Fixes #968131
@@ -573,55 +586,8 @@ class Array(Leaf, hdf5Extension.Array, object):
 #         elif arr.shape == (1,):  # Scalar case
 #             return arr[0]  # return the value.
         else:
-            return self._convToFlavor(arr)
-        
-    def _convToFlavor(self, arr):
-        "Convert the numarray parameter to the correct flavor"
+            return convToFlavor(self, arr)
 
-        # Convert to Numeric, tuple or list if needed
-        if self.flavor == "Numeric":
-            if Numeric_imported:
-                # This works for both numeric and chararrays
-                # arr=Numeric.array(arr, typecode=arr.typecode())
-                # The next is 10 times faster (for tolist(),
-                # we should check for tostring()!)
-                if repr(self.type) == "CharType":
-                    arrstr = arr.tostring()
-                    arr=Numeric.reshape(Numeric.array(arrstr), arr.shape)
-                else:
-                    # tolist() method creates a list with a sane byteorder
-                    if arr.shape <> ():
-                        arr=Numeric.array(arr.tolist(), typecode=arr.typecode())
-                    else:
-                        # This works for rank-0 arrays
-                        # (but is slower for big arrays)
-                        arr=Numeric.array(arr[()], typecode=arr.typecode())
-                        
-            else:
-                # Warn the user
-                warnings.warn( \
-"""The object on-disk has Numeric flavor, but Numeric is not installed locally. Returning a numarray object instead!.""")
-        elif self.flavor == "Tuple":
-            #arr = tuple(arr.tolist())
-            arr = self.totuple(arr)
-        elif self.flavor == "List":
-            arr = arr.tolist()
-        elif self.flavor == "Int":
-            arr = int(arr)
-        elif self.flavor == "Float":
-            arr = float(arr)
-        elif self.flavor == "String":
-            arr = arr.tostring()
-
-        return arr
-    
-    def totuple(self, arr):
-        """Returns array as a (nested) tuple of elements."""
-        if len(arr._shape) == 1:
-            return tuple([ x for x in arr ])
-        else:
-            return tuple([ self.totuple(ni) for ni in arr ])
-        
     # Accessor for the _readArray method in superclass
     def read(self, start=None, stop=None, step=None):
         """Read the array from disk and return it as a self.flavor object."""
@@ -638,21 +604,23 @@ class Array(Leaf, hdf5Extension.Array, object):
             shape[extdim] = rowstoread
             shape = tuple(shape)
         if repr(self.type) == "CharType":
-            arr = strings.array(None, itemsize=self.itemsize, shape=shape)
+            #arr = strings.array(None, itemsize=self.itemsize, shape=shape)
+            # a workaround for a bug intoduced in numarray 1.3.1
+            if self.shape == ():
+                arr = strings.array(' '*self.itemsize, 
+		                    itemsize=self.itemsize, shape=shape)
+            else:
+                arr = strings.array(None, itemsize=self.itemsize, shape=shape)
         else:
             arr = numarray.array(None, type=self.type, shape=shape)
             # Set the same byteorder than on-disk
             arr._byteorder = self.byteorder
-        # Protection against reading empty arrays
-        zerodim = 0
-        for i in xrange(len(shape)):
-            if shape[i] == 0:
-                zerodim = 1
 
-        if not zerodim:
+        # Protection against reading empty arrays
+        if 0 not in shape:
             # Arrays that have non-zero dimensionality
-            self._readArray(start, stop, step, arr._data)
-            
+            self._readArray(start, stop, step, arr)
+
         if self.flavor in ["NumArray", "CharArray"]:
             # No conversion needed
             return arr
@@ -660,9 +628,9 @@ class Array(Leaf, hdf5Extension.Array, object):
         elif arr.shape == ():  # Scalar case
             return arr[()]  # return the value. Yes, this is a weird syntax :(
         else:
-            return self._convToFlavor(arr)
-        
-    def _g_copy(self, group, name, start, stop, step, title, filters):
+            return convToFlavor(self, arr)
+
+    def _g_copyWithStats(self, group, name, start, stop, step, title, filters):
         "Private part of Leaf.copy() for each kind of leaf"
         # Get the slice of the array
         # (non-buffered version)
@@ -671,8 +639,8 @@ class Array(Leaf, hdf5Extension.Array, object):
 	else:
 	    arr = self[()]
         # Build the new Array object
-        object = Array(arr, title=title)
-        setattr(group, name, object)
+        object = self._v_file.createArray(
+            group, name, arr, title=title, _log = False)
         nbytes = self.itemsize
         for i in self.shape:
             nbytes*=i
@@ -684,9 +652,25 @@ class Array(Leaf, hdf5Extension.Array, object):
 
         return """%s
   type = %r
+  stype = %r
   shape = %s
   itemsize = %s
   nrows = %s
   flavor = %r
-  byteorder = %r""" % (self, self.type, self.shape, self.itemsize,
+  byteorder = %r""" % (self, self.type, self.stype, self.shape, self.itemsize,
                        self.nrows, self.flavor, self.byteorder)
+
+
+
+class ImageArray(Array):
+
+    """
+    Array containing an image.
+
+    This class has no additional behaviour or functionality compared
+    to that of an ordinary array.  It simply enables the user to open
+    an ``IMAGE`` HDF5 node as a normal `Array` node in PyTables.
+    """
+
+    # Class identifier.
+    _c_classId = 'IMAGE'
