@@ -2,10 +2,10 @@
 #
 #       License: BSD
 #       Created: December 15, 2003
-#       Author:  Francesc Alted - falted@pytables.org
+#       Author:  Francesc Altet - faltet@carabos.com
 #
-#       $Source: /cvsroot/pytables/pytables/tables/EArray.py,v $
-#       $Id: EArray.py,v 1.21.2.4 2004/11/24 12:19:56 falted Exp $
+#       $Source: /home/ivan/_/programari/pytables/svn/cvs/pytables/pytables/tables/EArray.py,v $
+#       $Id: EArray.py 1153 2005-09-06 11:32:10Z faltet $
 #
 ########################################################################
 
@@ -27,27 +27,29 @@ Misc variables:
 
 """
 
-__version__ = "$Revision: 1.21.2.4 $"
-# default version for EARRAY objects
-#obversion = "1.0"    # initial version
-obversion = "1.1"    # support for complex datatypes
+import sys
 
-import types, warnings, sys
-from Array import Array
-from VLArray import Atom
-from utils import convertIntoNA, processRangeRead
-import hdf5Extension
 import numarray
-import numarray.strings as strings
 import numarray.records as records
 
-try:
-    import Numeric
-    Numeric_imported = 1
-except:
-    Numeric_imported = 0
+from tables.utils import convertIntoNA, processRangeRead
+from tables.Atom import Atom, EnumAtom
+from tables.Array import Array
 
-class EArray(Array, hdf5Extension.Array, object):
+
+
+__version__ = "$Revision: 1153 $"
+
+
+# default version for EARRAY objects
+#obversion = "1.0"    # initial version
+#obversion = "1.1"    # support for complex datatypes
+#obversion = "1.2"    # This adds support for time datatypes.
+obversion = "1.3"    # This adds support for enumerated datatypes.
+
+
+
+class EArray(Array):
     """Represent an homogeneous dataset in HDF5 file.
 
     It enables to create new datasets on-disk from Numeric and
@@ -63,8 +65,8 @@ class EArray(Array, hdf5Extension.Array, object):
         iterrows(start, stop, step)
 
       Specific of EArray:
-        append(object)
-        
+        append(sequence)
+
     Instance variables:
 
       Common to all Array's:
@@ -77,13 +79,24 @@ class EArray(Array, hdf5Extension.Array, object):
             dealed with.
 
       Specific of EArray:
-      
+
         extdim -- The enlargeable dimension.
         nrows -- The value of the enlargeable dimension.
-            
+
 
     """
-    
+
+    # Class identifier.
+    _c_classId = 'EARRAY'
+
+
+    # <undo-redo support>
+    _c_canUndoCreate = True  # Can creation/copying be undone and redone?
+    _c_canUndoRemove = True  # Can removal be undone and redone?
+    _c_canUndoMove   = True  # Can movement/renaming be undone and redone?
+    # </undo-redo support>
+
+
     def __init__(self, atom = None, title = "",
                  filters = None, expectedrows = 1000):
         """Create EArray instance.
@@ -94,7 +107,7 @@ class EArray(Array, hdf5Extension.Array, object):
             of the atomic objects to be saved. One of the shape
             dimensions must be 0. The dimension being 0 means that the
             resulting EArray object can be extended along it.
-        
+
         title -- Sets a TITLE attribute on the array entity.
 
         filters -- An instance of the Filters class that provides
@@ -121,53 +134,15 @@ class EArray(Array, hdf5Extension.Array, object):
             self.atom = atom
         else:
             self._v_new = 0
-            
+
     def _calcBufferSize(self, atom, extdim, expectedrows, compress):
         """Calculate the buffer size and the HDF5 chunk size.
-
-        The logic to do that is based purely in experiments playing
-        with different buffer sizes, chunksize and compression
-        flag. It is obvious that using big buffers optimize the I/O
-        speed. This might (should) be further optimized doing more
-        experiments.
-
         """
 
+        # The buffer size
         rowsize = atom.atomsize()
-        
-        # Increasing the bufmultfactor would enable a good compression
-        # ratio (up to an extend), but it would affect to reading
-        # performance. Be careful when touching this
-        # F. Altet 2004-11-10
-        #bufmultfactor = int(1000 * 5) # Conservative value
-        bufmultfactor = int(1000 * 10) # Medium value
-        #bufmultfactor = int(1000 * 20)  # Agressive value
-        #bufmultfactor = int(1000 * 50) # Very Aggresive value
-        
-        rowsizeinfile = rowsize
-        expectedfsizeinKb = (expectedrows * rowsizeinfile) / 1024
-
-        if expectedfsizeinKb <= 100:
-            # Values for files less than 100 KB of size
-            buffersize = 5 * bufmultfactor
-        elif (expectedfsizeinKb > 100 and
-            expectedfsizeinKb <= 1000):
-            # Values for files less than 1 MB of size
-            buffersize = 10 * bufmultfactor
-        elif (expectedfsizeinKb > 1000 and
-              expectedfsizeinKb <= 20 * 1000):
-            # Values for sizes between 1 MB and 20 MB
-            buffersize = 20  * bufmultfactor
-        elif (expectedfsizeinKb > 20 * 1000 and
-              expectedfsizeinKb <= 200 * 1000):
-            # Values for sizes between 20 MB and 200 MB
-            buffersize = 40 * bufmultfactor
-        elif (expectedfsizeinKb > 200 * 1000 and
-              expectedfsizeinKb <= 2000 * 1000):
-            # Values for sizes between 200 MB and 2 GB
-            buffersize = 50 * bufmultfactor
-        else:  # Greater than 2 GB
-            buffersize = 60 * bufmultfactor
+        expectedfsizeinKb = (expectedrows * rowsize) / 1024
+        buffersize = self._g_calcBufferSize(expectedfsizeinKb)
 
         # Max Tuples to fill the buffer
         maxTuples = buffersize // rowsize
@@ -206,21 +181,23 @@ class EArray(Array, hdf5Extension.Array, object):
 
     def _create(self):
         """Save a fresh array (i.e., not present on HDF5 file)."""
+
         global obversion
-        
-        assert isinstance(self.atom, Atom), "The object passed to the IndexArray constructor must be a descendent of the Atom class."
-        assert isinstance(self.atom.shape, types.TupleType), "The Atom shape has to be a tuple for IndexArrays, and you passed a '%s' object." % (self.atom.shape)
+
+        assert isinstance(self.atom, Atom), "The object passed to the EArray constructor must be a descendent of the Atom class."
+        assert isinstance(self.atom.shape, tuple), "The Atom shape has to be a tuple for EArrays, and you passed a '%s' object." % (self.atom.shape)
         # Version, type, shape, flavor, byteorder
         self._v_version = obversion
         self.type = self.atom.type
+        self.stype = self.atom.stype
         self.shape = self.atom.shape
-        self.flavor = self.atom.flavor        
+        self.flavor = self.atom.flavor
         if self.type == "CharType" or isinstance(self.type, records.Char):
             self.byteorder = "non-relevant"
         else:
             # Only support for creating objects in system byteorder
             self.byteorder  = sys.byteorder
-        
+
         # extdim computation
         zerodims = numarray.sum(numarray.array(self.shape) == 0)
         if zerodims > 0:
@@ -243,7 +220,24 @@ class EArray(Array, hdf5Extension.Array, object):
         #print "chunksizes-->", self._v_chunksize
         self.nrows = 0   # No rows initially
         self.itemsize = self.atom.itemsize
-        self._createEArray("EARRAY", self._v_new_title)
+        self._createEArray(self._v_new_title)
+
+
+    def getEnum(self):
+        """
+        Get the enumerated type associated with this array.
+
+        If this array is of an enumerated type, the corresponding `Enum`
+        instance is returned.  If it is not of an enumerated type, a
+        ``TypeError`` is raised.
+        """
+
+        if self.atom.stype != 'Enum':
+            raise TypeError("array ``%s`` is not of an enumerated type"
+                            % self._v_pathname)
+
+        return self.atom.enum
+
 
     def _checkTypeShape(self, naarr):
         "Test that naarr parameter is shape and type compliant"
@@ -264,21 +258,31 @@ class EArray(Array, hdf5Extension.Array, object):
 (naarr, self.type)
 
         # The arrays conforms self expandibility?
-        assert len(self.shape) == len(naarr.shape), \
-"Sorry, the ranks of the EArray %r (%d) and object to be appended (%d) differ." % (self._v_pathname, len(self.shape), len(naarr.shape))
-        for i in range(len(self.shape)):
-            if i <> self.extdim:
-                assert self.shape[i] == naarr.shape[i], \
-"Sorry, shapes of EArray '%r' and object differ in non-enlargeable dimension (%d) " % (self._v_pathname, i) 
+        myshlen = len(self.shape)
+        nashlen = len(naarr.shape)
+        if myshlen != nashlen:
+            raise ValueError("""\
+the ranks of the appended object (%d) and the ``%s`` EArray (%d) differ"""
+                             % (nashlen, self._v_pathname, myshlen))
+        for i in range(myshlen):
+            if i != self.extdim and self.shape[i] != naarr.shape[i]:
+                raise ValueError("""\
+the shapes of the appended object and the ``%s`` EArray \
+differ in non-enlargeable dimension %d""" % (self._v_pathname, i))
         # Ok. all conditions are met. Return the numarray object
         return naarr
-            
-    def append(self, object):
-        """Append the object to this (enlargeable) object"""
-        assert self._v_file.mode <> "r", "Attempt to write over a file opened in read-only mode"
 
-        # Convert the object into a numarray object
-        naarr = convertIntoNA(object, self.atom)
+    def append(self, sequence):
+        """Append the sequence to this (enlargeable) object"""
+
+        if self._v_file.mode == 'r':
+            raise IOError("attempt to write over a file opened in read-only mode")
+
+        # The sequence needs to be copied to make the operation safe
+        # to in-place conversion.
+        copy = self.stype in ['Time64']
+        # Convert the sequence into a numarray object
+        naarr = convertIntoNA(sequence, self.atom, copy)
         # Check if it is correct type and shape
         naarr = self._checkTypeShape(naarr)
         self._append(naarr)
@@ -286,27 +290,34 @@ class EArray(Array, hdf5Extension.Array, object):
     def truncate(self, size):
         "Truncate the extendable dimension to at most size rows"
 
-        #assert size >= 0, "Size should be 0 or a positive value"
-        assert size > 0, "Size should be an integer greater than 0"
-        return self._truncateArray(size)
+        if size <= 0:
+            raise ValueError("`size` must be greater than 0")
+        self._truncateArray(size)
 
     def _open(self):
         """Get the metadata info for an array in file."""
-        (self.type, self.shape, self.itemsize, self.byteorder,
+
+        (type_, self.stype, self.shape, self.itemsize, self.byteorder,
          self._v_chunksize) = self._openArray()
+        stype = self.stype
+        flavor = self.flavor
         #print "chunksizes-->", self._v_chunksize
         # Post-condition
         assert self.extdim >= 0, "extdim < 0: this should never happen!"
         # Compute the real shape for atom:
         shape = list(self.shape)
         shape[self.extdim] = 0
-        if self.type == "CharType" or isinstance(self.type, records.Char):
+        if type_ == "CharType" or isinstance(type_, records.Char):
             # Add the length of the array at the end of the shape for atom
             shape.append(self.itemsize)
         shape = tuple(shape)
-        # Create the atom instance
-        self.atom = Atom(dtype=self.type, shape=shape,
-                         flavor=self.attrs.FLAVOR)
+        # Create the atom instance and set definitive type
+        if stype == 'Enum':
+            (enum, type_) = self._loadEnum()
+            self.atom = EnumAtom(enum, type_, shape, flavor)
+        else:
+            self.atom = Atom(stype, shape, flavor)
+        self.type = type_
         # Compute the rowsize for each element
         self.rowsize = self.atom.atomsize()
         # nrows in this instance
@@ -321,14 +332,12 @@ class EArray(Array, hdf5Extension.Array, object):
         self._v_maxTuples = self._v_buffersize // chunksize
         #print "maxTuples-->", self._v_maxTuples
 
-    def _g_copy(self, group, name, start, stop, step, title, filters):
+    def _g_copyWithStats(self, group, name, start, stop, step, title, filters):
         "Private part of Leaf.copy() for each kind of leaf"
         # Build the new EArray object
-        object = EArray(atom=self.atom,
-                        title=title,
-                        filters=filters,
-                        expectedrows=self.nrows)
-        setattr(group, name, object)
+        object = self._v_file.createEArray(
+            group, name, atom=self.atom, title=title, filters=filters,
+            expectedrows=self.nrows, _log = False)
         # Now, fill the new earray with values from source
         nrowsinbuf = self._v_maxTuples
         # The slices parameter for self.__getitem__
@@ -358,11 +367,9 @@ class EArray(Array, hdf5Extension.Array, object):
         """This provides more metainfo in addition to standard __str__"""
 
         return """%s
-  type = %r
-  shape = %s
-  itemsize = %s
+  atom = %r
   nrows = %s
   extdim = %r
   flavor = %r
-  byteorder = %r""" % (self, self.type, self.shape, self.itemsize, self.nrows,
-                       self.extdim, self.flavor, self.byteorder)
+  byteorder = %r""" % (self, self.atom, self.nrows, self.extdim, self.flavor,
+                       self.byteorder)
