@@ -2,10 +2,10 @@
 #
 #       License: BSD
 #       Created: June 08, 2004
-#       Author:  Francesc Alted - falted@pytables.org
+#       Author:  Francesc Altet - faltet@carabos.com
 #
-#       $Source: /cvsroot/pytables/pytables/tables/Index.py,v $
-#       $Id: Index.py,v 1.25.2.1 2004/12/02 11:52:33 falted Exp $
+#       $Source: /home/ivan/_/programari/pytables/svn/cvs/pytables/pytables/tables/Index.py,v $
+#       $Id: Index.py 1015 2005-06-17 17:55:14Z faltet $
 #
 ########################################################################
 
@@ -27,22 +27,26 @@ Misc variables:
 
 """
 
-__version__ = "$Revision: 1.25.2.1 $"
-# default version for INDEX objects
-obversion = "1.0"    # initial version
-
-import cPickle
-import types, warnings, sys
-from IndexArray import IndexArray
-from VLArray import Atom
-from Leaf import Filters
-from AttributeSet import AttributeSet
-import hdf5Extension
-#from hdf5Extension import PyNextAfter, PyNextAfterF
-import numarray
-import time
+import warnings
 import math
-import struct # we use this to define testNaN
+import cPickle
+
+import numarray
+
+import tables.hdf5Extension as hdf5Extension
+import tables.utilsExtension as utilsExtension
+from tables.AttributeSet import AttributeSet
+from tables.Atom import Atom
+from tables.Leaf import Filters
+from tables.IndexArray import IndexArray
+from tables.Group import Group
+from tables.utils import joinPath
+
+__version__ = "$Revision: 1015 $"
+
+# default version for INDEX objects
+#obversion = "1.0"    # initial version
+obversion = "2.0"    # indexes moved to a hidden directory
 
 # Python implementations of NextAfter and NextAfterF
 #
@@ -59,8 +63,8 @@ import struct # we use this to define testNaN
 epsilon  = math.ldexp(1.0, -53) # smallest double such that 0.5+epsilon != 0.5
 epsilonF = math.ldexp(1.0, -24) # smallest float such that 0.5+epsilonF != 0.5
 
-maxFloat=float(2**1024 - 2**971)  # From the IEEE 754 standard
-maxFloatF=float(2**128 - 2**104)  # From the IEEE 754 standard
+maxFloat = float(2**1024 - 2**971)  # From the IEEE 754 standard
+maxFloatF = float(2**128 - 2**104)  # From the IEEE 754 standard
 
 minFloat  = math.ldexp(1.0, -1022) # min positive normalized double
 minFloatF = math.ldexp(1.0, -126)  # min positive normalized float
@@ -70,7 +74,7 @@ smallEpsilonF = math.ldexp(1.0, -149)  # smallest increment for floats < minFloa
 
 infinity = math.ldexp(1.0, 1023) * 2
 infinityF = math.ldexp(1.0, 128)
-#Finf=float("inf")  # Infinite in the IEEE 754 standard (not avail in Win)
+#Finf = float("inf")  # Infinite in the IEEE 754 standard (not avail in Win)
 
 # A portable representation of NaN
 # if sys.byteorder == "little":
@@ -78,10 +82,10 @@ infinityF = math.ldexp(1.0, 128)
 # elif sys.byteorder == "big":
 #     testNaN = struct.unpack("d", '\x7f\xf0\x00\x00\x00\x00\x00\x01')[0]
 # else:
-#     raise RuntimeError, "Byteorder '%s' not supported!" % sys.byteorder
+#     raise ValueError, "Byteorder '%s' not supported!" % sys.byteorder
 # This one seems better
 testNaN = infinity - infinity
-    
+
 # Utility functions
 def infType(type, itemsize, sign=0):
     """Return a superior limit for maximum representable data type"""
@@ -104,7 +108,6 @@ def IsNaN(x):
 
 def PyNextAfter(x, y):
     """returns the next float after x in the direction of y if possible, else returns x"""
-    
     # if x or y is Nan, we don't do much
     if IsNaN(x) or IsNaN(y):
         return x
@@ -128,7 +131,7 @@ def PyNextAfter(x, y):
     # break x down into a mantissa and exponent
     m, e = math.frexp(x)
 
-    # all the special cases have been handled        
+    # all the special cases have been handled
     if y > x:
         m += epsilon
     else:
@@ -138,7 +141,7 @@ def PyNextAfter(x, y):
 
 def PyNextAfterF(x, y):
     """returns the next IEEE single after x in the direction of y if possible, else returns x"""
-    
+
     # if x or y is Nan, we don't do much
     if IsNaN(x) or IsNaN(y):
         return x
@@ -163,7 +166,6 @@ def PyNextAfterF(x, y):
             extra = x % -smallEpsilonF
         else:
             extra = 0.0
-            
         if y > x:
             return x - extra + smallEpsilonF
         else:
@@ -221,7 +223,7 @@ def CharTypeNextAfter(x, direction, itemsize):
             i += 1
     xlist.reverse()
     return "".join(xlist)
-        
+
 
 def nextafter(x, direction, type, itemsize):
     "Return the next representable neighbor of x in the appropriate direction."
@@ -267,7 +269,7 @@ class IndexProps(object):
         """Create a new IndexProps instance
 
         Parameters:
-        
+
         auto -- whether an existing index should be reindexed after a
             Table append operation. Defaults is reindexing.
         reindex -- whether the table fields are to be re-indexed
@@ -291,7 +293,7 @@ class IndexProps(object):
         elif isinstance(filters, Filters):
             self.filters = filters
         else:
-            raise ValueError, \
+            raise TypeError, \
 "If you pass a filters parameter, it should be a Filters instance."
 
     def __repr__(self):
@@ -302,14 +304,16 @@ class IndexProps(object):
         descr += ", reindex=%s" % (self.reindex)
         descr += ", filters=%s" % (self.filters)
         return descr+")"
-    
+
     def __str__(self):
         """The string reprsentation choosed for this object
         """
-        
+
         return repr(self)
 
-class Index(hdf5Extension.Group, hdf5Extension.Index, object):
+class Index(hdf5Extension.Index, Group):
+    _c_classId = 'CINDEX'
+
     """Represent the index (sorted and reverse index) dataset in HDF5 file.
 
     It enables to create indexes of Columns of Table objects.
@@ -322,7 +326,7 @@ class Index(hdf5Extension.Group, hdf5Extension.Index, object):
         search(start, stop, step, where)
         getCoords(startCoords, maxCoords)
         append(object)
-        
+
     Instance variables:
 
         column -- The column object this index belongs to
@@ -340,9 +344,9 @@ class Index(hdf5Extension.Group, hdf5Extension.Index, object):
 
     """
 
-    def __init__(self, atom = None, where= None, name = None,
+    def __init__(self, atom = None, column = None, name = None,
                  title = "", filters = None, expectedrows = 1000,
-                 testmode = 0):
+                 testmode = 0, new = True):
         """Create an Index instance.
 
         Keyword arguments:
@@ -351,10 +355,10 @@ class Index(hdf5Extension.Group, hdf5Extension.Index, object):
             of the atomic objects to be saved. Only scalar atoms are
             supported.
 
+        column -- The column object to be indexed
+
         name -- The name for this Index object.
 
-        where -- The indexed column who this instance pertains.
-        
         title -- Sets a TITLE attribute of the Index entity.
 
         filters -- An instance of the Filters class that provides
@@ -369,146 +373,83 @@ class Index(hdf5Extension.Group, hdf5Extension.Index, object):
             value is 1000 slices.
 
         """
-        self.name = name
-        self._v_hdf5name = name
-        self._v_new_title = title
-        self._v_new_filters = filters
-        self._v_expectedrows = expectedrows
-        self.testmode = testmode
-        self.column = where
-        self._v_parent = where.table._v_parent  #Parent of table in object tree
-        # Check whether we have to create a new object or read their contents
-        # from disk
-        if atom is not None:
-            self._v_new = 1
-            self.atom = atom
-            self._create()
-        else:
-            self._v_new = 0
-            self._open()
+        super(Index, self).__init__(title, new, filters)
 
-    def _g_join(self, name):
-        """Helper method to correctly concatenate a name child object
-        with the pathname of this group."""
+        mydict = self.__dict__
 
-        pathname = self._v_parent._g_join(self.name)
-        if name == "/":
-            # This case can happen when doing copies
-            return pathname
-        if pathname == "/":
-            return "/" + name
-        else:
-            return pathname + "/" + name
+        # The next are needed by the Group protocol
+        mydict['name'] = name
+        mydict['_v_expectedrows'] = expectedrows
+        mydict['_v_version'] = obversion
 
-    def _addAttrs(self, object, klassname):
-        """ Add attributes to object """
-        object.__dict__["_v_attrs"] = AttributeSet(object)
-        object._v_attrs._g_setAttr('TITLE',  object._v_new_title)
-        object._v_attrs._g_setAttr('CLASS', klassname)
-        object._v_attrs._g_setAttr('VERSION', object._v_version)
-        # Set the filters object
-        if object._v_new_filters is None:
+        # Collect info in case the index is to be created
+        if new:
+            mydict['atom'] = atom
+            mydict['column'] = column
+
+        mydict['testmode'] = testmode
+
+    def _g_create(self):
+        """Call the createGroup method in extension to create the
+        group on disk. Also set attributes for this group. Finally,
+        add the index information as IndexArrays."""
+
+        global obversion
+
+        mydict = self.__dict__
+        # Create the Index Group
+        mydict['_v_objectID'] =  self._g_createGroup()
+
+        # Set the filters for this object (they are *not* inherited)
+        if self._v_new_filters is None:
             # If not filters has been passed in the constructor,
             # set a sensible default, using zlib compression and shuffling
             filters = Filters(complevel = 1, complib = "zlib",
                               shuffle = 1, fletcher32 = 0)
-            # Now, the filters are not inheritated. 2004-08-04
-            #filters = object._v_parent._v_filters
         else:
-            filters = object._v_new_filters
-        filtersPickled = cPickle.dumps(filters, 0)
-        object._v_attrs._g_setAttr('FILTERS', filtersPickled)
-        # Add these attributes to the dictionary
-        attrlist = ['TITLE','CLASS','VERSION','FILTERS']
-        object._v_attrs._v_attrnames.extend(attrlist)
-        object._v_attrs._v_attrnamessys.extend(attrlist)
-        # Sort them
-        object._v_attrs._v_attrnames.sort()
-        object._v_attrs._v_attrnamessys.sort()
-        return filters
-            
-    def _create(self):
-        """Save a fresh array (i.e., not present on HDF5 file)."""
-        global obversion
+            filters = self._v_new_filters
 
-        assert isinstance(self.atom, Atom), "The object passed to the IndexArray constructor must be a descendent of the Atom class."
-        assert self.atom.shape == 1, "Only scalar columns can be indexed."
-        # Version, type, shape, byteorder
-        self._v_version = obversion
-        self.type = self.atom.type
-        self.title = self._v_new_title
-        # Create the Index Group
-        self._g_new(self._v_parent, self.name)
-        self._v_objectID = self._g_createGroup()
-        self.filters = self._addAttrs(self, "INDEX")
         # Create the IndexArray for sorted values
-        object = IndexArray(self, self.atom, "Sorted Values",
-                            self.filters, self._v_expectedrows,
-                            self.testmode)
-        object.name = object._v_name = object._v_hdf5name = "sortedArray"
-        object._g_new(self, object.name)
-        object.filters = self.filters
-        object._create()
-        object._v_parent = self
-        object._v_file = self._v_parent._v_file  
-        self._addAttrs(object, "IndexArray")
-        self.sorted = object
-        self.type = object.type
-        self.shape = object.shape
-        self.itemsize = object.itemsize
-        self.chunksize = object.chunksize
-        self.byteorder = object.byteorder
+        sorted = IndexArray(self, self.atom,
+                            "Sorted Values", filters,
+                            self._v_expectedrows, self.testmode)
+        # Register the sorted values array
+        setattr(self, "sorted", sorted)
+
         # Create the IndexArray for index values
-        object = IndexArray(self, Atom("Int32", shape=1), "Reverse Indices",
-                            self.filters, self._v_expectedrows,
-                            self.testmode)
-        object.name = object._v_name = object._v_hdf5name = "revIndexArray"
-        object._g_new(self, object.name)
-        object.filters = self.filters
-        object._create()
-        object._v_parent = self
-        object._v_file = self._v_parent._v_file  
-        self.nrows = object.nrows
-        self.nelemslice = object.nelemslice
-        self.nelements = self.nrows * self.nelemslice
-        self._addAttrs(object, "IndexArray")
-        self.indices = object
-            
-    def _open(self):
-        """Get the metadata info for an array in file."""
-        self._g_new(self._v_parent, self.name)
-        self._v_objectID = self._g_openIndex()
-        self.__dict__["_v_attrs"] = AttributeSet(self)
-        # Get the title, filters attributes for this index
-        self.title = self._v_attrs._g_getAttr("TITLE")
-        # Open the IndexArray for sorted values
-        object = IndexArray(parent=self)
-        object._v_parent = self
-        object._v_file = self._v_parent._v_file  
-        object.name = object._v_name = object._v_hdf5name = "sortedArray"
-        object._g_new(self, object._v_hdf5name)
-        object.filters = object._g_getFilters()
-        object._open()
-        self.sorted = object
-        self.type = object.type
-        self.shape = object.shape
-        self.itemsize = object.itemsize
-        self.chunksize = object.chunksize
-        self.byteorder = object.byteorder
-        # Open the IndexArray for reverse Index values
-        object = IndexArray(parent=self)
-        object._v_parent = self
-        object._v_file = self._v_parent._v_file
-        object.name = object._v_name = object._v_hdf5name = "revIndexArray"
-        object._g_new(self, object._v_hdf5name)
-        object.filters = object._g_getFilters()
-        object._open()
-        # these attrs should be the same for both sorted and indexed 
-        self.nrows = object.nrows
-        self.nelemslice = object.nelemslice
-        self.nelements = self.nrows * self.nelemslice
-        self.filters = object.filters
-        self.indices = object
+        indices = IndexArray(self, Atom("Int32", shape=1),
+                             "Reverse Indices", filters,
+                             self._v_expectedrows, self.testmode)
+        # Register the indices array
+        setattr(self, "indices", indices)
+
+        # get the remaining variables
+        self._getCompleteObject()
+
+    def _g_afterOpen(self):
+        """Tidy object tree after its creation."""
+
+        # Call the post-open methods in sorted and indices
+        # This has to executed *before* _getCompleteObject
+        self.sorted._g_afterOpen()
+        self.indices._g_afterOpen()
+
+        # get the remaining variables
+        self._getCompleteObject()
+
+    def _getCompleteObject(self):
+        "Get the remaining variables for a complete Index object"
+
+        mydict = self.__dict__
+        mydict['type'] = self.sorted.type
+        mydict['itemsize'] = self.sorted.itemsize
+        mydict['chunksize'] = self.sorted.chunksize
+        mydict['byteorder'] = self.sorted.byteorder
+        mydict['nrows'] = self.sorted.nrows
+        mydict['nelemslice'] = self.sorted.nelemslice
+        mydict['nelements'] = self.nrows * self.nelemslice
+        mydict['shape'] = (self.nrows, self.nelemslice)
+        mydict['filters'] = self.sorted.filters
 
     def append(self, arr):
         """Append the object to this (enlargeable) object"""
@@ -534,15 +475,17 @@ class Index(hdf5Extension.Group, hdf5Extension.Index, object):
             self.indices.append(numarray.array(s, type="Int32"))
             self.sorted.append(arr[s])
         # Update nrows after a successful append
-        self.nrows = self.sorted.nrows
-        self.nelements = self.nrows * self.nelemslice
-        self.shape = (self.nrows, self.nelemslice)
-        
+        mydict = self.__dict__
+        mydict['nrows'] = self.sorted.nrows
+        mydict['nelements'] = self.nrows * self.nelemslice
+        mydict['shape'] = (self.nrows, self.nelemslice)
+
     def search(self, item):
         """Do a binary search in this index for an item"""
         #t1=time.time()
         ntotaliter = 0; tlen = 0
-        self.starts = []; self.lengths = []
+        mydict = self.__dict__
+        mydict['starts'] = []; mydict['lengths'] = []
         #self.irow = 0; self.len1 = 0; self.len2 = 0;  # useful for getCoords()
         self.sorted._initSortedSlice(self.chunksize)
         # Do the lookup for values fullfilling the conditions
@@ -642,47 +585,52 @@ class Index(hdf5Extension.Group, hdf5Extension.Index, object):
     def getLookupRange(self, column):
         #import time
         table = column.table
-        # Get the coordenates for those values
+        # Get the coordinates for those values
         ilimit = table.opsValues
         ctype = column.type
-        itemsize = table.colitemsizes[column.name]
+        sctype = str(ctype)
+        itemsize = table.colitemsizes[column.pathname]
+
         # Check that limits are compatible with type
         for limit in ilimit:
             # Check for strings
-            if str(ctype) == "CharType":
-                assert type(limit) == types.StringType, \
-"Bounds (or range limits) for strings columns can only be strings."
+            if sctype == "CharType":
+                if type(limit) is not str:
+                    raise TypeError("""\
+Bounds (or range limits) for string columns can only be strings.""")
+                else:
+                    continue
+
+            nactype = numarray.typeDict[sctype]
+
             # Check for booleans
-            elif isinstance(numarray.typeDict[str(ctype)],
-                          numarray.BooleanType):
-                assert (type(limit) == types.IntType or
-                        type(limit) == types.BooleanType), \
-"Bounds (or range limits) for bool columns can only be ints or booleans."
+            if isinstance(nactype, numarray.BooleanType):
+                if type(limit) not in (int, long, bool):
+                    raise TypeError("""\
+Bounds (or range limits) for bool columns can only be ints or booleans.""")
             # Check for ints
-            elif isinstance(numarray.typeDict[str(ctype)],
-                          numarray.IntegralType):
-                assert (type(limit) == types.IntType or
-                        type(limit) == types.FloatType), \
-"Bounds (or range limits) for integer columns can only be ints or floats."
+            elif isinstance(nactype, numarray.IntegralType):
+                if type(limit) not in (int, long, float):
+                    raise TypeError("""\
+Bounds (or range limits) for integer columns can only be ints or floats.""")
             # Check for floats
-            elif isinstance(numarray.typeDict[str(ctype)],
-                          numarray.FloatingType):
-                assert (type(limit) == types.IntType or
-                        type(limit) == types.FloatType), \
-"Bounds (or range limits) for float columns can only be ints or floats."
+            elif isinstance(nactype, numarray.FloatingType):
+                if type(limit) not in (int, long, float):
+                    raise TypeError("""\
+Bounds (or range limits) for float columns can only be ints or floats.""")
             else:
-                raise ValueError, \
-"Bounds (or range limits) can only be strings, bools, ints or floats."
-            
+                raise TypeError("""
+Bounds (or range limits) can only be strings, bools, ints or floats.""")
+
         # Boolean types are a special case for searching
-        if str(ctype) == "Bool":
+        if sctype == "Bool":
             if len(table.ops) == 1 and table.ops[0] == 5: # __eq__
                 item = (ilimit[0], ilimit[0])
                 ncoords = self.search(item)
                 return ncoords
             else:
                 raise NotImplementedError, \
-                      "Only equality operator is suported for boolean columns."
+                      "Only equality operator is supported for boolean columns."
         # Other types are treated here
         if len(ilimit) == 1:
             ilimit = ilimit[0]
@@ -706,8 +654,10 @@ class Index(hdf5Extension.Group, hdf5Extension.Index, object):
                 raise NotImplementedError, "'!=' or '<>' not supported yet"
         elif len(ilimit) == 2:
             item1, item2 = ilimit
-            assert item1 <= item2, \
-"On 'val1 <{=} col <{=} val2' selections, val1 must be less or equal than val2"
+            if item1 > item2:
+                raise ValueError("""\
+On 'val1 <{=} col <{=} val2' selections, \
+val1 must be less or equal than val2""")
             op1, op2 = table.ops
             if op1 == 3 and op2 == 1:  # item1 < col < item2
                 item = (nextafter(item1, +1, ctype, itemsize),
@@ -719,70 +669,64 @@ class Index(hdf5Extension.Group, hdf5Extension.Index, object):
             elif op1 == 4 and op2 == 2:  # item1 <= col <= item2
                 item = (item1, item2)
             else:
-                raise SyntaxError, \
+                raise ValueError, \
 "Combination of operators not supported. Use val1 <{=} col <{=} val2"
-                      
+
         #t1=time.time()
         ncoords = self.search(item)
         #print "time reading indices:", time.time()-t1
         return ncoords
 
-    def _g_remove(self):
+    def _remove(self):
         """Remove this Index object"""
 
-        if hdf5Extension.whichLibVersion("hdf5")[1] == "1.6.3":
-            warnings.warn( \
-"""\nYou are using HDF5 version 1.6.3. It turns out that this precise
+        if utilsExtension.whichLibVersion("hdf5")[1] == "1.6.3":
+            warnings.warn("""\
+You are using HDF5 version 1.6.3. It turns out that this precise
 version has a bug that causes a seg fault when deleting a chunked
 dataset. If you are getting such a seg fault immediately after this
-message, please, get a patched version of HDF5 1.6.3.""", UserWarning)
+message, please, get a patched version of HDF5 1.6.3, or, better,
+get HDF5 1.6.4.""")
 
         # Delete the associated IndexArrays
-        #self.sorted._close()
-        #self.indices._close()
-        self.sorted.flush()
-        self.indices.flush()
-        # The next cannot be done because sortedArray and revIndexArray are
-        # not regular Leafs
-        #self.sorted.remove()
-        #self.indices.remove()
-        self._g_deleteLeaf(self.sorted.name)
-        self._g_deleteLeaf(self.indices.name)
-        self.sorted = None
-        self.indices = None
-        # close the Index Group
-        self._f_close()
-        # delete it (this is defined in hdf5Extension)
-        self._g_deleteGroup()
+        self.sorted.remove()
+        self.indices.remove()
+        # delete some references
+        mydict = self.__dict__
+        mydict['indices'] = None
+        mydict['sorted'] = None
+        # delete self
+        self._f_remove()
 
     def _f_close(self):
-        # close the indices
-        if self.sorted:  # that might be already removed
-            self.sorted._close()
-        if self.indices: # that might be already removed
-            self.indices._close()
+        # Check that the index is not already closed
+        if self._v_file == None:
+            return
+        # close the indices (might be already removed by self._g_remove!)
+        if hasattr(self,"sorted") and self.sorted <> None:
+            self.sorted.close()
+        if hasattr(self,"indices") and self.indices <> None:
+            self.indices.close()
         # delete some references
-        self.atom=None
-        self.column=None
-        self.filters=None
-        self.indices = None
-        self.sorted = None
-        self._v_attrs = None
-        self._v_parent = None
-        # Close this group
-        self._g_closeGroup()
-        #self.__dict__.clear()
+        mydict = self.__dict__
+        mydict['atom'] = None
+        mydict['column'] = None
+        mydict['filters'] = None
+        mydict['indices'] = None
+        mydict['sorted'] = None
+        #mydict['_v_attrs'] = None  # This is deleted in Group._f_close()
+        # Call the superclass close method
+        Group._f_close(self)
 
     def __str__(self):
         """This provides a more compact representation than __repr__"""
         return "Index(%s, shape=%s, chunksize=%s)" % \
                (self.nelements, self.shape, self.chunksize)
-    
+
     def __repr__(self):
         """This provides more metainfo than standard __repr__"""
 
         cpathname = self.column.table._v_pathname + ".cols." + self.column.name
-        pathname = self._v_parent._g_join(self.name)
         dirty = self.column.dirty
         return """%s (Index for column %s)
   type := %r
@@ -793,7 +737,7 @@ message, please, get a patched version of HDF5 1.6.3.""", UserWarning)
   filters := %s
   dirty := %s
   sorted := %s
-  indices := %s""" % (pathname, cpathname,
+  indices := %s""" % (self._v_pathname, cpathname,
                      self.type, self.nelements, self.shape,
-                     self.chunksize, self.byteorder,
+                     self.sorted.chunksize, self.sorted.byteorder,
                      self.filters, dirty, self.sorted, self.indices)
