@@ -5,26 +5,23 @@
 #
 #       License: BSD
 #       Created: November 12, 2003
-#       Author:  Francesc Alted - falted@pytables.org
+#       Author:  Francesc Altet - faltet@carabos.com
 #
-#       $Source: /cvsroot/pytables/pytables/tables/VLArray.py,v $
-#       $Id: VLArray.py,v 1.36.2.1 2004/11/25 11:50:37 falted Exp $
+#       $Source: /home/ivan/_/programari/pytables/svn/cvs/pytables/pytables/tables/VLArray.py,v $
+#       $Id: VLArray.py 1153 2005-09-06 11:32:10Z faltet $
 #
 ########################################################################
 
-"""Here is defined the VLArray class and declarative classes for its components
+"""Here is defined the VLArray class
 
-See VLArray class docstring and *Atom docstrings for more info.
+See VLArray class docstring for more info.
 
 Classes:
 
-    Atom, ObjectAtom, VLStringAtom, StringAtom, BoolAtom,
-    IntAtom, Int8Atom, UInt8Atom, Int16Atom, UInt16Atom,
     VLArray
 
 Functions:
 
-   checkflavor
 
 Misc variables:
 
@@ -33,219 +30,36 @@ Misc variables:
 
 """
 
-__version__ = "$Revision: 1.36.2.1 $"
-
-# default version for VLARRAY objects
-#obversion = "1.0"    # initial version
-obversion = "1.0"    # add support for complex datatypes
-
-import types, warnings, sys
+import sys
+import warnings
 import cPickle
+
 import numarray
-import numarray.strings as strings
 import numarray.records as records
-from Leaf import Leaf
-import hdf5Extension
-#import IsDescription # to access BaseCol without polluting public namespace
-from IsDescription import Col, BoolCol, StringCol, IntCol, FloatCol, ComplexCol
-from utils import processRange, processRangeRead, convertIntoNA
 
 try:
     import Numeric
-    Numeric_imported = 1
-except:
-    Numeric_imported = 0
+    Numeric_imported = True
+except ImportError:
+    Numeric_imported = False
 
-def checkflavor(flavor, dtype):
-    
-    #if dtype == "CharType" or isinstance(dtype, records.Char):
-    if str(dtype) == "CharType":
-        if flavor in ["CharArray", "String"]:
-            return flavor
-        else:
-            raise ValueError, \
-"""flavor of type '%s' must be one of the "CharArray" or "String" values, and you tried to set it to "%s".
-"""  % (dtype, flavor)
-    else:
-        if flavor in ["NumArray", "Numeric", "Tuple", "List"]:
-            return flavor
-        else:
-            raise ValueError, \
-"""flavor of type '%s' must be one of the "NumArray", "Numeric", "Tuple" or "List" values, and you tried to set it to "%s".
-"""  % (dtype, flavor)
-
-# Class to support variable length strings as components of VLArray
-# It supports UNICODE strings as well.
-class VLStringAtom(IntCol):
-    """ Define an atom of type Variable Length String """
-    def __init__(self):
-        # This special strings will be represented by unsigned bytes
-        IntCol.__init__(self, itemsize=1, shape=1, sign=0)
-        self.flavor = "VLString"
-
-    def __repr__(self):
-        return "VLString()"
-
-    def atomsize(self):
-        " Compute the item size of the VLStringAtom "
-        # Always return 1 because strings are saved in UTF-8 format
-        return 1
-
-class ObjectAtom(IntCol):
-    """ Define an atom of type Object """
-    def __init__(self):
-        IntCol.__init__(self, shape=1, itemsize=1, sign=0)
-        self.flavor = "Object"
-
-    def __repr__(self):
-        return "Object()"
-
-    def atomsize(self):
-        " Compute the item size of the Object "
-        # Always return 1 because strings are saved in UInt8 format
-        return 1
+import tables.hdf5Extension as hdf5Extension
+from tables.utils import processRangeRead, convertIntoNA, convToFlavor
+from tables.Atom import Atom, ObjectAtom, VLStringAtom, StringAtom, EnumAtom
+from tables.Leaf import Leaf
 
 
-#class Atom(IsDescription.BaseCol):
-class Atom(Col):
-    """ Define an Atomic object to be used in VLArray objects """
 
-    def __init__(self, dtype="Float64", shape=1, flavor="NumArray"):
-        Col.__init__(self, dtype, shape)
-        self.flavor = checkflavor(flavor, self.type)
+__version__ = "$Revision: 1153 $"
 
-    def __repr__(self):
-        if self.type == "CharType" or isinstance(self.type, records.Char):
-            if self.shape == 1:
-                shape = [self.itemsize]
-            else:
-                shape = list(self.shape)
-                shape.append(self.itemsize)
-            shape = tuple(shape)
-        else:
-            shape = self.shape
+# default version for VLARRAY objects
+#obversion = "1.0"    # initial version
+#obversion = "1.0"    # add support for complex datatypes
+#obversion = "1.1"    # This adds support for time datatypes.
+obversion = "1.2"    # This adds support for enumerated datatypes.
 
-        out = "Atom(type=" +  str(self.type) + \
-              ", shape=" +  str(shape) + \
-              ", flavor=" + "'" + str(self.flavor) + "'" + \
-              ")"
-        return out
 
-    def atomsize(self):
-        " Compute the size of the atom type "
-        atomicsize = self.itemsize
-        if isinstance(self.shape, types.TupleType):
-            for i in self.shape:
-                if i > 0:  # To deal with EArray Atoms
-                    atomicsize *= i
-        else:
-            atomicsize *= self.shape
-        return atomicsize
 
-    
-class StringAtom(Atom, StringCol):
-    """ Define an atom of type String """
-    def __init__(self, shape=1, length=None, flavor="CharArray"):
-        StringCol.__init__(self, length=length, shape=shape)
-        self.flavor = checkflavor(flavor, self.type)
-        
-class BoolAtom(Atom, BoolCol):
-    """ Define an atom of type Bool """
-    def __init__(self, shape=1, flavor="NumArray"):
-        BoolCol.__init__(self, shape=shape)
-        self.flavor = checkflavor(flavor, self.type)
-
-class IntAtom(Atom, IntCol):
-    """ Define an atom of type Integer """
-    def __init__(self, shape=1, itemsize=4, sign=1, flavor="NumArray"):
-        IntCol.__init__(self, shape=shape, itemsize=itemsize, sign=sign)
-        self.flavor = checkflavor(flavor, self.type)
-
-class Int8Atom(Atom, IntCol):
-    """ Define an atom of type Int8 """
-    def __init__(self, shape=1, flavor="NumArray"):
-        IntCol.__init__(self, shape=shape, itemsize=1, sign=1)
-        self.flavor = checkflavor(flavor, self.type)
-
-class UInt8Atom(Atom, IntCol):
-    """ Define an atom of type UInt8 """
-    def __init__(self, shape=1, flavor="NumArray"):
-        IntCol.__init__(self, shape=shape, itemsize=1, sign=0)
-        self.flavor = checkflavor(flavor, self.type)
-
-class Int16Atom(Atom, IntCol):
-    """ Define an atom of type Int16 """
-    def __init__(self, shape=1, flavor="NumArray"):
-        IntCol.__init__(self, shape=shape, itemsize=2, sign=1)
-        self.flavor = checkflavor(flavor, self.type)
-
-class UInt16Atom(Atom, IntCol):
-    """ Define an atom of type UInt16 """
-    def __init__(self, shape=1, flavor="NumArray"):
-        IntCol.__init__(self, shape=shape, itemsize=2, sign=0)
-        self.flavor = checkflavor(flavor, self.type)
-
-class Int32Atom(Atom, IntCol):
-    """ Define an atom of type Int32 """
-    def __init__(self, shape=1, flavor="NumArray"):
-        IntCol.__init__(self, shape=shape, itemsize=4, sign=1)
-        self.flavor = checkflavor(flavor, self.type)
-
-class UInt32Atom(IntCol, Atom):
-    """ Define an atom of type UInt32 """
-    def __init__(self, shape=1, flavor="NumArray"):
-        IntCol.__init__(self, shape=shape, itemsize=4, sign=0)
-        self.flavor = checkflavor(flavor, self.type)
-
-class Int64Atom(Atom, IntCol):
-    """ Define an atom of type Int64 """
-    def __init__(self, shape=1, flavor="NumArray"):
-        IntCol.__init__(self, shape=shape, itemsize=8, sign=1)
-        self.flavor = checkflavor(flavor, self.type)
-
-class UInt64Atom(Atom, IntCol):
-    """ Define an atom of type UInt64 """
-    def __init__(self, shape=1, flavor="NumArray"):
-        IntCol.__init__(self, shape=shape, itemsize=8, sign=0)
-        self.flavor = checkflavor(flavor, self.type)
-
-class FloatAtom(Atom, FloatCol):
-    """ Define an atom of type Float """
-    def __init__(self, shape=1, itemsize=8, flavor="NumArray"):
-        FloatCol.__init__(self, shape=shape, itemsize=itemsize)
-        self.flavor = checkflavor(flavor, self.type)
-
-class Float32Atom(Atom, FloatCol):
-    """ Define an atom of type Float32 """
-    def __init__(self, shape=1, flavor="NumArray"):
-        FloatCol.__init__(self, shape=shape, itemsize=4)
-        self.flavor = checkflavor(flavor, self.type)
-
-class Float64Atom(Atom, FloatCol):
-    """ Define an atom of type Float64 """
-    def __init__(self, shape=1, flavor="NumArray"):
-        FloatCol.__init__(self, shape=shape, itemsize=8)
-        self.flavor = checkflavor(flavor, self.type)
-
-class ComplexAtom(Atom, ComplexCol):
-    """ Define an atom of type Complex """
-    def __init__(self, shape=1, itemsize=16, flavor="NumArray"):
-        ComplexCol.__init__(self, shape=shape, itemsize=itemsize)
-        self.flavor = checkflavor(flavor, self.type)
-
-class Complex32Atom(Atom, ComplexCol):
-    """ Define an atom of type Complex32 """
-    def __init__(self, shape=1, flavor="NumArray"):
-        ComplexCol.__init__(self, shape=shape, itemsize=8)
-        self.flavor = checkflavor(flavor, self.type)
-
-class Complex64Atom(Atom, ComplexCol):
-    """ Define an atom of type Complex64 """
-    def __init__(self, shape=1, flavor="NumArray"):
-        ComplexCol.__init__(self, shape=shape, itemsize=16)
-        self.flavor = checkflavor(flavor, self.type)
-
-        
 def calcChunkSize(expectedsizeinMB, complevel):
     """Computes the optimum value for the chunksize"""
     if expectedsizeinMB <= 100:
@@ -273,7 +87,7 @@ def calcChunkSize(expectedsizeinMB, complevel):
     return chunksize
 
 
-class VLArray(Leaf, hdf5Extension.VLArray, object):
+class VLArray(hdf5Extension.VLArray, Leaf):
     """Represent a variable length (ragged) array in HDF5 file.
 
     It enables to create new datasets on-disk from Numeric, numarray,
@@ -287,7 +101,7 @@ class VLArray(Leaf, hdf5Extension.VLArray, object):
 
     Methods:
 
-        append(*objects)
+        append(sequence)
         read(start, stop, step)
         __iter__()
         iterrows(start, stop, step)
@@ -302,7 +116,18 @@ class VLArray(Leaf, hdf5Extension.VLArray, object):
         nrows -- The total number of rows
 
     """
-    
+
+    # Class identifier.
+    _c_classId = 'VLARRAY'
+
+
+    # <undo-redo support>
+    _c_canUndoCreate = True  # Can creation/copying be undone and redone?
+    _c_canUndoRemove = True  # Can removal be undone and redone?
+    _c_canUndoMove   = True  # Can movement/renaming be undone and redone?
+    # </undo-redo support>
+
+
     def __init__(self, atom=None, title = "",
                  filters = None, expectedsizeinMB = 1.0):
         """Create the instance Array.
@@ -311,7 +136,7 @@ class VLArray(Leaf, hdf5Extension.VLArray, object):
 
         atom -- An Atom object representing the shape, type and
             flavor of the atomic objects to be saved.
-        
+
         title -- Sets a TITLE attribute on the HDF5 array entity.
 
         filters -- An instance of the Filters class that provides
@@ -340,6 +165,7 @@ class VLArray(Leaf, hdf5Extension.VLArray, object):
 
     def _create(self):
         """Create a variable length array (ragged array)."""
+
         global obversion
 
         self._v_version = obversion
@@ -356,6 +182,7 @@ class VLArray(Leaf, hdf5Extension.VLArray, object):
                   "When creating VLArrays, none of the dimensions of the Atom instance can be zero."
 
         self._atomictype = self.atom.type
+        self._atomicstype = self.atom.stype
         self._atomicshape = self.atom.shape
         self._atomicsize = self.atom.atomsize()
         self._basesize = self.atom.itemsize
@@ -367,7 +194,7 @@ class VLArray(Leaf, hdf5Extension.VLArray, object):
         self.nrows = 0     # No rows in creation time
         self.shape = (0,)
         self._createArray(self._v_new_title)
-            
+
     def _checkShape(self, naarr):
         # Check for zero dimensionality array
         zerodims = numarray.sum(numarray.array(naarr.shape) == 0)
@@ -377,7 +204,7 @@ class VLArray(Leaf, hdf5Extension.VLArray, object):
         shape = naarr.shape
         atom_shape = self.atom.shape
         shapelen = len(naarr.shape)
-        if isinstance(atom_shape, types.TupleType):
+        if isinstance(atom_shape, tuple):
             atomshapelen = len(self.atom.shape)
         else:
             atom_shape = (self.atom.shape,)
@@ -402,23 +229,86 @@ class VLArray(Leaf, hdf5Extension.VLArray, object):
 """The object '%s' is composed of elements with shape '%s', which is not compatible with the atom shape ('%s').""" % \
 (naarr, shape, atom_shape)
         return nobjects
-            
-    def append(self, *objects):
-        """Append the objects to this enlargeable object"""
-        assert not self._v_file.mode == "r", "Attempt to write over a file opened in read-only mode"
 
-        # To make append([1,0,1]) equivalent to append(1,0,1)
-        if len(objects) == 0:
-            object = None
-        elif len(objects) == 1:
-            # Correction for only one parameter passed
-            object = objects[0]
+
+    def getEnum(self):
+        """
+        Get the enumerated type associated with this array.
+
+        If this array is of an enumerated type, the corresponding `Enum`
+        instance is returned.  If it is not of an enumerated type, a
+        ``TypeError`` is raised.
+        """
+
+        if self.atom.stype != 'Enum':
+            raise TypeError("array ``%s`` is not of an enumerated type"
+                            % self._v_pathname)
+
+        return self.atom.enum
+
+
+    def append(self, sequence, *objects):
+        """
+        Append objects in the `sequence` to the array.
+
+        This method appends the objects in the `sequence` to a *single
+        row* in this array.  The type of individual objects must be
+        compliant with the type of atoms in the array.  In the case of
+        variable length strings, the very string to append is the
+        `sequence`.
+
+        Example of use (code available in ``examples/vlarray1.py``)::
+
+            import tables
+            from Numeric import *   # or, from numarray import *
+
+            # Create a VLArray:
+            fileh = tables.openFile("vlarray1.h5", mode = "w")
+            vlarray = fileh.createVLArray(fileh.root, 'vlarray1',
+            tables.Int32Atom(flavor="Numeric"),
+                             "ragged array of ints", Filters(complevel=1))
+            # Append some (variable length) rows:
+            vlarray.append(array([5, 6]))
+            vlarray.append(array([5, 6, 7]))
+            vlarray.append([5, 6, 9, 8])
+
+            # Now, read it through an iterator:
+            for x in vlarray:
+                print vlarray.name+"["+str(vlarray.nrow)+"]-->", x
+
+            # Close the file
+            fileh.close()
+
+        The output of the previous program looks like this::
+
+            vlarray1[0]--> [5 6]
+            vlarray1[1]--> [5 6 7]
+            vlarray1[2]--> [5 6 9 8]
+
+        The `objects` argument is only retained for backwards
+        compatibility; please do *not* use it.
+        """
+
+        if self._v_file.mode == 'r':
+            raise IOError("attempt to write over a file opened in read-only mode")
+
+        isseq = True
+        try:  # fastest check in most cases
+            len(sequence)
+        except TypeError:
+            isseq = False
+
+        if not isseq or len(objects) > 0:
+            warnings.warn(DeprecationWarning("""\
+using multiple arguments with ``append()`` is *strongly deprecated*; \
+please put them in a single sequence object"""),
+                          stacklevel=2)
+            # This is not optimum, but neither frequent.
+            object = (sequence,) + tuple(objects)
         else:
-            if self.atom.flavor == "VLString":
-                raise ValueError, \
-"""The append method only accepts one parameter for 'VLString' data type."""
-            else:
-                object = objects
+            object = sequence
+        # After that, `object` is assured to be a sequence.
+
         # Prepare the object to convert it into a numarray object
         if self.atom.flavor == "Object":
             # Special case for a generic object
@@ -427,48 +317,53 @@ class VLArray(Leaf, hdf5Extension.VLArray, object):
         elif self.atom.flavor == "VLString":
             # Special case for a generic object
             # (to be pickled and saved as an array of unsigned bytes)
-            if not (isinstance(object, types.StringType) or
-                    isinstance(object, types.UnicodeType)):
+            if type(object) not in (str,unicode):
                 raise TypeError, \
 """The object "%s" is not of type String or Unicode.""" % (str(object))
             try:
                 object = object.encode('utf-8')
-            except:
-                (type, value, traceback) = sys.exc_info()
+            except UnicodeError:
+                (typerr, value, traceback) = sys.exc_info()
                 raise ValueError, "Problems when converting the object '%s' to the encoding 'utf-8'. The error was: %s" % (object, value)
             object = numarray.array(object, type=numarray.UInt8)
 
-        if len(objects) > 0:
-            naarr = convertIntoNA(object, self.atom)
+        if len(object) > 0:
+            # The object needs to be copied to make the operation safe
+            # to in-place conversion.
+            copy = self._atomicstype in ['Time64']
+            naarr = convertIntoNA(object, self.atom, copy)
             nobjects = self._checkShape(naarr)
         else:
             nobjects = 0
             naarr = None
-        if self._append(naarr, nobjects) > 0:
-            self.nrows += 1
-            self.shape = (self.nrows,)
-            # Return the last entry in object
-            return self.nrows
-        else:
-            return -1
+
+        self._append(naarr, nobjects)
+
+        self.nrows += 1
+        self.shape = (self.nrows,)
+
 
     def _open(self):
         """Get the metadata info for an array in file."""
+
         self.nrows = self._openArray()
         self.shape = (self.nrows,)
+
+        flavor = self.flavor
+        stype = self._atomicstype
         # First, check the special cases VLString and Object types
-        if self.flavor == "VLString":
+        if flavor == "VLString":
             self.atom = VLStringAtom()
-        elif self.flavor == "Object":
+        elif flavor == "Object":
             self.atom = ObjectAtom()
+        elif stype == 'CharType':
+            self.atom = StringAtom(self._atomicshape, self._basesize, flavor)
+        elif stype == 'Enum':
+            (enum, type_) = self._loadEnum()
+            self.atom = EnumAtom(enum, type_, self._atomicshape, flavor)
+            self._atomictype = type_
         else:
-            if str(self._atomictype) == "CharType":
-                self.atom = StringAtom(shape=self._atomicshape,
-                                       length=self._basesize,
-                                       flavor=self.flavor)
-            else:
-                self.atom = Atom(self._atomictype, self._atomicshape,
-                                 self.flavor)
+            self.atom = Atom(stype, self._atomicshape, flavor)
 
     def iterrows(self, start=None, stop=None, step=None):
         """Iterate over all the rows or a range.
@@ -479,7 +374,7 @@ class VLArray(Leaf, hdf5Extension.VLArray, object):
                      processRangeRead(self.nrows, start, stop, step)
         self._initLoop()
         return self
-        
+
     def __iter__(self):
         """Iterate over all the rows."""
 
@@ -518,54 +413,6 @@ class VLArray(Leaf, hdf5Extension.VLArray, object):
             self._nrowsread += self._step
             return self.listarr[self._row]
 
-    def _convToFlavor(self, arr):
-        "convert the numarray parameter to to correct flavor"
-
-        # Convert to Numeric, tuple or list if needed
-        if self.flavor == "Numeric":
-            if Numeric_imported:
-                if str(arr.type()) == "Bool":
-                    # Typecode boolean does not exist on Numeric
-                    typecode = "1"
-                else:
-                    typecode = arr.typecode()                        
-                # tolist() method creates a list with a sane byteorder
-                if arr.shape <> ():
-                    arr=Numeric.array(arr.tolist(), typecode)
-                else:
-                    # This works for rank-0 arrays
-                    # (but is slower for big arrays)
-                    arr=Numeric.array(arr, typecode)
-
-            else:
-                # Warn the user
-                warnings.warn( \
-"""The object on-disk is type Numeric, but Numeric is not installed locally.
-  Returning a numarray object instead!.""")
-        elif self.flavor == "Tuple":
-            arr = tuple(arr.tolist())
-        elif self.flavor == "List":
-            arr = arr.tolist()
-        elif self.flavor == "String":
-            arr = arr.tolist()
-        elif self.flavor == "VLString":
-            arr = arr.tostring().decode('utf-8')
-        elif self.flavor == "Object":
-            # We have to check for an empty array because of a
-            # possible bug in HDF5 that claims that a dataset
-            # has one record when in fact, it is empty
-            if len(arr) == 0:
-                arr = []
-            else:
-                arr = cPickle.loads(arr.tostring())
-            # The next should also do the job
-#             try:
-#                 arr = cPickle.loads(arr.tostring())
-#             except:
-#                 arr = []
-
-        return arr
-
     def __getitem__(self, key):
         """Returns a vlarray row or slice.
 
@@ -577,19 +424,19 @@ class VLArray(Leaf, hdf5Extension.VLArray, object):
 
         """
 
-        if isinstance(key, types.IntType):
+        if type(key) in (int,long):
             if key >= self.nrows:
                 raise IndexError, "Index out of range"
             if key < 0:
                 # To support negative values
                 key += self.nrows
             return self.read(key)[0]
-        elif isinstance(key, types.SliceType):
+        elif isinstance(key, slice):
             return self.read(key.start, key.stop, key.step)
         else:
             raise IndexError, "Non-valid index or slice: %s" % \
                   key
-        
+
     def __setitem__(self, keys, value):
         """Updates a vlarray row "keys" by setting it to "value".
 
@@ -616,15 +463,16 @@ class VLArray(Leaf, hdf5Extension.VLArray, object):
 
         """
 
-        assert not self._v_file.mode == "r", "Attempt to write over a file opened in read-only mode"
+        if self._v_file.mode == 'r':
+            raise IOError("attempt to write over a file opened in read-only mode")
 
-        if not isinstance(keys, types.TupleType):
+        if not isinstance(keys, tuple):
             keys = (keys, None)
         if len(keys) > 2:
             raise IndexError, "You cannot specify more than two dimensions"
         nrow, rng = keys
         # Process the first index
-        if not isinstance(nrow, types.IntType):
+        if type(nrow) not in (int,long):
             raise IndexError, "The first dimension only can be an integer"
         if nrow >= self.nrows:
             raise IndexError, "First index out of range"
@@ -632,15 +480,15 @@ class VLArray(Leaf, hdf5Extension.VLArray, object):
             # To support negative values
             nrow += self.nrows
         # Process the second index
-        if isinstance(rng, types.IntType):
+        if type(rng) in (int,long):
             start = rng; stop = start+1; step = 1
-        elif isinstance(rng, types.SliceType):
+        elif isinstance(rng, slice):
             start, stop, step = rng.start, rng.stop, rng.step
-        elif isinstance(rng, types.NoneType):
+        elif rng is None:
             start, stop, step = None, None, None
         else:
             raise IndexError, "Non-valid second index or slice: %s" % rng
-        
+
         object = value
         # Prepare the object to convert it into a numarray object
         if self.atom.flavor == "Object":
@@ -650,14 +498,13 @@ class VLArray(Leaf, hdf5Extension.VLArray, object):
         elif self.atom.flavor == "VLString":
             # Special case for a generic object
             # (to be pickled and saved as an array of unsigned bytes)
-            if not (isinstance(object, types.StringType) or
-                    isinstance(object, types.UnicodeType)):
+            if type(object) not in (str,unicode):
                 raise TypeError, \
 """The object "%s" is not of type String or Unicode.""" % (str(object))
             try:
                 object = object.encode('utf-8')
-            except:
-                (type, value, traceback) = sys.exc_info()
+            except UnicodeError:
+                (typerr, value, traceback) = sys.exc_info()
                 raise ValueError, "Problems when converting the object '%s' to the encoding 'utf-8'. The error was: %s" % (object, value)
             object = numarray.array(object, type=numarray.UInt8)
 
@@ -677,8 +524,8 @@ class VLArray(Leaf, hdf5Extension.VLArray, object):
         # Assign the value to it
         try:
             naarr[slice(start, stop, step)] = value
-        except:
-            (type, value2, traceback) = sys.exc_info()
+        except:  #XXX
+            (typerr, value2, traceback) = sys.exc_info()
             raise ValueError, \
 "Value parameter:\n'%r'\ncannot be converted into an array object compliant vlarray[%s] row: \n'%r'\nThe error was: <%s>" % \
         (value, keys, naarr[slice(start, stop, step)], value2)
@@ -697,19 +544,21 @@ class VLArray(Leaf, hdf5Extension.VLArray, object):
             listarr = self._readArray(start, stop, step)
         if self.flavor <> "NumArray":
             # Convert the list to the right flavor
-            outlistarr = [self._convToFlavor(arr) for arr in listarr ]
+            outlistarr = [convToFlavor(self, arr, "VLArray")
+                          for arr in listarr ]
+            if self.flavor == "Tuple":
+                outlistarr = tuple(outlistarr)
         else:
             # NumArray flavor does not need additional conversion
             outlistarr = listarr
         return outlistarr
 
-    def _g_copy(self, group, name, start, stop, step, title, filters):
+    def _g_copyWithStats(self, group, name, start, stop, step, title, filters):
         "Private part of Leaf.copy() for each kind of leaf"
         # Build the new VLArray object
-        object = VLArray(self.atom, title=title,
-                         filters=filters,
-                         expectedsizeinMB=self._v_expectedsizeinMB)
-        setattr(group, name, object)
+        object = self._v_file.createVLArray(
+            group, name, self.atom, title=title, filters=filters,
+            expectedsizeinMB=self._v_expectedsizeinMB, _log = False)
         # Now, fill the new vlarray with values from the old one
         # This is not buffered because we cannot forsee the length
         # of each record. So, the safest would be a copy row by row.
@@ -725,7 +574,7 @@ class VLArray(Leaf, hdf5Extension.VLArray, object):
             # Save the records on disk
             stop2 = start2+step*nrowsinbuf
             if stop2 > stop:
-                stop2 = stop 
+                stop2 = stop
             naarr = self._readArray(start=start2, stop=stop2, step=step)[0]
             nobjects = naarr.shape[0]
             object._append(naarr, nobjects)
