@@ -5,9 +5,12 @@ Run all test cases.
 import sys
 import os
 import unittest
+
 from numarray import *
 from numarray import strings
 
+
+
 verbose = 0
 heavy = 0  # Default is not doing heavy testing
 if 'verbose' in sys.argv:
@@ -30,39 +33,55 @@ import test_all
 test_all.verbose = verbose
 test_all.heavy = heavy
 
+def cleanup(klass):
+    #klass.__dict__.clear()     # This is too hard. Don't do that
+#    print "Class attributes deleted"
+    for key in klass.__dict__.keys():
+	if not klass.__dict__[key].__class__.__name__ in ('instancemethod'):
+	    klass.__dict__[key] = None
+
 def allequal(a,b, flavor="numarray"):
     """Checks if two numarrays are equal"""
 
-#     print "a-->", repr(a)
-#     print "b-->", repr(b)
+    #print "a-->", repr(a)
+    #print "b-->", repr(b)
     if not hasattr(b, "shape"):
         # Scalar case
         return a == b
 
     if flavor == "Numeric":
         # Convert the parameters to numarray objects
-        try:
-            a = array(buffer(a), type=typeDict[a.typecode()], shape=a.shape)
-        except:
+        if a.typecode() == "c":
+	    a = strings.array(a.tostring(), itemsize=1, shape=a.shape)
+	    b = strings.array(b.tostring(), itemsize=1, shape=b.shape)
+        else:
+            # if using:
+            #a = array(buffer(a),type=typeDict[a.typecode()],shape=a.shape)
+            # the next error is issued for non-contiguous arrays (at
+            # least from numarray 1.2.3 and Numeric 23.8 on):
+            # """ TypeError: single-segment buffer object expected"""
             try:
-                a = strings.array(a.tolist(), itemsize=1, shape=a.shape)
-            except:
-                # Fallback case
-                a = array(a)
-        try:
-            b = array(buffer(b), type=typeDict[b.typecode()], shape=b.shape)
-        except:
+                a = array(a,type=typeDict[a.typecode()],shape=a.shape)
+            except ValueError:
+                # This line is able to import Numeric objects like:
+                # zeros((0,), 'l')
+                # while the previous one don't
+                a = array(buffer(a),type=typeDict[a.typecode()],shape=a.shape)
             try:
-                b = strings.array(b.tolist(), itemsize=1, shape=b.shape)
-            except:
-                # Fallback case
-                b = array(b)
+                b = array(b,type=typeDict[b.typecode()],shape=b.shape)
+            except ValueError:
+                # This line is able to import Numeric objects like:
+                # zeros((0,), 'l')
+                # while the previous one don't
+                b = array(buffer(b),type=typeDict[b.typecode()],shape=b.shape)
+
 
     if a.shape <> b.shape:
         if verbose:
             print "Shape is not equal:", a.shape, "<>", b.shape
         return 0
 
+    #if hasattr(a, "type") and hasattr(b, "type") and a.type() <> b.type():
     if hasattr(b, "type") and a.type() <> b.type():
         if verbose:
             print "Type is not equal:", a.type(), "<>", b.type()
@@ -70,7 +89,7 @@ def allequal(a,b, flavor="numarray"):
 
     # Rank-0 case
     if len(a.shape) == 0:
-        if str(equal(a,b)) == '1':
+        if a[()] == b[()]:
             return 1
         else:
             if verbose:
@@ -95,27 +114,34 @@ def allequal(a,b, flavor="numarray"):
         result = logical_and.reduce(result)
     if not result and verbose:
         print "Some of the elements in arrays are not equal"
-        
+
     return result
 
 
 def suite():
-    test_modules = ['test_basics',
+    test_modules = ['test_attributes',
+                    'test_basics',
                     'test_create',
                     'test_backcompat',
                     'test_types',
                     'test_numarray',
                     'test_lists',
                     'test_tables',
+                    'test_tablesMD',
                     'test_indexes',
                     'test_indexvalues',
-                    'test_tablesMD',
                     'test_vlarray',
                     'test_earray',
+                    'test_carray',
 		    'test_tree',
+		    'test_timetype',
+                    'test_do_undo',
+                    'test_enum',
+                    'test_nestedrecords',
+                    'test_nestedtypes',
                     # Sub-packages
                     'test_filenode',
-		    ]
+	    ]
 
     # Add test_Numeric only if Numeric is installed
     try:
@@ -136,25 +162,28 @@ def suite():
 
 
 if __name__ == '__main__':
-    import tables
     import numarray
+    import tables
 
     print '-=' * 38
     print "PyTables version:  %s" % tables.__version__
-    print "Extension version: %s" % tables.ExtVersion
+    print "Extension version: %s" % tables.extVersion
     print "HDF5 version:      %s" % tables.whichLibVersion("hdf5")[1]
-    #print "HDF5 version:      %s" % tables.HDF5Version
+    #print "HDF5 version:      %s" % tables.hdf5Version
     print "numarray version:  %s" % numarray.__version__
     #print "Zlib version:      %s" % tables.whichLibVersion("zlib")[1]
     tinfo = tables.whichLibVersion("zlib")
-    if tinfo[0]:
+    if tinfo is not None:
         print "Zlib version:      %s" % (tinfo[1])
     tinfo = tables.whichLibVersion("lzo")
-    if tinfo[0]:
+    if tinfo is not None:
         print "LZO version:       %s (%s)" % (tinfo[1], tinfo[2])
     tinfo = tables.whichLibVersion("ucl")
-    if tinfo[0]:
+    if tinfo is not None:
         print "UCL version:       %s (%s)" % (tinfo[1], tinfo[2])
+    tinfo = tables.whichLibVersion("bzip2")
+    if tinfo is not None:
+        print "BZIP2 version:     %s (%s)" % (tinfo[1], tinfo[2])
     print 'Python version:    %s' % sys.version
     if os.name == 'posix':
         (sysname, nodename, release, version, machine) = os.uname()
@@ -163,7 +192,7 @@ if __name__ == '__main__':
     print '-=' * 38
 
     # Handle --show-versions-only
-    only_versions = 0    
+    only_versions = 0
     args = sys.argv[:]
     for arg in args:
         if arg == '--show-versions-only':
