@@ -11,12 +11,15 @@ import unittest
 import os
 import re
 import tempfile
+import warnings
 
 from tables import *
 # important objects to test
 from tables import File, Group, Leaf, Table, Array
 
-from test_all import verbose, heavy
+from test_all import verbose, heavy, cleanup
+# To delete the internal attributes automagically
+unittest.TestCase.tearDown = cleanup
 
 class Record(IsDescription):
     var1 = StringCol(length=4)     # 4-character String
@@ -26,7 +29,6 @@ class Record(IsDescription):
     var5 = Float32Col()            # float  (single-precision)
 
 class createTestCase(unittest.TestCase):
-    
     file  = "test.h5"
     title = "This is the table title"
     expectedrows = 100
@@ -34,7 +36,7 @@ class createTestCase(unittest.TestCase):
     maxint   = 2147483648   # (2 ** 31)
     compress = 0
 
-    
+
     def setUp(self):
         # Create an instance of HDF5 Table
         self.fileh = openFile(self.file, mode = "w")
@@ -54,6 +56,7 @@ class createTestCase(unittest.TestCase):
 
         self.fileh.close()
         os.remove(self.file)
+        cleanup(self)
 
     #----------------------------------------
 
@@ -70,58 +73,61 @@ class createTestCase(unittest.TestCase):
         try:
             self.array = self.fileh.createArray(self.root, 'anarray',
                                                 [1], "Array title")
-        except NameError:
+        except NodeError:
             if verbose:
                 (type, value, traceback) = sys.exc_info()
                 print "\nGreat!, the next NameError was catched!"
                 print value
         else:
-            self.fail("expected a NameError")
-	
+            self.fail("expected a NodeError")
+
     def test02_syntaxname(self):
         """Checking syntax in object tree names"""
 
 	# Now, try to attach an array to the object tree with
 	# a not allowed Python variable name
+        warnings.filterwarnings("error", category=NaturalNameWarning)
         try:
             self.array = self.fileh.createArray(self.root, ' array',
                                                 [1], "Array title")
-        except NameError:
+        except NaturalNameWarning:
             if verbose:
                 (type, value, traceback) = sys.exc_info()
-                print "\nGreat!, the next NameError was catched!"
+                print "\nGreat!, the next NaturalNameWarning was catched!"
                 print value
         else:
-            self.fail("expected a NameError")
-	    
+            self.fail("expected a NaturalNameWarning")
+
 	# another name error
         try:
             self.array = self.fileh.createArray(self.root, '$array',
                                                 [1], "Array title")
-        except NameError:
+        except NaturalNameWarning:
             if verbose:
                 (type, value, traceback) = sys.exc_info()
-                print "\nGreat!, the next NameError was catched!"
+                print "\nGreat!, the next NaturalNameWarning was catched!"
                 print value
         else:
-            self.fail("expected a NameError")
+            self.fail("expected a NaturalNameWarning")
 
 	# Finally, test a reserved word
         try:
             self.array = self.fileh.createArray(self.root, 'for',
                                                 [1], "Array title")
-        except NameError:
+        except NaturalNameWarning:
             if verbose:
                 (type, value, traceback) = sys.exc_info()
-                print "\nGreat!, the next NameError was catched!"
+                print "\nGreat!, the next NaturalNameWarning was catched!"
                 print value
         else:
-            self.fail("expected a NameError")
+            self.fail("expected a NaturalNameWarning")
+        # Reset the warning
+        warnings.filterwarnings("default", category=NaturalNameWarning)
 
     def test03a_titleAttr(self):
         """Checking the self.title attr in nodes"""
 
-        # Close the opened file to destroy the object tree 
+        # Close the opened file to destroy the object tree
         self.fileh.close()
         # Open the file again to re-create the objects
         self.fileh = openFile(self.file,"r")
@@ -140,13 +146,13 @@ class createTestCase(unittest.TestCase):
                                        "t" * titlelength)
         assert group._v_title == "t" * titlelength
         assert group._f_getAttr('TITLE') == "t" * titlelength
-	
+
 	# Now, try with a table object
 	table = self.fileh.createTable(self.root, 'table',
                                        Record, "t" * titlelength)
 	assert table.title == "t" * titlelength
 	assert table.getAttr("TITLE") == "t" * titlelength
-	    
+
 	# Finally, try with an Array object
         arr = self.fileh.createArray(self.root, 'arr',
                                      [1], "t" * titlelength)
@@ -154,10 +160,11 @@ class createTestCase(unittest.TestCase):
 	assert arr.getAttr("TITLE") == "t" * titlelength
 
     def test04_maxFields(self):
-	"Checking the maximum number of fields (256) in tables"
+	"Checking a large number of fields (257) in tables"
 
 	# The number of fields for a table
-	varnumber = 256
+	#varnumber = 256
+	varnumber = 257
 
 	varnames = []
 	for i in range(varnumber):
@@ -181,7 +188,7 @@ class createTestCase(unittest.TestCase):
             for i in range(len(table.cols)):
                 row[varnames[i]] = i*j
                 rowlist.append(i*j)
-	    
+
             row.append()
             listrows.append(tuple(rowlist))
 
@@ -198,71 +205,88 @@ class createTestCase(unittest.TestCase):
             print "Retrieved row list:", listout[-1]
         assert listrows == listout
 
+    # The next limitation has been released. A warning is still there, though
     def test05_maxFieldsExceeded(self):
-	"Checking an excess (257) of the maximum number of fields in tables"
+	"Checking an excess (1025) of the maximum number of fields in tables"
 
 	# The number of fields for a table
-	varnumber = 257
+	varnumber = 1025
 
 	varnames = []
 	for i in range(varnumber):
 	    varnames.append('int%d' % i)
-            
+
 	# Build a dictionary with the types as values and varnames as keys
 	recordDict = {}
 	i = 0
 	for varname in varnames:
-	    #recordDict[varname] = vartypes[i]
 	    recordDict[varname] = Col("Int32", 1)
 	    i += 1
-            
-	# Append this entry to indicate the alignment!
-	recordDict['_v_align'] = "="
-        
+
 	# Now, create a table with this record object
         # This way of creating node objects has been deprecated
 	#table = Table(recordDict, "MetaRecord instance")
 
 	# Attach the table to object tree
-        # Here, IndexError should be raised!
+        warnings.filterwarnings("error", category=PerformanceWarning)
+        # Here, a PerformanceWarning should be raised!
         try:
-            #self.root.table = table
             table = self.fileh.createTable(self.root, 'table',
                                            recordDict, "MetaRecord instance")
-        except IndexError:
+        except PerformanceWarning:
             if verbose:
                 (type, value, traceback) = sys.exc_info()
-                print "\nGreat!, the next IndexError was catched!"
+                print "\nGreat!, the next PerformanceWarning was catched!"
                 print value
         else:
-            self.fail("expected an IndexError")
+            self.fail("expected an PerformanceWarning")
+        # Reset the warning
+        warnings.filterwarnings("default", category=PerformanceWarning)
 
-    def test06_maxColumnNameLengthExceeded(self):
+    # The next limitation has been released
+    def _test06_maxColumnNameLengthExceeded(self):
 	"Checking an excess (256) of the maximum length in column names"
 
 	# Build a dictionary with the types as values and varnames as keys
 	recordDict = {}
-	recordDict["a"*255] = IntCol(1) 
-	recordDict["b"*256] = IntCol(1) # Should trigger a NameError
-        
+	recordDict["a"*255] = IntCol(1)
+	recordDict["b"*256] = IntCol(1) # Should trigger a ValueError
+
+	# Now, create a table with this record object
+        # This way of creating node objects has been deprecated
+	table = Table(recordDict, "MetaRecord instance")
+
+	# Attach the table to object tree
+        # Here, ValueError should be raised!
+        try:
+            table = self.fileh.createTable(self.root, 'table',
+                                           recordDict, "MetaRecord instance")
+        except ValueError:
+            if verbose:
+                (type, value, traceback) = sys.exc_info()
+                print "\nGreat!, the next ValueError was catched!"
+                print value
+        else:
+            self.fail("expected an ValueError")
+
+    def test06_noMaxColumnNameLength(self):
+	"Checking unlimited length in column names"
+
+	# Build a dictionary with the types as values and varnames as keys
+	recordDict = {}
+	recordDict["a"*255] = IntCol(1, pos=0)
+	recordDict["b"*1024] = IntCol(1, pos=1) # Should work well
+
 	# Now, create a table with this record object
         # This way of creating node objects has been deprecated
 	table = Table(recordDict, "MetaRecord instance")
 
 	# Attach the table to object tree
         # Here, IndexError should be raised!
-        try:
-            # self.root.table = table
-            table = self.fileh.createTable(self.root, 'table',
-                                           recordDict, "MetaRecord instance")
-        except NameError:
-            if verbose:
-                (type, value, traceback) = sys.exc_info()
-                print "\nGreat!, the next NameError was catched!"
-                print value
-        else:
-            self.fail("expected an NameError")
-
+        table = self.fileh.createTable(self.root, 'table',
+                                       recordDict, "MetaRecord instance")
+        assert table.colnames[0] == "a"*255
+        assert table.colnames[1] == "b"*1024
 
 class createAttrTestCase(unittest.TestCase):
     file  = "test.h5"
@@ -271,7 +295,7 @@ class createAttrTestCase(unittest.TestCase):
     maxshort = 2 ** 15
     maxint   = 2147483648   # (2 ** 31)
     compress = 0
-    
+
     def setUp(self):
         # Create an instance of HDF5 Table
         self.fileh = openFile(self.file, mode = "w")
@@ -290,6 +314,7 @@ class createAttrTestCase(unittest.TestCase):
     def tearDown(self):
         self.fileh.close()
         os.remove(self.file)
+        cleanup(self)
 
 #---------------------------------------
 
@@ -298,13 +323,13 @@ class createAttrTestCase(unittest.TestCase):
 
 	attrlength = 2048
 	# Try to put a long string attribute on a group object
-	attr = self.fileh.setAttrNode(self.root.agroup,
+	attr = self.fileh.setNodeAttr(self.root.agroup,
                                       "attr1", "p" * attrlength)
 	# Now, try with a Table object
-	attr = self.fileh.setAttrNode(self.root.atable,
+	attr = self.fileh.setNodeAttr(self.root.atable,
                                       "attr1", "a" * attrlength)
 	# Finally, try with an Array object
-	attr = self.fileh.setAttrNode(self.root.anarray,
+	attr = self.fileh.setNodeAttr(self.root.anarray,
                                       "attr1", "n" * attrlength)
 
         if self.close:
@@ -314,14 +339,13 @@ class createAttrTestCase(unittest.TestCase):
             self.fileh = openFile(self.file, mode = "r+")
             self.root = self.fileh.root
 
-        assert self.fileh.getAttrNode(self.root.agroup, 'attr1') == \
+        assert self.fileh.getNodeAttr(self.root.agroup, 'attr1') == \
                "p" * attrlength
-        assert self.fileh.getAttrNode(self.root.atable, 'attr1') == \
+        assert self.fileh.getNodeAttr(self.root.atable, 'attr1') == \
                "a" * attrlength
-        assert self.fileh.getAttrNode(self.root.anarray, 'attr1') == \
+        assert self.fileh.getNodeAttr(self.root.anarray, 'attr1') == \
                "n" * attrlength
-	    
-	    
+
     def test02_setAttributes(self):
         """Checking setting large string attributes (Node methods)"""
 
@@ -330,7 +354,7 @@ class createAttrTestCase(unittest.TestCase):
         self.root.agroup._f_setAttr('attr1', "p" * attrlength)
 	# Now, try with a Table object
         self.root.atable.setAttr('attr1', "a" * attrlength)
-	    
+
 	# Finally, try with an Array object
         self.root.anarray.setAttr('attr1', "n" * attrlength)
 
@@ -344,8 +368,8 @@ class createAttrTestCase(unittest.TestCase):
         assert self.root.agroup._f_getAttr('attr1') == "p" * attrlength
 	assert self.root.atable.getAttr("attr1") == "a" * attrlength
 	assert self.root.anarray.getAttr("attr1") == "n" * attrlength
-	    
-	    
+
+
     def test03_setAttributes(self):
         """Checking setting large string attributes (AttributeSet methods)"""
 
@@ -356,7 +380,7 @@ class createAttrTestCase(unittest.TestCase):
         self.table.attrs.attr1 = "a" * attrlength
 	# Finally, try with an Array object
         self.array.attrs.attr1 = "n" * attrlength
-        
+
         if self.close:
             if verbose:
                 print "(closing file version)"
@@ -367,7 +391,7 @@ class createAttrTestCase(unittest.TestCase):
         assert self.root.agroup._v_attrs.attr1 == "p" * attrlength
 	assert self.root.atable.attrs.attr1 == "a" * attrlength
 	assert self.root.anarray.attrs.attr1 == "n" * attrlength
-	    
+
     def test04_listAttributes(self):
         """Checking listing attributes """
 
@@ -398,26 +422,27 @@ class createAttrTestCase(unittest.TestCase):
             self.fileh.close()
             self.fileh = openFile(self.file, mode = "r+")
             self.root = self.fileh.root
-            
+
         assert self.root.agroup._v_attrs._f_list("user") == \
                ["pq", "qr", "rs"]
         assert self.root.agroup._v_attrs._f_list("sys") == \
-               ['CLASS','FILTERS', 'TITLE','VERSION']
+               ['CLASS','FILTERS', 'TITLE', 'VERSION']
         assert self.root.agroup._v_attrs._f_list("all") == \
                ['CLASS','FILTERS', 'TITLE', 'VERSION', "pq", "qr", "rs"]
 
         assert self.root.atable.attrs._f_list() == ["a", "b", "c"]
         assert self.root.atable.attrs._f_list("sys") == \
-               ['CLASS', 'FIELD_0_NAME', 'FIELD_1_NAME', 'FIELD_2_NAME',
-                'FIELD_3_NAME', 'FIELD_4_NAME', 'NROWS', 'TITLE',
-                'VERSION']
-        assert self.root.atable.attrs._f_list("readonly") == \
-               ['CLASS', 'NROWS', 'VERSION']
+               ['AUTOMATIC_INDEX', 'CLASS', 'FIELD_0_NAME', 'FIELD_1_NAME',
+                'FIELD_2_NAME', 'FIELD_3_NAME', 'FIELD_4_NAME',
+                'FILTERS_INDEX', 'NROWS', 'REINDEX', 'TITLE', 'VERSION']
+#         assert self.root.atable.attrs._f_list("readonly") == \
+#                ['CLASS', 'NROWS', 'VERSION']
         assert self.root.atable.attrs._f_list("all") == \
-               ['CLASS', 'FIELD_0_NAME', 'FIELD_1_NAME', 'FIELD_2_NAME',
-                'FIELD_3_NAME', 'FIELD_4_NAME', 'NROWS', 'TITLE',
-                'VERSION', "a", "b", "c"]
-	    
+               ['AUTOMATIC_INDEX', 'CLASS', 'FIELD_0_NAME', 'FIELD_1_NAME',
+                'FIELD_2_NAME', 'FIELD_3_NAME', 'FIELD_4_NAME',
+                'FILTERS_INDEX', 'NROWS', 'REINDEX', 'TITLE', 'VERSION',
+                "a", "b", "c"]
+
         assert self.root.anarray.attrs._f_list() == ["i", "j", "k"]
         assert self.root.anarray.attrs._f_list("sys") == \
                ['CLASS', 'FLAVOR', 'TITLE', 'VERSION']
@@ -434,14 +459,14 @@ class createAttrTestCase(unittest.TestCase):
         self.group._v_attrs.rs = "3"
         # delete an attribute
         del self.group._v_attrs.pq
-        
+
         if self.close:
             if verbose:
                 print "(closing file version)"
             self.fileh.close()
             self.fileh = openFile(self.file, mode = "r+")
             self.root = self.fileh.root
-            
+
         if verbose:
             print "Attribute list:", self.root.agroup._v_attrs._f_list()
         # Check the local attributes names
@@ -467,22 +492,22 @@ class createAttrTestCase(unittest.TestCase):
                ['CLASS', 'FILTERS', 'TITLE', 'VERSION', "rs"]
 
     def test05b_removeAttributes(self):
-        """Checking removing attributes (using File.delAttrNode()) """
+        """Checking removing attributes (using File.delNodeAttr()) """
 
         # With a Group object
         self.group._v_attrs.pq = "1"
         self.group._v_attrs.qr = "2"
         self.group._v_attrs.rs = "3"
         # delete an attribute
-        self.fileh.delAttrNode(self.group, "pq")
-        
+        self.fileh.delNodeAttr(self.group, "pq")
+
         if self.close:
             if verbose:
                 print "(closing file version)"
             self.fileh.close()
             self.fileh = openFile(self.file, mode = "r+")
             self.root = self.fileh.root
-            
+
         if verbose:
             print "Attribute list:", self.root.agroup._v_attrs._f_list()
         # Check the local attributes names
@@ -494,8 +519,8 @@ class createAttrTestCase(unittest.TestCase):
         assert self.root.agroup._v_attrs._f_list("all") == \
                ['CLASS', 'FILTERS', 'TITLE', 'VERSION', "qr", "rs"]
 
-        # delete an attribute (File.delAttrNode method)
-        self.fileh.delAttrNode(self.root, "qr", "agroup")
+        # delete an attribute (File.delNodeAttr method)
+        self.fileh.delNodeAttr(self.root, "qr", "agroup")
         if verbose:
             print "Attribute list:", self.root.agroup._v_attrs._f_list()
         # Check the local attributes names
@@ -514,14 +539,15 @@ class createAttrTestCase(unittest.TestCase):
         try:
             if verbose:
                 print "System attrs:", self.group._v_attrs._v_attrnamessys
+                print "local dict:", self.group._v_attrs.__dict__
             del self.group._v_attrs.CLASS
-        except RuntimeError:
+        except AttributeError:
             if verbose:
                 (type, value, traceback) = sys.exc_info()
-                print "\nGreat!, the next RuntimeError was catched!"
+                print "\nGreat!, the next AttributeError was catched!"
                 print value
         else:
-            self.fail("expected a RuntimeError")
+            self.fail("expected a AttributeError")
 
     def test07_renameAttributes(self):
         """Checking renaming attributes """
@@ -539,7 +565,7 @@ class createAttrTestCase(unittest.TestCase):
             self.fileh.close()
             self.fileh = openFile(self.file, mode = "r+")
             self.root = self.fileh.root
-            
+
         if verbose:
             print "Attribute list:", self.root.agroup._v_attrs._f_list()
         # Check the local attributes names (alphabetically sorted)
@@ -556,13 +582,13 @@ class createAttrTestCase(unittest.TestCase):
         # rename a system attribute
         try:
             self.group._v_attrs._f_rename("CLASS", "op")
-        except RuntimeError:
+        except AttributeError:
             if verbose:
                 (type, value, traceback) = sys.exc_info()
-                print "\nGreat!, the next RuntimeError was catched!"
+                print "\nGreat!, the next AttributeError was catched!"
                 print value
         else:
-            self.fail("expected a RuntimeError")
+            self.fail("expected a AttributeError")
 
     def test09_setIntAttributes(self):
         """Checking setting Int attributes"""
@@ -577,14 +603,14 @@ class createAttrTestCase(unittest.TestCase):
             print "pq -->", self.table.attrs.pq
             print "qr -->", self.table.attrs.qr
             print "rs -->", self.table.attrs.rs
-            
+
         if self.close:
             if verbose:
                 print "(closing file version)"
             self.fileh.close()
             self.fileh = openFile(self.file, mode = "r+")
             self.root = self.fileh.root
-            
+
         assert self.root.atable.attrs.pq == 1
         assert self.root.atable.attrs.qr == 2
         assert self.root.atable.attrs.rs == 3
@@ -602,7 +628,7 @@ class createAttrTestCase(unittest.TestCase):
         self.table.attrs._g_setAttrShort("qr", 2)
         self.table.attrs._v_attrnames.append("qr")
         self.table.attrs._v_attrnamesuser.append("qr")
-        
+
         self.table.attrs._g_setAttrInt("rs", 3)
         self.table.attrs._v_attrnames.append("rs")
         self.table.attrs._v_attrnamesuser.append("rs")
@@ -612,7 +638,7 @@ class createAttrTestCase(unittest.TestCase):
             print "pq -->", self.table.attrs.pq
             print "qr -->", self.table.attrs.qr
             print "rs -->", self.table.attrs.rs
-            
+
         if self.close:
             if verbose:
                 print "(closing file version)"
@@ -637,7 +663,7 @@ class createAttrTestCase(unittest.TestCase):
             print "pq -->", self.table.attrs.pq
             print "qr -->", self.table.attrs.qr
             print "rs -->", self.table.attrs.rs
-            
+
         if self.close:
             if verbose:
                 print "(closing file version)"
@@ -662,7 +688,7 @@ class createAttrTestCase(unittest.TestCase):
         self.table.attrs._g_setAttrFloat("qr", 2.0)
         self.table.attrs._v_attrnames.append("qr")
         self.table.attrs._v_attrnamesuser.append("qr")
-        
+
         self.table.attrs._g_setAttrFloat("rs", 3.0)
         self.table.attrs._v_attrnames.append("rs")
         self.table.attrs._v_attrnamesuser.append("rs")
@@ -672,7 +698,7 @@ class createAttrTestCase(unittest.TestCase):
             print "pq -->", self.table.attrs.pq
             print "qr -->", self.table.attrs.qr
             print "rs -->", self.table.attrs.rs
-            
+
         if self.close:
             if verbose:
                 print "(closing file version)"
@@ -697,7 +723,7 @@ class createAttrTestCase(unittest.TestCase):
             print "pq -->", self.table.attrs.pq
             print "qr -->", self.table.attrs.qr
             print "rs -->", self.table.attrs.rs
-            
+
         if self.close:
             if verbose:
                 print "(closing file version)"
@@ -705,7 +731,7 @@ class createAttrTestCase(unittest.TestCase):
             self.fileh = openFile(self.file, mode = "r+")
             self.root = self.fileh.root
 
-        assert self.root.atable.attrs.pq == [1.0, 2]             
+        assert self.root.atable.attrs.pq == [1.0, 2]
         assert self.root.atable.attrs.qr == (1,2)
         assert self.root.atable.attrs.rs == {"ddf":32.1, "dsd":1}
 
@@ -727,7 +753,7 @@ class createAttrTestCase(unittest.TestCase):
             self.fileh.close()
             self.fileh = openFile(self.file, mode = "r+")
             self.root = self.fileh.root
-            
+
         if verbose:
             print "Value of Attribute pq:", self.root.agroup._v_attrs.pq
         # Check the local attributes names (alphabetically sorted)
@@ -763,7 +789,7 @@ class FiltersTreeTestCase(unittest.TestCase):
         # Create an instance of HDF5 Table
         self.h5file = openFile(self.file, "w", filters=self.filters)
         self.populateFile()
-            
+
     def populateFile(self):
         group = self.h5file.root
         # Create a tree with three levels of depth
@@ -773,16 +799,16 @@ class FiltersTreeTestCase(unittest.TestCase):
                                         title = self.title,
                                         filters = None)
             # Get the record object associated with the new table
-            d = table.row 
+            d = table.row
             # Fill the table
             for i in xrange(self.nrows):
                 d['var1'] = '%04d' % (self.nrows - i)
-                d['var2'] = i 
+                d['var2'] = i
                 d['var3'] = i * 2
                 d.append()      # This injects the Record values
             # Flush the buffer for this table
             table.flush()
-            
+
             # Create a couple of arrays in each group
             var1List = [ x['var1'] for x in table.iterrows() ]
             var3List = [ x['var3'] for x in table.iterrows() ]
@@ -799,7 +825,7 @@ class FiltersTreeTestCase(unittest.TestCase):
             # And fill them with some values
             ea1.append(var1List)
             ea2.append(var3List)
-            
+
             # Create a new group (descendant of group)
             if j == 1: # The second level
                 group2 = self.h5file.createGroup(group, 'group'+str(j),
@@ -811,13 +837,14 @@ class FiltersTreeTestCase(unittest.TestCase):
                                                  filters=self.filters)
             # Iterate over this new group (group2)
             group = group2
-    
+
     def tearDown(self):
         # Close the file
         if self.h5file.isopen:
             self.h5file.close()
 
         os.remove(self.file)
+        cleanup(self)
 
     #----------------------------------------
 
@@ -1045,6 +1072,10 @@ class FiltersCase8(FiltersTreeTestCase):
     filters = None
     gfilters = None
 
+class FiltersCase9(FiltersTreeTestCase):
+    filters = Filters(shuffle=1, complib="zlib")
+    gfilters = Filters(complevel=5, shuffle=1, complib="bzip2")
+
 class CopyGroupTestCase(unittest.TestCase):
     title = "A title"
     nrows = 10
@@ -1128,6 +1159,7 @@ class CopyGroupTestCase(unittest.TestCase):
 
         os.remove(self.file)
         os.remove(self.file2)
+        cleanup(self)
 
     #----------------------------------------
 
@@ -1219,7 +1251,7 @@ class CopyGroupTestCase(unittest.TestCase):
                     assert str(filters) == dstattrvalue
                 else:
                     assert srcattrvalue == dstattrvalue
-                    
+
             if verbose:
                 print "The attrs contents has been copied correctly"
 
@@ -1378,25 +1410,25 @@ class CopyGroupCase7(CopyGroupTestCase):
     filters = Filters(complevel=1, shuffle=0)
     srcnode = '/'
     dstnode = '/'
-        
+
 class CopyGroupCase8(CopyGroupTestCase):
     close = 1
     filters = Filters(complevel=1, complib="lzo")
     srcnode = '/'
     dstnode = '/'
-        
+
 class CopyFileTestCase(unittest.TestCase):
     title = "A title"
     nrows = 10
 
     def setUp(self):
         # Create a temporary file
-        self.file = tempfile.mktemp(".h5") 
-        self.file2 = tempfile.mktemp(".h5") 
+        self.file = tempfile.mktemp(".h5")
+        self.file2 = tempfile.mktemp(".h5")
         # Create the source file
         self.h5file = openFile(self.file, "w")
         self.populateFile()
-            
+
     def populateFile(self):
         group = self.h5file.root
         # Add some user attrs:
@@ -1414,11 +1446,11 @@ class CopyFileTestCase(unittest.TestCase):
                                             title = self.title,
                                             filters = None)
                 # Get the record object associated with the new table
-                d = table.row 
+                d = table.row
                 # Fill the table
                 for i in xrange(self.nrows):
                     d['var1'] = '%04d' % (self.nrows - i)
-                    d['var2'] = i 
+                    d['var2'] = i
                     d['var3'] = i * 2
                     d.append()      # This injects the Record values
                 # Flush the buffer for this table
@@ -1467,6 +1499,7 @@ class CopyFileTestCase(unittest.TestCase):
 
         os.remove(self.file)
         os.remove(self.file2)
+        cleanup(self)
 
     #----------------------------------------
 
@@ -1518,11 +1551,11 @@ class CopyFileTestCase(unittest.TestCase):
 
         # Close the temporary file
         self.h5file.close()
-        
+
         # Copy the file to the destination
         copyFile(self.file, self.file2, title=self.title,
                  copyuserattrs = 0, filters = None, overwrite = 1)
-        
+
         # ...and open the source and destination file
         self.h5file = openFile(self.file, "r")
         self.h5file2 = openFile(self.file2, "r")
@@ -1548,7 +1581,6 @@ class CopyFileTestCase(unittest.TestCase):
             print '\n', '-=' * 30
             print "Running %s.test01_copy..." % self.__class__.__name__
 
-
         # Copy the file to the destination
         self.h5file.copyFile(self.file2, title=self.title,
                              copyuserattrs = 0,
@@ -1559,7 +1591,7 @@ class CopyFileTestCase(unittest.TestCase):
             self.h5file.close()
             # re-open it
             self.h5file = openFile(self.file, "r")
-        
+
         # ...and open the destination file
         self.h5file2 = openFile(self.file2, "r")
 
@@ -1575,21 +1607,17 @@ class CopyFileTestCase(unittest.TestCase):
             print "The copied node list -->", nodelist2
         assert srcgroup._v_nchildren == dstgroup._v_nchildren
         assert nodelist1 == nodelist2
+        #print "_v_attrnames-->", self.h5file2.root._v_attrs._v_attrnames
+        #print "--> <%s,%s>" % (self.h5file2.title, self.title)
         assert self.h5file2.title == self.title
 
         # Check that user attributes has not been copied
         for srcnode in srcgroup:
             dstnode = getattr(dstgroup, srcnode._v_name)
-            if isinstance(srcnode, Group):
-                srcattrs = srcnode._v_attrs
-                srcattrskeys = srcattrs._f_list("sys")
-                dstattrs = dstnode._v_attrs
-                dstattrskeys = dstattrs._f_list("all")
-            else:
-                srcattrs = srcnode.attrs
-                srcattrskeys = srcattrs._f_list("sys")
-                dstattrs = dstnode.attrs
-                dstattrskeys = dstattrs._f_list("all")
+            srcattrs = srcnode._v_attrs
+            srcattrskeys = srcattrs._f_list("sys")
+            dstattrs = dstnode._v_attrs
+            dstattrskeys = dstattrs._f_list("all")
             # These lists should already be ordered
             if verbose:
                 print "srcattrskeys for node %s: %s" %(srcnode._v_name,
@@ -1609,13 +1637,10 @@ class CopyFileTestCase(unittest.TestCase):
                         filters = Filters()
                     else:
                         filters = self.filters
-                    if str(filters) <> dstattrvalue:
-                        print "filters-->", str(filters)
-                        print "content-_>", dstattrvalue
                     assert str(filters) == dstattrvalue
                 else:
                     assert srcattrvalue == dstattrvalue
-                    
+
             if verbose:
                 print "The attrs contents has been copied correctly"
 
@@ -1638,7 +1663,7 @@ class CopyFileTestCase(unittest.TestCase):
             self.h5file.close()
             # re-open it
             self.h5file = openFile(self.file, "r")
-        
+
         # ...and open the destination file
         self.h5file2 = openFile(self.file2, "r")
 
@@ -1647,16 +1672,10 @@ class CopyFileTestCase(unittest.TestCase):
         dstgroup = self.h5file2.root
         for srcnode in srcgroup:
             dstnode = getattr(dstgroup, srcnode._v_name)
-            if isinstance(srcnode, Group):
-                srcattrs = srcnode._v_attrs
-                srcattrskeys = srcattrs._f_list("all")
-                dstattrs = dstnode._v_attrs
-                dstattrskeys = dstattrs._f_list("all")
-            else:
-                srcattrs = srcnode.attrs
-                srcattrskeys = srcattrs._f_list("all")
-                dstattrs = dstnode.attrs
-                dstattrskeys = dstattrs._f_list("all")
+            srcattrs = srcnode._v_attrs
+            srcattrskeys = srcattrs._f_list("all")
+            dstattrs = dstnode._v_attrs
+            dstattrskeys = dstattrs._f_list("all")
             # These lists should already be ordered
             if verbose:
                 print "srcattrskeys for node %s: %s" %(srcnode._v_name,
@@ -1679,7 +1698,7 @@ class CopyFileTestCase(unittest.TestCase):
                     assert str(filters) == dstattrvalue
                 else:
                     assert srcattrvalue == dstattrvalue
-                    
+
             if verbose:
                 print "The attrs contents has been copied correctly"
 
@@ -1749,26 +1768,28 @@ class CopyFileCase10(unittest.TestCase):
                 print value
         else:
             self.fail("expected a IOError")
-            
+
 
         # Delete files
         fileh.close()
         os.remove(file)
         os.remove(file2)
-	
-	
+
+
 #----------------------------------------------------------------------
 
 def suite():
     theSuite = unittest.TestSuite()
     niter = 1
     #heavy = 1 # Uncomment this only for testing purposes!
-    
+
+    #theSuite.addTest(unittest.makeSuite(createAttrNotCloseTestCase))
+    #theSuite.addTest(unittest.makeSuite(FiltersCase1))
     #theSuite.addTest(unittest.makeSuite(createTestCase))
     #theSuite.addTest(unittest.makeSuite(CopyGroupCase1))
     #theSuite.addTest(unittest.makeSuite(CopyGroupCase2))
+    #theSuite.addTest(unittest.makeSuite(CopyFileCase1))
     for i in range(niter):
-        theSuite.addTest(unittest.makeSuite(createTestCase))
         theSuite.addTest(unittest.makeSuite(createAttrNotCloseTestCase))
         theSuite.addTest(unittest.makeSuite(createAttrCloseTestCase))
         theSuite.addTest(unittest.makeSuite(FiltersCase1))
@@ -1778,12 +1799,14 @@ def suite():
         theSuite.addTest(unittest.makeSuite(CopyFileCase1))
         theSuite.addTest(unittest.makeSuite(CopyFileCase2))
     if heavy:
+        theSuite.addTest(unittest.makeSuite(createTestCase))
         theSuite.addTest(unittest.makeSuite(FiltersCase3))
         theSuite.addTest(unittest.makeSuite(FiltersCase4))
         theSuite.addTest(unittest.makeSuite(FiltersCase5))
         theSuite.addTest(unittest.makeSuite(FiltersCase6))
         theSuite.addTest(unittest.makeSuite(FiltersCase7))
         theSuite.addTest(unittest.makeSuite(FiltersCase8))
+        theSuite.addTest(unittest.makeSuite(FiltersCase9))
         theSuite.addTest(unittest.makeSuite(CopyFileCase3))
         theSuite.addTest(unittest.makeSuite(CopyFileCase4))
         theSuite.addTest(unittest.makeSuite(CopyFileCase5))
