@@ -16,7 +16,9 @@ try:
 except:
     numeric = 0
 
-from test_all import verbose, allequal
+from test_all import verbose, allequal, cleanup, heavy
+# To delete the internal attributes automagically
+unittest.TestCase.tearDown = cleanup
 
 class BasicTestCase(unittest.TestCase):
     # Default values
@@ -45,7 +47,7 @@ class BasicTestCase(unittest.TestCase):
         if self.reopen:
             # Close the file
             self.fileh.close()
-        
+
     def populateFile(self):
         group = self.rootgroup
         if self.flavor == "numarray":
@@ -107,7 +109,8 @@ class BasicTestCase(unittest.TestCase):
     def tearDown(self):
         self.fileh.close()
         os.remove(self.file)
-        
+        cleanup(self)
+
     #----------------------------------------
 
     def test01_iterEArray(self):
@@ -129,7 +132,7 @@ class BasicTestCase(unittest.TestCase):
             print "EArray descr:", repr(earray)
             print "shape of read array ==>", earray.shape
             print "reopening?:", self.reopen
-            
+
         # Build the array to do comparisons
         if self.flavor == "numarray":
             if str(self.type) == "CharType":
@@ -144,7 +147,7 @@ class BasicTestCase(unittest.TestCase):
                                      typecode=typecode[earray.type])
             object_ = Numeric.reshape(object_, self.rowshape)
             object_ = Numeric.swapaxes(object_, earray.extdim, 0)
-            
+
         # Read all the array
         for row in earray:
             chunk = int(earray.nrow % self.chunksize)
@@ -177,7 +180,7 @@ class BasicTestCase(unittest.TestCase):
                 print "Error in compress. Class:", self.__class__.__name__
                 print "self, earray:", self.compress, earray.filters.complevel
             tinfo = whichLibVersion(self.complib)
-            if tinfo[0] == 0:
+            if tinfo is None:
                 self.complib = "zlib"
             assert earray.filters.complib == self.complib
             assert earray.filters.complevel == self.compress
@@ -209,7 +212,7 @@ class BasicTestCase(unittest.TestCase):
             print "EArray descr:", repr(earray)
             print "shape of read array ==>", earray.shape
             print "reopening?:", self.reopen
-            
+
         # Build the array to do comparisons
         if self.flavor == "numarray":
             if str(self.type) == "CharType":
@@ -224,7 +227,7 @@ class BasicTestCase(unittest.TestCase):
                                      typecode=typecode[earray.type])
             object_ = Numeric.reshape(object_, self.rowshape)
             object_ = Numeric.swapaxes(object_, earray.extdim, 0)
-            
+
         # Read all the array
         for row in earray.iterrows(start=self.start, stop=self.stop, step=self.step):
             if self.chunksize == 1:
@@ -261,7 +264,7 @@ class BasicTestCase(unittest.TestCase):
         if verbose:
             print '\n', '-=' * 30
             print "Running %s.test03_readEArray..." % self.__class__.__name__
-            
+
         # Create an instance of an HDF5 Table
         if self.reopen:
             self.fileh = openFile(self.file, "r")
@@ -273,7 +276,7 @@ class BasicTestCase(unittest.TestCase):
             print "EArray descr:", repr(earray)
             print "shape of read array ==>", earray.shape
             print "reopening?:", self.reopen
-            
+
         # Build the array to do comparisons
         if self.flavor == "numarray":
             if str(self.type) == "CharType":
@@ -288,7 +291,7 @@ class BasicTestCase(unittest.TestCase):
                                      typecode=typecode[earray.type])
             object_ = Numeric.reshape(object_, self.rowshape)
             object_ = Numeric.swapaxes(object_, earray.extdim, 0)
-            
+
         rowshape = self.rowshape
         rowshape[self.extdim] *= self.nappends
         if self.flavor == "numarray":
@@ -354,7 +357,7 @@ class BasicTestCase(unittest.TestCase):
                 print "shape should look as:", object.shape
             print "Object read ==>", repr(row)
             print "Should look like ==>", repr(object)
-            
+
         assert self.nappends*self.chunksize == earray.nrows
         assert allequal(row, object, self.flavor)
         if hasattr(row, "shape"):
@@ -374,7 +377,7 @@ class BasicTestCase(unittest.TestCase):
         if not hasattr(self, "slices"):
             # If there is not a slices attribute, create it
             self.slices = (slice(self.start, self.stop, self.step),)
-            
+
         # Create an instance of an HDF5 Table
         if self.reopen:
             self.fileh = openFile(self.file, "r")
@@ -395,7 +398,7 @@ class BasicTestCase(unittest.TestCase):
             object_ = arange(self.objsize, shape=self.rowshape,
                              type=earray.type)
         object_.swapaxes(earray.extdim, 0)
-            
+
         rowshape = self.rowshape
         rowshape[self.extdim] *= self.nappends
         if str(self.type) == "CharType":
@@ -459,7 +462,7 @@ class BasicTestCase(unittest.TestCase):
         if self.__class__.__name__ == "Ellipsis6EArrayTestCase":
             # We have a problem with test design here, but I think
             # it is not worth the effort to solve it
-            # F.Alted 2004-10-27
+            # F.Altet 2004-10-27
             return
         if verbose:
             print '\n', '-=' * 30
@@ -468,7 +471,7 @@ class BasicTestCase(unittest.TestCase):
         if not hasattr(self, "slices"):
             # If there is not a slices attribute, create it
             self.slices = (slice(self.start, self.stop, self.step),)
-            
+
         # Create an instance of an HDF5 Table
         if self.reopen:
             self.fileh = openFile(self.file, "a")
@@ -489,7 +492,7 @@ class BasicTestCase(unittest.TestCase):
             object_ = arange(self.objsize, shape=self.rowshape,
                              type=earray.type)
         object_.swapaxes(earray.extdim, 0)
-            
+
         rowshape = self.rowshape
         rowshape[self.extdim] *= self.nappends
         if str(self.type) == "CharType":
@@ -582,7 +585,7 @@ class BasicWrite2TestCase(BasicTestCase):
     step = 1
     wslice = slice(chunksize-2,nappends,2)  # range of elements
     reopen = 0  # This case does not reopen files
-    
+
 class EmptyEArrayTestCase(BasicTestCase):
     type = Int32
     shape = (2, 0)
@@ -611,14 +614,6 @@ class SlicesEArrayTestCase(BasicTestCase):
     nappends = 2
     slices = (slice(1,2,1), slice(1,3,1))
 
-class EllipsisEArrayTestCase(BasicTestCase):
-    type = Int32
-    shape = (2, 0)
-    chunksize = 5
-    nappends = 2
-    #slices = (slice(1,2,1), Ellipsis)
-    slices = (Ellipsis, slice(1,2,1))
-
 class Slices2EArrayTestCase(BasicTestCase):
     compress = 1
     complib = "lzo"
@@ -628,6 +623,14 @@ class Slices2EArrayTestCase(BasicTestCase):
     nappends = 20
     slices = (slice(1,2,1), slice(None, None, None), slice(1,4,2))
 
+class EllipsisEArrayTestCase(BasicTestCase):
+    type = Int32
+    shape = (2, 0)
+    chunksize = 5
+    nappends = 2
+    #slices = (slice(1,2,1), Ellipsis)
+    slices = (Ellipsis, slice(1,2,1))
+
 class Ellipsis2EArrayTestCase(BasicTestCase):
     type = Int32
     shape = (2, 0, 4)
@@ -809,6 +812,25 @@ class UCLShuffleTestCase(BasicTestCase):
     stop = 10
     step = 6
 
+class BZIP2ComprTestCase(BasicTestCase):
+    compress = 1
+    complib = "bzip2"
+    chunksize = 100
+    nappends = 10
+    start = 3
+    stop = 10
+    step = 8
+
+class BZIP2ShuffleTestCase(BasicTestCase):
+    compress = 1
+    shuffle = 1
+    complib = "bzip2"
+    chunksize = 100
+    nappends = 10
+    start = 3
+    stop = 10
+    step = 6
+
 class Fletcher32TestCase(BasicTestCase):
     compress = 0
     fletcher32 = 1
@@ -826,7 +848,7 @@ class AllFiltersTestCase(BasicTestCase):
     chunksize = 20  # sss
     nappends = 50
     start = 2
-    stop = 99 
+    stop = 99
     step = 6
 #     chunksize = 3
 #     nappends = 2
@@ -933,7 +955,7 @@ class NumericComprTestCase(BasicTestCase):
 # It remains a test of Numeric char types, but the code is getting too messy
 
 class OffsetStrideTestCase(unittest.TestCase):
-    mode  = "w" 
+    mode  = "w"
     compress = 0
     complib = "zlib"  # Default compression library
 
@@ -947,7 +969,8 @@ class OffsetStrideTestCase(unittest.TestCase):
     def tearDown(self):
         self.fileh.close()
         os.remove(self.file)
-        
+        cleanup(self)
+
     #----------------------------------------
 
     def test01a_String(self):
@@ -974,7 +997,7 @@ class OffsetStrideTestCase(unittest.TestCase):
             print "Object read:", row
             print "Nrows in", earray._v_pathname, ":", earray.nrows
             print "Second row in earray ==>", row[1].tolist()
-            
+
         assert earray.nrows == 2
         assert row[0].tolist() == [["123", "45"],["45", "123"]]
         assert row[1].tolist() == [["s", "abc"],["abc", "f"]]
@@ -1005,7 +1028,7 @@ class OffsetStrideTestCase(unittest.TestCase):
             print "Object read:", row
             print "Nrows in", earray._v_pathname, ":", earray.nrows
             print "Second row in earray ==>", row[1].tolist()
-            
+
         assert earray.nrows == 2
         assert row[0].tolist() == [["a","b"],["45", "123"]]
         assert row[1].tolist() == [["s", "a"],["s", "abc"]]
@@ -1035,7 +1058,7 @@ class OffsetStrideTestCase(unittest.TestCase):
             print "Object read:", row
             print "Nrows in", earray._v_pathname, ":", earray.nrows
             print "Third row in vlarray ==>", row[2]
-            
+
         assert earray.nrows == 3
         assert allequal(row[0], array([1,1,1], type=Int32))
         assert allequal(row[1], array([0,0,0], type=Int32))
@@ -1064,7 +1087,7 @@ class OffsetStrideTestCase(unittest.TestCase):
             print "Object read:", row
             print "Nrows in", earray._v_pathname, ":", earray.nrows
             print "Third row in vlarray ==>", row[2]
-            
+
         assert earray.nrows == 3
         assert allequal(row[0], array([0,0,0], type=Int32))
         assert allequal(row[1], array([3,3,3], type=Int32))
@@ -1094,9 +1117,9 @@ class CopyTestCase(unittest.TestCase):
             fileh.close()
             fileh = openFile(file, mode = "a")
             array1 = fileh.root.array1
-            
+
         # Copy it to another location
-        array2, size = array1.copy('/', 'array2')
+        array2 = array1.copy('/', 'array2')
 
         if self.close:
             if verbose:
@@ -1112,7 +1135,7 @@ class CopyTestCase(unittest.TestCase):
             #print "dirs-->", dir(array1), dir(array2)
             print "attrs array1-->", repr(array1.attrs)
             print "attrs array2-->", repr(array2.attrs)
-            
+
         # Check that all the elements are equal
         assert allequal(array1.read(), array2.read())
 
@@ -1152,10 +1175,10 @@ class CopyTestCase(unittest.TestCase):
             fileh.close()
             fileh = openFile(file, mode = "a")
             array1 = fileh.root.array1
-            
+
         # Copy to another location
         group1 = fileh.createGroup("/", "group1")
-        array2, size = array1.copy(group1, 'array2')
+        array2 = array1.copy(group1, 'array2')
 
         if self.close:
             if verbose:
@@ -1171,7 +1194,7 @@ class CopyTestCase(unittest.TestCase):
             #print "dirs-->", dir(array1), dir(array2)
             print "attrs array1-->", repr(array1.attrs)
             print "attrs array2-->", repr(array2.attrs)
-            
+
         # Check that all the elements are equal
         assert allequal(array1.read(), array2.read())
 
@@ -1214,9 +1237,9 @@ class CopyTestCase(unittest.TestCase):
             fileh.close()
             fileh = openFile(file, mode = "a")
             array1 = fileh.root.array1
-            
+
         # Copy to another location
-        array2, size = array1.copy('/', 'array2')
+        array2 = array1.copy('/', 'array2')
 
         if self.close:
             if verbose:
@@ -1229,7 +1252,7 @@ class CopyTestCase(unittest.TestCase):
         if verbose:
             print "attrs array1-->", repr(array1.attrs)
             print "attrs array2-->", repr(array2.attrs)
-            
+
         # Assert other properties in array
         assert array1.nrows == array2.nrows
         assert array1.shape == array2.shape
@@ -1265,9 +1288,9 @@ class CopyTestCase(unittest.TestCase):
             fileh.close()
             fileh = openFile(file, mode = "a")
             array1 = fileh.root.array1
-            
+
         # Copy to another location
-        array2, size = array1.copy('/', 'array2')
+        array2 = array1.copy('/', 'array2')
 
         if self.close:
             if verbose:
@@ -1318,9 +1341,9 @@ class CopyTestCase(unittest.TestCase):
             fileh.close()
             fileh = openFile(file, mode = "a")
             array1 = fileh.root.array1
-            
+
         # Copy to another location
-        array2, size = array1.copy('/', 'array2')
+        array2 = array1.copy('/', 'array2')
 
         if self.close:
             if verbose:
@@ -1371,9 +1394,9 @@ class CopyTestCase(unittest.TestCase):
             fileh.close()
             fileh = openFile(file, mode = "a")
             array1 = fileh.root.array1
-            
+
         # Copy to another location
-        array2, size = array1.copy('/', 'array2')
+        array2 = array1.copy('/', 'array2')
 
         if self.close:
             if verbose:
@@ -1389,7 +1412,7 @@ class CopyTestCase(unittest.TestCase):
 
         # Check that all elements are equal
         assert array1.read() == array2.read()
-        
+
         # Assert other properties in array
         assert array1.nrows == array2.nrows
         assert array1.shape == array2.shape
@@ -1425,9 +1448,9 @@ class CopyTestCase(unittest.TestCase):
             fileh.close()
             fileh = openFile(file, mode = "a")
             array1 = fileh.root.array1
-            
+
         # Copy to another location
-        array2, size = array1.copy('/', 'array2')
+        array2 = array1.copy('/', 'array2')
 
         if self.close:
             if verbose:
@@ -1482,9 +1505,9 @@ class CopyTestCase(unittest.TestCase):
             fileh.close()
             fileh = openFile(file, mode = "a")
             array1 = fileh.root.array1
-            
+
         # Copy it to another Array
-        array2, size = array1.copy('/', 'array2', title="title array2")
+        array2 = array1.copy('/', 'array2', title="title array2")
 
         if self.close:
             if verbose:
@@ -1493,7 +1516,7 @@ class CopyTestCase(unittest.TestCase):
             fileh = openFile(file, mode = "r")
             array1 = fileh.root.array1
             array2 = fileh.root.array2
-            
+
         # Assert user attributes
         if verbose:
             print "title of destination array-->", array2.title
@@ -1528,9 +1551,9 @@ class CopyTestCase(unittest.TestCase):
             fileh.close()
             fileh = openFile(file, mode = "a")
             array1 = fileh.root.array1
-            
+
         # Copy it to another Array
-        array2, size = array1.copy('/', 'array2', copyuserattrs=1)
+        array2 = array1.copy('/', 'array2', copyuserattrs=1)
 
         if self.close:
             if verbose:
@@ -1543,7 +1566,7 @@ class CopyTestCase(unittest.TestCase):
         if verbose:
             print "attrs array1-->", repr(array1.attrs)
             print "attrs array2-->", repr(array2.attrs)
-            
+
         # Assert user attributes
         array2.attrs.attr1 == "attr1"
         array2.attrs.attr2 == 2
@@ -1577,9 +1600,9 @@ class CopyTestCase(unittest.TestCase):
             fileh.close()
             fileh = openFile(file, mode = "a")
             array1 = fileh.root.array1
-            
+
         # Copy it to another Array
-        array2, size = array1.copy('/', 'array2', copyuserattrs=0)
+        array2 = array1.copy('/', 'array2', copyuserattrs=0)
 
         if self.close:
             if verbose:
@@ -1592,7 +1615,7 @@ class CopyTestCase(unittest.TestCase):
         if verbose:
             print "attrs array1-->", repr(array1.attrs)
             print "attrs array2-->", repr(array2.attrs)
-            
+
         # Assert user attributes
         hasattr(array2.attrs, "attr1") == 0
         hasattr(array2.attrs, "attr2") == 0
@@ -1627,12 +1650,12 @@ class CopyIndexTestCase(unittest.TestCase):
         array1 = fileh.createEArray(fileh.root, 'array1', atom, "title array1")
         r = arange(200, type=Int32, shape=(100,2))
         array1.append(r)
-        
+
         # Select a different buffer size:
         array1._v_maxTuples = self.maxTuples
-        
+
         # Copy to another array
-        array2, size = array1.copy("/", 'array2',
+        array2 = array1.copy("/", 'array2',
                              start=self.start,
                              stop=self.stop,
                              step=self.step)
@@ -1641,7 +1664,7 @@ class CopyIndexTestCase(unittest.TestCase):
             print "array2-->", array2.read()
             print "attrs array1-->", repr(array1.attrs)
             print "attrs array2-->", repr(array2.attrs)
-            
+
         # Check that all the elements are equal
         r2 = r[self.start:self.stop:self.step]
         assert allequal(r2, array2.read())
@@ -1677,7 +1700,7 @@ class CopyIndexTestCase(unittest.TestCase):
         array1._v_maxTuples = self.maxTuples
 
         # Copy to another array
-        array2, size = array1.copy("/", 'array2',
+        array2 = array1.copy("/", 'array2',
                              start=self.start,
                              stop=self.stop,
                              step=self.step)
@@ -1692,7 +1715,7 @@ class CopyIndexTestCase(unittest.TestCase):
             print "array2-->", array2.read()
             print "attrs array1-->", repr(array1.attrs)
             print "attrs array2-->", repr(array2.attrs)
-            
+
         # Check that all the elements are equal
         r2 = r[self.start:self.stop:self.step]
         assert allequal(r2, array2.read())
@@ -1713,12 +1736,12 @@ class CopyIndex1TestCase(CopyIndexTestCase):
     stop = 7
     step = 1
 
-class CopyIndex2TestCase(CopyIndexTestCase): 
+class CopyIndex2TestCase(CopyIndexTestCase):
     maxTuples = 2
     start = 0
     stop = -1
     step = 1
-	
+
 class CopyIndex3TestCase(CopyIndexTestCase):
     maxTuples = 3
     start = 1
@@ -1726,7 +1749,7 @@ class CopyIndex3TestCase(CopyIndexTestCase):
     step = 1
 
 class CopyIndex4TestCase(CopyIndexTestCase):
-    maxTuples = 4        
+    maxTuples = 4
     start = 0
     stop = 6
     step = 1
@@ -1797,9 +1820,9 @@ class TruncateTestCase(unittest.TestCase):
             fileh.close()
             fileh = openFile(file, mode = "a")
             array1 = fileh.root.array1
-            
+
         # Truncate to 0 elements
-        size = array1.truncate(0)
+        array1.truncate(0)
 
         if self.close:
             if verbose:
@@ -1810,11 +1833,13 @@ class TruncateTestCase(unittest.TestCase):
 
         if verbose:
             print "array1-->", array1.read()
-            print "size-->", size
 
-        assert size == 0
         assert allequal(array1[:], array([], type=Int16, shape=(0,2)))
 
+        # Close the file
+        fileh.close()
+        os.remove(file)
+
     def test01_truncate(self):
         """Checking EArray.truncate() method (truncating to 1 rows)"""
 
@@ -1836,9 +1861,9 @@ class TruncateTestCase(unittest.TestCase):
             fileh.close()
             fileh = openFile(file, mode = "a")
             array1 = fileh.root.array1
-            
+
         # Truncate to 1 element
-        size = array1.truncate(1)
+        array1.truncate(1)
 
         if self.close:
             if verbose:
@@ -1849,11 +1874,13 @@ class TruncateTestCase(unittest.TestCase):
 
         if verbose:
             print "array1-->", array1.read()
-            print "size-->", size
 
-        assert size == 1
         assert allequal(array1.read(), array([[456, 2]], type=Int16))
 
+        # Close the file
+        fileh.close()
+        os.remove(file)
+
     def test02_truncate(self):
         """Checking EArray.truncate() method (truncating to >= earray.nrows)"""
 
@@ -1875,9 +1902,9 @@ class TruncateTestCase(unittest.TestCase):
             fileh.close()
             fileh = openFile(file, mode = "a")
             array1 = fileh.root.array1
-            
+
         # Truncate to 2 elements
-        size = array1.truncate(2)
+        array1.truncate(2)
 
         if self.close:
             if verbose:
@@ -1888,11 +1915,13 @@ class TruncateTestCase(unittest.TestCase):
 
         if verbose:
             print "array1-->", array1.read()
-            print "size-->", size
 
-        assert size == 2
         assert allequal(array1.read(), array([[456, 2],[3, 457]], type=Int16))
 
+        # Close the file
+        fileh.close()
+        os.remove(file)
+
     def test03_truncate(self):
         """Checking EArray.truncate() method (truncating to > earray.nrows)"""
 
@@ -1914,9 +1943,9 @@ class TruncateTestCase(unittest.TestCase):
             fileh.close()
             fileh = openFile(file, mode = "a")
             array1 = fileh.root.array1
-            
+
         # Truncate to 3 element
-        size = array1.truncate(3)
+        array1.truncate(3)
 
         if self.close:
             if verbose:
@@ -1927,11 +1956,13 @@ class TruncateTestCase(unittest.TestCase):
 
         if verbose:
             print "array1-->", array1.read()
-            print "size-->", size
 
-        assert size == 2
         assert allequal(array1.read(), array([[456, 2],[3, 457]], type=Int16))
 
+        # Close the file
+        fileh.close()
+        os.remove(file)
+
 
 class TruncateOpenTestCase(TruncateTestCase):
     close = 0
@@ -1939,6 +1970,93 @@ class TruncateOpenTestCase(TruncateTestCase):
 class TruncateCloseTestCase(TruncateTestCase):
     close = 1
 
+# The next test should be run only in **heavy** mode
+class Rows64bitsTestCase(unittest.TestCase):
+    narows = 1000*1000L   # each numarray will have 1 million entries
+    #narows = 1000L   # for testing only
+    nanumber = 1000*3L    # That should account for more than 2**31-1
+
+    def setUp(self):
+
+        # Create an instance of an HDF5 Table
+        self.file = tempfile.mktemp(".h5")
+        fileh = self.fileh = openFile(self.file, "a")
+        # Create an EArray
+        array = fileh.createEArray(fileh.root, 'array',
+                                   Int8Atom((0,)),
+                                   filters=Filters(complib='lzo',
+                                                   complevel=1),
+                                   # Specifying expectedrows takes more
+                                   # CPU, but less disk
+                                   expectedrows=self.narows*self.nanumber)
+
+        # Fill the array
+        na = arange(self.narows, type='Int8')
+        for i in range(self.nanumber):
+            array.append(na)
+
+
+    def tearDown(self):
+        self.fileh.close()
+        os.remove(self.file)
+        cleanup(self)
+
+    #----------------------------------------
+
+    def test01_basiccheck(self):
+        "Some basic checks for earrays exceeding 2**31 rows"
+
+        fileh = self.fileh
+        array = fileh.root.array
+
+        if self.close:
+            if verbose:
+                # Check how many entries there are in the array
+                print "Before closing"
+                print "Entries:", array.nrows, type(array.nrows)
+                print "Entries:", array.nrows / (1000*1000), "Millions"
+                print "Shape:", array.shape
+            # Close the file
+            fileh.close()
+            # Re-open the file
+            fileh = self.fileh = openFile(self.file)
+            array = fileh.root.array
+            if verbose:
+                print "After re-open"
+
+        # Check how many entries there are in the array
+        if verbose:
+            print "Entries:", array.nrows, type(array.nrows)
+            print "Entries:", array.nrows / (1000*1000), "Millions"
+            print "Shape:", array.shape
+            print "Last 10 elements-->", array[-10:]
+            stop = self.narows%256
+            if stop > 127:
+                stop -= 256
+            start = stop - 10
+            #print "start, stop-->", start, stop
+            print "Should look like-->", arange(start, stop, type='Int8')
+
+        nrows = self.narows*self.nanumber
+        # check nrows
+        assert array.nrows == nrows
+        # Check shape
+        assert array.shape == (nrows,)
+        # check the 10 first elements
+        assert allequal(array[:10], arange(10, type='Int8'))
+        # check the 10 last elements
+        stop = self.narows%256
+        if stop > 127:
+            stop -= 256
+        start = stop - 10
+        assert allequal(array[-10:], arange(start, stop, type='Int8'))
+
+
+class Rows64bitsTestCase1(Rows64bitsTestCase):
+    close = 0
+
+class Rows64bitsTestCase2(Rows64bitsTestCase):
+    close = 1
 
 #----------------------------------------------------------------------
 
@@ -1946,10 +2064,11 @@ def suite():
     theSuite = unittest.TestSuite()
     global numeric
     niter = 1
+    #heavy = 1  # uncomment this only for testing purposes
 
     #theSuite.addTest(unittest.makeSuite(BasicWriteTestCase))
-#     theSuite.addTest(unittest.makeSuite(TruncateOpenTestCase))
-#     theSuite.addTest(unittest.makeSuite(TruncateCloseTestCase))
+    #theSuite.addTest(unittest.makeSuite(Rows64bitsTestCase1))
+    #theSuite.addTest(unittest.makeSuite(Rows64bitsTestCase2))
     for n in range(niter):
         theSuite.addTest(unittest.makeSuite(BasicWriteTestCase))
         theSuite.addTest(unittest.makeSuite(BasicWrite2TestCase))
@@ -1957,30 +2076,21 @@ def suite():
         theSuite.addTest(unittest.makeSuite(EmptyEArray2TestCase))
         theSuite.addTest(unittest.makeSuite(SlicesEArrayTestCase))
         theSuite.addTest(unittest.makeSuite(Slices2EArrayTestCase))
-        theSuite.addTest(unittest.makeSuite(Slices3EArrayTestCase))
-        theSuite.addTest(unittest.makeSuite(Slices4EArrayTestCase))
         theSuite.addTest(unittest.makeSuite(EllipsisEArrayTestCase))
         theSuite.addTest(unittest.makeSuite(Ellipsis2EArrayTestCase))
         theSuite.addTest(unittest.makeSuite(Ellipsis3EArrayTestCase))
-        theSuite.addTest(unittest.makeSuite(Ellipsis4EArrayTestCase))
-        theSuite.addTest(unittest.makeSuite(Ellipsis5EArrayTestCase))
-        theSuite.addTest(unittest.makeSuite(Ellipsis6EArrayTestCase))
-        theSuite.addTest(unittest.makeSuite(Ellipsis7EArrayTestCase))
-        theSuite.addTest(unittest.makeSuite(MD3WriteTestCase))
-        theSuite.addTest(unittest.makeSuite(MD5WriteTestCase))
-        theSuite.addTest(unittest.makeSuite(MD6WriteTestCase))
-        theSuite.addTest(unittest.makeSuite(MD7WriteTestCase))
-        theSuite.addTest(unittest.makeSuite(MD10WriteTestCase))
         theSuite.addTest(unittest.makeSuite(ZlibComprTestCase))
         theSuite.addTest(unittest.makeSuite(ZlibShuffleTestCase))
         theSuite.addTest(unittest.makeSuite(LZOComprTestCase))
         theSuite.addTest(unittest.makeSuite(LZOShuffleTestCase))
         theSuite.addTest(unittest.makeSuite(UCLComprTestCase))
         theSuite.addTest(unittest.makeSuite(UCLShuffleTestCase))
+        theSuite.addTest(unittest.makeSuite(BZIP2ComprTestCase))
+        theSuite.addTest(unittest.makeSuite(BZIP2ShuffleTestCase))
         theSuite.addTest(unittest.makeSuite(FloatTypeTestCase))
         theSuite.addTest(unittest.makeSuite(ComplexTypeTestCase))
-        theSuite.addTest(unittest.makeSuite(CharTypeTestCase))    
-        theSuite.addTest(unittest.makeSuite(CharType2TestCase))    
+        theSuite.addTest(unittest.makeSuite(CharTypeTestCase))
+        theSuite.addTest(unittest.makeSuite(CharType2TestCase))
         theSuite.addTest(unittest.makeSuite(CharTypeComprTestCase))
         if numeric:
             theSuite.addTest(unittest.makeSuite(Numeric1TestCase))
@@ -1996,6 +2106,20 @@ def suite():
         theSuite.addTest(unittest.makeSuite(CopyIndex3TestCase))
         theSuite.addTest(unittest.makeSuite(CopyIndex4TestCase))
         theSuite.addTest(unittest.makeSuite(CopyIndex5TestCase))
+        theSuite.addTest(unittest.makeSuite(TruncateOpenTestCase))
+        theSuite.addTest(unittest.makeSuite(TruncateCloseTestCase))
+    if heavy:
+        theSuite.addTest(unittest.makeSuite(Slices3EArrayTestCase))
+        theSuite.addTest(unittest.makeSuite(Slices4EArrayTestCase))
+        theSuite.addTest(unittest.makeSuite(Ellipsis4EArrayTestCase))
+        theSuite.addTest(unittest.makeSuite(Ellipsis5EArrayTestCase))
+        theSuite.addTest(unittest.makeSuite(Ellipsis6EArrayTestCase))
+        theSuite.addTest(unittest.makeSuite(Ellipsis7EArrayTestCase))
+        theSuite.addTest(unittest.makeSuite(MD3WriteTestCase))
+        theSuite.addTest(unittest.makeSuite(MD5WriteTestCase))
+        theSuite.addTest(unittest.makeSuite(MD6WriteTestCase))
+        theSuite.addTest(unittest.makeSuite(MD7WriteTestCase))
+        theSuite.addTest(unittest.makeSuite(MD10WriteTestCase))
         theSuite.addTest(unittest.makeSuite(CopyIndex6TestCase))
         theSuite.addTest(unittest.makeSuite(CopyIndex7TestCase))
         theSuite.addTest(unittest.makeSuite(CopyIndex8TestCase))
@@ -2003,8 +2127,8 @@ def suite():
         theSuite.addTest(unittest.makeSuite(CopyIndex10TestCase))
         theSuite.addTest(unittest.makeSuite(CopyIndex11TestCase))
         theSuite.addTest(unittest.makeSuite(CopyIndex12TestCase))
-        theSuite.addTest(unittest.makeSuite(TruncateOpenTestCase))
-        theSuite.addTest(unittest.makeSuite(TruncateCloseTestCase))
+        theSuite.addTest(unittest.makeSuite(Rows64bitsTestCase1))
+        theSuite.addTest(unittest.makeSuite(Rows64bitsTestCase2))
 
     return theSuite
 
