@@ -2,10 +2,10 @@
 #
 #	License: BSD
 #	Created: October 2, 2004
-#	Author:  Ivan Vilata i Balaguer - reverse:net.selidor@ivan
+#	Author:  Ivan Vilata i Balaguer - reverse:com.carabos@ivilata
 #
-#	$Source: /cvsroot/pytables/pytables/test/test_filenode.py,v $
-#	$Id: test_filenode.py,v 1.3.2.5 2004/11/18 19:22:15 ivilata Exp $
+#	$Source: /home/ivan/_/programari/pytables/svn/cvs/pytables/pytables/test/test_filenode.py,v $
+#	$Id: test_filenode.py 1009 2005-06-15 13:39:15Z faltet $
 #
 ########################################################################
 
@@ -17,7 +17,7 @@ from tables.nodes import FileNode
 import warnings
 
 
-__revision__ = '$Id: test_filenode.py,v 1.3.2.5 2004/11/18 19:22:15 ivilata Exp $'
+__revision__ = '$Id: test_filenode.py 1009 2005-06-15 13:39:15Z faltet $'
 
 
 
@@ -377,14 +377,16 @@ class OpenFileTestCase(unittest.TestCase):
 			IOError, FileNode.openNode, self.h5file.getNode('/test'), 'w')
 
 
-	def test03_OpenFileNoAttrs(self):
-		"Opening a node with no type attributes."
-
-		node = self.h5file.getNode('/test')
-		self.h5file.delAttrNode('/test', '_type')
-		# Another way to get the same result is changing the value.
-		##self.h5file.setAttrNode('/test', '_type', 'foobar')
-		self.assertRaises(ValueError, FileNode.openNode, node)
+	# This no longer works since type and type version attributes
+	# are now system attributes.  ivb(2004-12-29)
+	##def test03_OpenFileNoAttrs(self):
+	##	"Opening a node with no type attributes."
+	##
+	##	node = self.h5file.getNode('/test')
+	##	self.h5file.delNodeAttr('/test', '_type')
+	##	# Another way to get the same result is changing the value.
+	##	##self.h5file.setNodeAttr('/test', '_type', 'foobar')
+	##	self.assertRaises(ValueError, FileNode.openNode, node)
 
 
 
@@ -752,19 +754,51 @@ class AttrsTestCase(unittest.TestCase):
 		os.remove(self.h5fname)
 
 
-	def test00_GetTypeAttr(self):
-		"Getting the type attribute of a file node."
+	# This no longer works since type and type version attributes
+	# are now system attributes.  ivb(2004-12-29)
+	##def test00_GetTypeAttr(self):
+	##	"Getting the type attribute of a file node."
+	##
+	##	self.assertEqual(
+	##		getattr(self.fnode.attrs, '_type', None), FileNode.NodeType,
+	##		"File node has no '_type' attribute.")
 
+
+	def test00_MangleTypeAttrs(self):
+		"Mangling the type attributes on a file node."
+
+		nodeType = getattr(self.fnode.attrs, 'NODE_TYPE', None)
 		self.assertEqual(
-			getattr(self.fnode.attrs, '_type', None), FileNode.NodeType,
-			"File node has no '_type' attribute.")
+			nodeType, FileNode.NodeType,
+			"File node does not have a valid 'NODE_TYPE' attribute.")
 
+		nodeTypeVersion = getattr(self.fnode.attrs, 'NODE_TYPE_VERSION', None)
+		self.assert_(
+			nodeTypeVersion in FileNode.NodeTypeVersions,
+			"File node does not have a valid 'NODE_TYPE_VERSION' attribute.")
 
-	def test01_SetSystemAttr(self):
-		"Setting a system attribute on a file node."
+		# System attributes are now writable.  ivb(2004-12-30)
+		##self.assertRaises(
+		##	AttributeError,
+		##	setattr, self.fnode.attrs, 'NODE_TYPE', 'foobar')
+		##self.assertRaises(
+		##	AttributeError,
+		##	setattr, self.fnode.attrs, 'NODE_TYPE_VERSION', 'foobar')
 
 		self.assertRaises(
-			RuntimeError, setattr, self.fnode.attrs, 'CLASS', 'foobar')
+			AttributeError,
+			delattr, self.fnode.attrs, 'NODE_TYPE')
+		self.assertRaises(
+			AttributeError,
+			delattr, self.fnode.attrs, 'NODE_TYPE_VERSION')
+
+
+	# System attributes are now writable.  ivb(2004-12-30)
+	##def test01_SetSystemAttr(self):
+	##	"Setting a system attribute on a file node."
+	##
+	##	self.assertRaises(
+	##		AttributeError, setattr, self.fnode.attrs, 'CLASS', 'foobar')
 
 
 	def test02_SetGetDelUserAttr(self):
@@ -853,6 +887,109 @@ class ClosedH5FileTestCase(unittest.TestCase):
 
 
 
+class OldVersionTestCaseMixin:
+	"""Mix-in class for old version compatibility test cases.
+
+	It provides some basic tests for file operations and attribute handling.
+	Sub-classes must provide the 'oldversion' attribute
+	and the 'oldh5fname' attribute.
+	"""
+
+	def setUp(self):
+		"""setUp() -> None
+
+		This method sets the following instance attributes:
+		  * 'h5fname', the name of the temporary HDF5 file
+		  * 'h5file', the writable, temporary HDF5 file with a '/test' node
+		  * 'fnode', the writable file node in '/test'
+		"""
+
+		self.h5fname = tempfile.mktemp(suffix = '.h5')
+
+		oldh5f = tables.openFile(self.oldh5fname)
+		oldh5f.copyFile(self.h5fname)
+		oldh5f.close()
+
+		self.h5file = tables.openFile(
+			self.h5fname, 'r+',
+			title = "Test for file node old version compatibility")
+		self.fnode = FileNode.openNode(self.h5file.root.test, 'a+')
+
+
+	def tearDown(self):
+		"""tearDown() -> None
+
+		Closes 'fnode' and 'h5file'; removes 'h5fname'.
+		"""
+
+		self.fnode.close()
+		self.fnode = None
+		self.h5file.close()
+		self.h5file = None
+		os.remove(self.h5fname)
+
+
+	def test00_Read(self):
+		"Reading an old version file node."
+
+		self.fnode.lineSeparator = '\n'
+
+		line = self.fnode.readline()
+		self.assertEqual(line, 'This is only\n')
+
+		line = self.fnode.readline()
+		self.assertEqual(line, 'a test file\n')
+
+		line = self.fnode.readline()
+		self.assertEqual(line, 'for FileNode version %d\n' % self.oldversion)
+
+		line = self.fnode.readline()
+		self.assertEqual(line, '')
+
+		self.fnode.seek(0)
+		line = self.fnode.readline()
+		self.assertEqual(line, 'This is only\n')
+
+
+	def test01_Write(self):
+		"Writing an old version file node."
+
+		self.fnode.lineSeparator = '\n'
+
+		self.fnode.write('foobar\n')
+		self.fnode.seek(-7, 2)
+		line = self.fnode.readline()
+		self.assertEqual(line, 'foobar\n')
+
+
+	def test02_Attributes(self):
+		"Accessing attributes in an old version file node."
+
+		self.fnode.attrs.userAttr = 'foobar'
+		self.assertEqual(
+			getattr(self.fnode.attrs, 'userAttr', None), 'foobar',
+			"User attribute was not correctly set.")
+
+		self.fnode.attrs.userAttr = 'bazquux'
+		self.assertEqual(
+			getattr(self.fnode.attrs, 'userAttr', None), 'bazquux',
+			"User attribute was not correctly changed.")
+
+		del self.fnode.attrs.userAttr
+		self.assertEqual(
+			getattr(self.fnode.attrs, 'userAttr', None), None,
+			"User attribute was not deleted.")
+
+
+
+class Version1TestCase(OldVersionTestCaseMixin, unittest.TestCase):
+	"Basic test for version 1 format compatibility."
+
+	oldversion = 1
+	oldh5fname = 'test_filenode_v1.h5'
+
+
+
 #----------------------------------------------------------------------
 
 def suite():
@@ -873,6 +1010,7 @@ def suite():
 	theSuite.addTest(unittest.makeSuite(LineSeparatorTestCase))
 	theSuite.addTest(unittest.makeSuite(AttrsTestCase))
 	theSuite.addTest(unittest.makeSuite(ClosedH5FileTestCase))
+	theSuite.addTest(unittest.makeSuite(Version1TestCase))
 
 	return theSuite
 
