@@ -7,7 +7,9 @@ import random
 from tables import *
 #from tables.Index import Index
 from tables.IndexArray import calcChunksize
-from test_all import verbose, allequal, heavy
+from test_all import verbose, allequal, heavy, cleanup
+# To delete the internal attributes automagically
+unittest.TestCase.tearDown = cleanup
 
 # The minimum number of rows that can be indexed
 # Remember to change that if the number is changed in
@@ -92,6 +94,7 @@ class SelectValuesTestCase(unittest.TestCase):
     def tearDown(self):
         self.fileh.close()
         os.remove(self.file)
+        cleanup(self)
         
     #----------------------------------------
 
@@ -439,6 +442,168 @@ class SelectValuesTestCase(unittest.TestCase):
         if verbose:
             print "Limit:", sl
 #             print "Selection results (index):", results1
+#             print "Should look like:", results2
+            print "Length results:", len(results1)
+            print "Should be:", len(results2)
+        assert len(results1) == len(results2)
+        assert results1 == results2
+
+    def test03c(self):
+        """Checking selecting values from an Index (long flavor)"""
+
+        if verbose:
+            print '\n', '-=' * 30
+            print "Running %s.test03c..." % self.__class__.__name__
+
+        table1 = self.fileh.root.table1
+        table2 = self.fileh.root.table2
+
+        # Convert the limits to the appropriate type
+        il = long(self.il)
+        sl = long(self.sl)
+
+        # Do some selections and check the results
+        t1col = table1.cols.var3
+
+        # First selection
+        results1 = [p["var3"] for p in table1.where(t1col < sl)]
+        results2 = [p["var3"] for p in table2
+                    if p["var3"] < sl]
+        # sort lists (indexing does not guarantee that rows are returned in
+        # order)
+        results1.sort(); results2.sort()
+        if verbose:
+            print "Limit:", sl
+            print "Selection results (index):", results1
+            print "Should look like:", results2
+            print "Length results:", len(results1)
+            print "Should be:", len(results2)
+        assert len(results1) == len(results2)
+        assert results1 == results2
+
+        # Second selection
+        results1 = [p["var3"] for p in table1.where(t1col <= sl)]
+        results2 = [p["var3"] for p in table2
+                    if p["var3"] <= sl]
+        # sort lists (indexing does not guarantee that rows are returned in
+        # order)
+        results1.sort(); results2.sort()
+        if verbose:
+            print "Limit:", sl
+#             print "Selection results (index):", results1
+#             print "Should look like:", results2
+            print "Length results:", len(results1)
+            print "Should be:", len(results2)
+        assert len(results1) == len(results2)
+        assert results1 == results2
+
+        # Third selection
+        results1 = [p["var3"] for p in table1.where(t1col > sl)]
+        results2 = [p["var3"] for p in table2
+                    if p["var3"] > sl]
+        # sort lists (indexing does not guarantee that rows are returned in
+        # order)
+        results1.sort(); results2.sort()
+        if verbose:
+            print "Limit:", sl
+#             print "Selection results (index):", results1
+#             print "Should look like:", results2
+            print "Length results:", len(results1)
+            print "Should be:", len(results2)
+        assert len(results1) == len(results2)
+        assert results1 == results2
+
+        # Fourth selection
+        results1 = [p["var3"] for p in table1.where(t1col >= sl)]
+        results2 = [p["var3"] for p in table2
+                    if p["var3"] >= sl]
+        # sort lists (indexing does not guarantee that rows are returned in
+        # order)
+        results1.sort(); results2.sort()
+        if verbose:
+            print "Limit:", sl
+#             print "Selection results (index):", results1
+#             print "Should look like:", results2
+            print "Length results:", len(results1)
+            print "Should be:", len(results2)
+        assert len(results1) == len(results2)
+        assert results1 == results2
+
+    def test03d(self):
+        """Checking selecting values from an Index (long and int flavor)"""
+
+        if verbose:
+            print '\n', '-=' * 30
+            print "Running %s.test03d..." % self.__class__.__name__
+
+        table1 = self.fileh.root.table1
+        table2 = self.fileh.root.table2
+
+        # Convert the limits to the appropriate type
+        il = int(self.il)
+        sl = long(self.sl)
+
+        # Do some selections and check the results
+        t1col = table1.cols.var3
+
+        # First selection
+        results1 = [p["var3"] for p in table1.where(t1col < sl)]
+        results2 = [p["var3"] for p in table2
+                    if p["var3"] < sl]
+        # sort lists (indexing does not guarantee that rows are returned in
+        # order)
+        results1.sort(); results2.sort()
+        if verbose:
+            print "Limit:", sl
+            print "Selection results (index):", results1
+            print "Should look like:", results2
+            print "Length results:", len(results1)
+            print "Should be:", len(results2)
+        assert len(results1) == len(results2)
+        assert results1 == results2
+
+        # Second selection
+        results1 = [p["var3"] for p in table1.where(t1col <= sl)]
+        results2 = [p["var3"] for p in table2
+                    if p["var3"] <= sl]
+        # sort lists (indexing does not guarantee that rows are returned in
+        # order)
+        results1.sort(); results2.sort()
+        if verbose:
+            print "Limit:", sl
+#             print "Selection results (index):", results1
+#             print "Should look like:", results2
+            print "Length results:", len(results1)
+            print "Should be:", len(results2)
+        assert len(results1) == len(results2)
+        assert results1 == results2
+
+        # Third selection
+        results1 = [p["var3"] for p in table1.where(t1col > sl)]
+        results2 = [p["var3"] for p in table2
+                    if p["var3"] > sl]
+        # sort lists (indexing does not guarantee that rows are returned in
+        # order)
+        results1.sort(); results2.sort()
+        if verbose:
+            print "Limit:", sl
+#             print "Selection results (index):", results1
+#             print "Should look like:", results2
+            print "Length results:", len(results1)
+            print "Should be:", len(results2)
+        assert len(results1) == len(results2)
+        assert results1 == results2
+
+        # Fourth selection
+        results1 = [p["var3"] for p in table1.where(t1col >= sl)]
+        results2 = [p["var3"] for p in table2
+                    if p["var3"] >= sl]
+        # sort lists (indexing does not guarantee that rows are returned in
+        # order)
+        results1.sort(); results2.sort()
+        if verbose:
+            print "Limit:", sl
+#             print "Selection results (index):", results1
 #             print "Should look like:", results2
             print "Length results:", len(results1)
             print "Should be:", len(results2)
@@ -1156,7 +1321,7 @@ class SelectValuesTestCase(unittest.TestCase):
 
 
     def test09a(self):
-        """Checking whereInRange (string flavor)"""
+        """Checking _whereInRange (string flavor)"""
 
         if verbose:
             print '\n', '-=' * 30
@@ -1173,7 +1338,7 @@ class SelectValuesTestCase(unittest.TestCase):
         t1col = table1.cols.var1
         #print "t1col-->", t1col[:]
         # First selection
-        results1 = [p['var1'] for p in table1.whereInRange(t1col<=sl,2,10)]
+        results1 = [p['var1'] for p in table1._whereInRange(t1col<=sl,2,10)]
         results2 = [p["var1"] for p in table2.iterrows(2, 10)
                     if p["var1"] <= sl]
         if verbose:
@@ -1186,7 +1351,7 @@ class SelectValuesTestCase(unittest.TestCase):
         assert results1 == results2
 
         # Second selection
-        results1 = [p['var1'] for p in table1.whereInRange(il<t1col<sl,2,30,2)]
+        results1 = [p['var1'] for p in table1._whereInRange(il<t1col<sl,2,30,2)]
         results2 = [p["var1"] for p in table2.iterrows(2,30,2)
                     if il<p["var1"]<sl]
         if verbose:
@@ -1199,7 +1364,7 @@ class SelectValuesTestCase(unittest.TestCase):
         assert results1 == results2
 
         # Third selection
-        results1 = [p['var1'] for p in table1.whereInRange(il>t1col>sl,2,-5)]
+        results1 = [p['var1'] for p in table1._whereInRange(il>t1col>sl,2,-5)]
         results2 = [p["var1"] for p in table2.iterrows(2, -5)  # Negative indices
                     if (il > p["var1"] > sl)]
         if verbose:
@@ -1213,7 +1378,7 @@ class SelectValuesTestCase(unittest.TestCase):
         assert results1 == results2
 
         # This selection to be commented out
-#         results1 = [p['var1'] for p in table1.whereInRange(t1col>=sl,2,-1,1)]
+#         results1 = [p['var1'] for p in table1._whereInRange(t1col>=sl,2,-1,1)]
 #         results2 = [p["var1"] for p in table2.iterrows(2, -1, 1)
 #                     if p["var1"] >= sl]
 #         if verbose:
@@ -1227,8 +1392,8 @@ class SelectValuesTestCase(unittest.TestCase):
 
         # Fourth selection
         #print "t1col-->", t1col[:]
-        #results1 = [p['var1'] for p in table1.whereInRange(t1col>=sl,2,-1,3)]
-        results1 = [p['var1'] for p in table1.whereInRange(t1col>=sl,2,-1,3)]
+        #results1 = [p['var1'] for p in table1._whereInRange(t1col>=sl,2,-1,3)]
+        results1 = [p['var1'] for p in table1._whereInRange(t1col>=sl,2,-1,3)]
         results2 = [p["var1"] for p in table2.iterrows(2, -1, 3)
                     if p["var1"] >= sl]
         if verbose:
@@ -1241,7 +1406,7 @@ class SelectValuesTestCase(unittest.TestCase):
         assert results1 == results2
 
     def test09b(self):
-        """Checking whereInRange (float flavor)"""
+        """Checking _whereInRange (float flavor)"""
 
         if verbose:
             print '\n', '-=' * 30
@@ -1258,7 +1423,7 @@ class SelectValuesTestCase(unittest.TestCase):
         t1col = table1.cols.var4
 
         # First selection
-        results1 = [p['var4'] for p in table1.whereInRange(t1col < sl, 2, 5)]
+        results1 = [p['var4'] for p in table1._whereInRange(t1col < sl, 2, 5)]
         results2 = [p["var4"] for p in table2.iterrows(2, 5)
                     if p["var4"] < sl]
         if verbose:
@@ -1271,7 +1436,7 @@ class SelectValuesTestCase(unittest.TestCase):
         assert results1 == results2
 
         # Second selection
-        results1 = [p['var4'] for p in table1.whereInRange(il < t1col<=sl, 2, -1, 2)]
+        results1 = [p['var4'] for p in table1._whereInRange(il < t1col<=sl, 2, -1, 2)]
         results2 = [p["var4"] for p in table2.iterrows(2,-1,2)
                     if il < p["var4"] <= sl]
         if verbose:
@@ -1284,7 +1449,7 @@ class SelectValuesTestCase(unittest.TestCase):
         assert results1 == results2
 
         # Third selection
-        results1 = [p['var4'] for p in table1.whereInRange(il<=t1col<=sl, 2, -5)]
+        results1 = [p['var4'] for p in table1._whereInRange(il<=t1col<=sl, 2, -5)]
         results2 = [p["var4"] for p in table2.iterrows(2, -5)  # Negative indices
                     if il <= p["var4"] <= sl]
         if verbose:
@@ -1297,7 +1462,7 @@ class SelectValuesTestCase(unittest.TestCase):
         assert results1 == results2
 
         # Fourth selection
-        results1 = [p['var4'] for p in table1.whereInRange(t1col>=sl, 0, -1, 3)]
+        results1 = [p['var4'] for p in table1._whereInRange(t1col>=sl, 0, -1, 3)]
         results2 = [p["var4"] for p in table2.iterrows(0, -1, 3)
                     if p["var4"] >= sl]
         if verbose:
@@ -1310,7 +1475,7 @@ class SelectValuesTestCase(unittest.TestCase):
         assert results1 == results2
 
     def test09c(self):
-        """Checking whereInRange with ranges, changing step (string flavor)"""
+        """Checking _whereInRange with ranges, changing step (string flavor)"""
 
         if verbose:
             print '\n', '-=' * 30
@@ -1328,7 +1493,7 @@ class SelectValuesTestCase(unittest.TestCase):
         #print "t1col-->", t1col[:]
 
         # First selection
-        results1 = [p['var1'] for p in table1.whereInRange(t1col>=sl,2,-1,3)]
+        results1 = [p['var1'] for p in table1._whereInRange(t1col>=sl,2,-1,3)]
         results2 = [p["var1"] for p in table2.iterrows(2, -1, 3)
                     if p["var1"] >= sl]
         # sort lists (indexing does not guarantee that rows are returned in
@@ -1344,7 +1509,7 @@ class SelectValuesTestCase(unittest.TestCase):
         assert results1 == results2
 
         # Second selection
-        results1 = [p['var1'] for p in table1.whereInRange(t1col>=sl,5,-1,10)]
+        results1 = [p['var1'] for p in table1._whereInRange(t1col>=sl,5,-1,10)]
         results2 = [p["var1"] for p in table2.iterrows(5, -1, 10)
                     if p["var1"] >= sl]
         # sort lists (indexing does not guarantee that rows are returned in
@@ -1360,7 +1525,7 @@ class SelectValuesTestCase(unittest.TestCase):
         assert results1 == results2
 
         # Third selection
-        results1 = [p['var1'] for p in table1.whereInRange(t1col>=sl,5,-3,11)]
+        results1 = [p['var1'] for p in table1._whereInRange(t1col>=sl,5,-3,11)]
         results2 = [p["var1"] for p in table2.iterrows(5, -3, 11)
                     if p["var1"] >= sl]
         # sort lists (indexing does not guarantee that rows are returned in
@@ -1376,7 +1541,7 @@ class SelectValuesTestCase(unittest.TestCase):
         assert results1 == results2
 
         # Fourth selection
-        results1 = [p['var1'] for p in table1.whereInRange(t1col>=sl,2,-1,300)]
+        results1 = [p['var1'] for p in table1._whereInRange(t1col>=sl,2,-1,300)]
         results2 = [p["var1"] for p in table2.iterrows(2, -1, 300)
                     if p["var1"] >= sl]
         # sort lists (indexing does not guarantee that rows are returned in
@@ -1410,7 +1575,7 @@ class SelectValuesTestCase(unittest.TestCase):
         #print "t3col-->", t3col[:]
 
         # First selection
-        results1 = [p['var3'] for p in table1.whereInRange(t3col>=sl,2,-1,3)]
+        results1 = [p['var3'] for p in table1._whereInRange(t3col>=sl,2,-1,3)]
         results2 = [p["var3"] for p in table2.iterrows(2, -1, 3)
                     if p["var3"] >= sl]
         # sort lists (indexing does not guarantee that rows are returned in
@@ -1426,7 +1591,7 @@ class SelectValuesTestCase(unittest.TestCase):
         assert results1 == results2
 
         # Second selection
-        results1 = [p['var3'] for p in table1.whereInRange(t3col>=sl,5,-1,10)]
+        results1 = [p['var3'] for p in table1._whereInRange(t3col>=sl,5,-1,10)]
         results2 = [p["var3"] for p in table2.iterrows(5, -1, 10)
                     if p["var3"] >= sl]
         # sort lists (indexing does not guarantee that rows are returned in
@@ -1442,7 +1607,7 @@ class SelectValuesTestCase(unittest.TestCase):
         assert results1 == results2
 
         # Third selection
-        results1 = [p['var3'] for p in table1.whereInRange(t3col>=sl,5,-3,11)]
+        results1 = [p['var3'] for p in table1._whereInRange(t3col>=sl,5,-3,11)]
         results2 = [p["var3"] for p in table2.iterrows(5, -3, 11)
                     if p["var3"] >= sl]
         # sort lists (indexing does not guarantee that rows are returned in
@@ -1458,7 +1623,7 @@ class SelectValuesTestCase(unittest.TestCase):
         assert results1 == results2
 
         # Fourth selection
-        results1 = [p['var3'] for p in table1.whereInRange(t3col>=sl,2,-1,300)]
+        results1 = [p['var3'] for p in table1._whereInRange(t3col>=sl,2,-1,300)]
         results2 = [p["var3"] for p in table2.iterrows(2, -1, 300)
                     if p["var3"] >= sl]
         # sort lists (indexing does not guarantee that rows are returned in
@@ -1474,7 +1639,7 @@ class SelectValuesTestCase(unittest.TestCase):
         assert results1 == results2
 
     def test10a(self):
-        """Checking whereIndexed with ranges (string flavor)"""
+        """Checking _whereIndexed with ranges (string flavor)"""
 
         if verbose:
             print '\n', '-=' * 30
@@ -1491,8 +1656,8 @@ class SelectValuesTestCase(unittest.TestCase):
         t1col = table1.cols.var1
         #print "t1col-->", t1col[:]
         # First selection
-        #print "-->", table1.whereIndexed(t1col<=sl,2,10)
-        results1 = [p['var1'] for p in table1.whereIndexed(t1col<=sl,2,10)]
+        #print "-->", table1._whereIndexed(t1col<=sl,2,10)
+        results1 = [p['var1'] for p in table1._whereIndexed(t1col<=sl,2,10)]
         results2 = [p["var1"] for p in table2.iterrows(2, 10)
                     if p["var1"] <= sl]
         # sort lists (indexing does not guarantee that rows are returned in
@@ -1508,7 +1673,7 @@ class SelectValuesTestCase(unittest.TestCase):
         assert results1 == results2
 
         # Selection to be deleted
-        results1 = [p['var1'] for p in table1.whereIndexed(il<=t1col<=sl,2,30,1)]
+        results1 = [p['var1'] for p in table1._whereIndexed(il<=t1col<=sl,2,30,1)]
         results2 = [p["var1"] for p in table2.iterrows(2,30,1)
                     if il<=p["var1"]<=sl]
         # sort lists (indexing does not guarantee that rows are returned in
@@ -1524,7 +1689,7 @@ class SelectValuesTestCase(unittest.TestCase):
         assert results1 == results2
 
         # Second selection
-        results1 = [p['var1'] for p in table1.whereIndexed(il<=t1col<=sl,2,30,2)]
+        results1 = [p['var1'] for p in table1._whereIndexed(il<=t1col<=sl,2,30,2)]
         results2 = [p["var1"] for p in table2.iterrows(2,30,2)
                     if il<=p["var1"]<=sl]
         # sort lists (indexing does not guarantee that rows are returned in
@@ -1540,7 +1705,7 @@ class SelectValuesTestCase(unittest.TestCase):
         assert results1 == results2
 
         # Third selection
-        results1 = [p['var1'] for p in table1.whereIndexed(il<t1col<sl,2,-5)]
+        results1 = [p['var1'] for p in table1._whereIndexed(il<t1col<sl,2,-5)]
         results2 = [p["var1"] for p in table2.iterrows(2, -5)  # Negative indices
                     if (il < p["var1"] < sl)]
         # sort lists (indexing does not guarantee that rows are returned in
@@ -1556,7 +1721,7 @@ class SelectValuesTestCase(unittest.TestCase):
         assert results1 == results2
 
         # Fourth selection
-        results1 = [p['var1'] for p in table1.whereIndexed(t1col>=sl,1,-1,3)]
+        results1 = [p['var1'] for p in table1._whereIndexed(t1col>=sl,1,-1,3)]
         #print "results1-->", results1
         results2 = [p["var1"] for p in table2.iterrows(1, -1, 3)
                     if p["var1"] >= sl]
@@ -1573,7 +1738,7 @@ class SelectValuesTestCase(unittest.TestCase):
         assert results1 == results2
 
     def test10b(self):
-        """Checking whereIndexed with ranges (int flavor)"""
+        """Checking _whereIndexed with ranges (int flavor)"""
 
         if verbose:
             print '\n', '-=' * 30
@@ -1589,8 +1754,8 @@ class SelectValuesTestCase(unittest.TestCase):
         # Do some selections and check the results
         t3col = table1.cols.var3
         # First selection
-        #print "-->", table1.whereIndexed(t3col<=sl,2,10)
-        results1 = [p['var3'] for p in table1.whereIndexed(t3col<=sl,2,10)]
+        #print "-->", table1._whereIndexed(t3col<=sl,2,10)
+        results1 = [p['var3'] for p in table1._whereIndexed(t3col<=sl,2,10)]
         results2 = [p["var3"] for p in table2.iterrows(2, 10)
                     if p["var3"] <= sl]
         # sort lists (indexing does not guarantee that rows are returned in
@@ -1606,7 +1771,7 @@ class SelectValuesTestCase(unittest.TestCase):
         assert results1 == results2
 
         # Second selection
-        results1 = [p['var3'] for p in table1.whereIndexed(il<=t3col<=sl,2,30,2)]
+        results1 = [p['var3'] for p in table1._whereIndexed(il<=t3col<=sl,2,30,2)]
         results2 = [p["var3"] for p in table2.iterrows(2,30,2)
                     if il<=p["var3"]<=sl]
         # sort lists (indexing does not guarantee that rows are returned in
@@ -1622,7 +1787,7 @@ class SelectValuesTestCase(unittest.TestCase):
         assert results1 == results2
 
         # Third selection
-        results1 = [p['var3'] for p in table1.whereIndexed(il<t3col<sl,2,-5)]
+        results1 = [p['var3'] for p in table1._whereIndexed(il<t3col<sl,2,-5)]
         results2 = [p["var3"] for p in table2.iterrows(2, -5)  # Negative indices
                     if (il < p["var3"] < sl)]
         # sort lists (indexing does not guarantee that rows are returned in
@@ -1639,7 +1804,7 @@ class SelectValuesTestCase(unittest.TestCase):
 
         # Fourth selection
         #print "t3col-->", t3col[:]
-        results1 = [p['var3'] for p in table1.whereIndexed(t3col>=sl,1,-1,3)]
+        results1 = [p['var3'] for p in table1._whereIndexed(t3col>=sl,1,-1,3)]
         results2 = [p["var3"] for p in table2.iterrows(1, -1, 3)
                     if p["var3"] >= sl]
         # sort lists (indexing does not guarantee that rows are returned in
@@ -1655,7 +1820,7 @@ class SelectValuesTestCase(unittest.TestCase):
         assert results1 == results2
 
     def test10c(self):
-        """Checking whereIndexed with ranges, changing step (string flavor)"""
+        """Checking _whereIndexed with ranges, changing step (string flavor)"""
 
         if verbose:
             print '\n', '-=' * 30
@@ -1673,7 +1838,7 @@ class SelectValuesTestCase(unittest.TestCase):
         #print "t1col-->", t1col[:]
 
         # First selection
-        results1 = [p['var1'] for p in table1.whereIndexed(t1col>=sl,2,-1,3)]
+        results1 = [p['var1'] for p in table1._whereIndexed(t1col>=sl,2,-1,3)]
         results2 = [p["var1"] for p in table2.iterrows(2, -1, 3)
                     if p["var1"] >= sl]
         # sort lists (indexing does not guarantee that rows are returned in
@@ -1689,7 +1854,7 @@ class SelectValuesTestCase(unittest.TestCase):
         assert results1 == results2
 
         # Second selection
-        results1 = [p['var1'] for p in table1.whereIndexed(t1col>=sl,5,-1,10)]
+        results1 = [p['var1'] for p in table1._whereIndexed(t1col>=sl,5,-1,10)]
         results2 = [p["var1"] for p in table2.iterrows(5, -1, 10)
                     if p["var1"] >= sl]
         # sort lists (indexing does not guarantee that rows are returned in
@@ -1705,7 +1870,7 @@ class SelectValuesTestCase(unittest.TestCase):
         assert results1 == results2
 
         # Third selection
-        results1 = [p['var1'] for p in table1.whereIndexed(t1col>=sl,5,-3,11)]
+        results1 = [p['var1'] for p in table1._whereIndexed(t1col>=sl,5,-3,11)]
         results2 = [p["var1"] for p in table2.iterrows(5, -3, 11)
                     if p["var1"] >= sl]
         # sort lists (indexing does not guarantee that rows are returned in
@@ -1721,7 +1886,7 @@ class SelectValuesTestCase(unittest.TestCase):
         assert results1 == results2
 
         # Fourth selection
-        results1 = [p['var1'] for p in table1.whereIndexed(t1col>=sl,2,-1,300)]
+        results1 = [p['var1'] for p in table1._whereIndexed(t1col>=sl,2,-1,300)]
         results2 = [p["var1"] for p in table2.iterrows(2, -1, 300)
                     if p["var1"] >= sl]
         # sort lists (indexing does not guarantee that rows are returned in
@@ -1737,7 +1902,7 @@ class SelectValuesTestCase(unittest.TestCase):
         assert results1 == results2
 
     def test10d(self):
-        """Checking whereIndexed with ranges, changing step (int flavor)"""
+        """Checking _whereIndexed with ranges, changing step (int flavor)"""
 
         if verbose:
             print '\n', '-=' * 30
@@ -1755,7 +1920,7 @@ class SelectValuesTestCase(unittest.TestCase):
         #print "t3col-->", t3col[:]
 
         # First selection
-        results1 = [p['var3'] for p in table1.whereIndexed(t3col>=sl,2,-1,3)]
+        results1 = [p['var3'] for p in table1._whereIndexed(t3col>=sl,2,-1,3)]
         results2 = [p["var3"] for p in table2.iterrows(2, -1, 3)
                     if p["var3"] >= sl]
         # sort lists (indexing does not guarantee that rows are returned in
@@ -1771,7 +1936,7 @@ class SelectValuesTestCase(unittest.TestCase):
         assert results1 == results2
 
         # Second selection
-        results1 = [p['var3'] for p in table1.whereIndexed(t3col>=sl,5,-1,10)]
+        results1 = [p['var3'] for p in table1._whereIndexed(t3col>=sl,5,-1,10)]
         results2 = [p["var3"] for p in table2.iterrows(5, -1, 10)
                     if p["var3"] >= sl]
         # sort lists (indexing does not guarantee that rows are returned in
@@ -1787,7 +1952,7 @@ class SelectValuesTestCase(unittest.TestCase):
         assert results1 == results2
 
         # Third selection
-        results1 = [p['var3'] for p in table1.whereIndexed(t3col>=sl,5,-3,11)]
+        results1 = [p['var3'] for p in table1._whereIndexed(t3col>=sl,5,-3,11)]
         results2 = [p["var3"] for p in table2.iterrows(5, -3, 11)
                     if p["var3"] >= sl]
         # sort lists (indexing does not guarantee that rows are returned in
@@ -1803,7 +1968,7 @@ class SelectValuesTestCase(unittest.TestCase):
         assert results1 == results2
 
         # Fourth selection
-        results1 = [p['var3'] for p in table1.whereIndexed(t3col>=sl,2,-1,300)]
+        results1 = [p['var3'] for p in table1._whereIndexed(t3col>=sl,2,-1,300)]
         results2 = [p["var3"] for p in table2.iterrows(2, -1, 300)
                     if p["var3"] >= sl]
         # sort lists (indexing does not guarantee that rows are returned in
@@ -1818,6 +1983,36 @@ class SelectValuesTestCase(unittest.TestCase):
         assert len(results1) == len(results2)
         assert results1 == results2
 
+    def test11a(self):
+        """Checking selecting values from an Index via readCoordinates()"""
+
+        if verbose:
+            print '\n', '-=' * 30
+            print "Running %s.test11a..." % self.__class__.__name__
+
+        table1 = self.fileh.root.table1
+        table2 = self.fileh.root.table2
+
+        # Convert the limits to the appropriate type
+        il = str(self.il)
+        sl = str(self.sl)
+
+        # Do a selection and check the result
+        t1var1 = table1.cols.var1
+        coords1 = table1.getWhereList(il <= t1var1 <= sl)
+        results1 = table1.readCoordinates(coords1, field="var1", flavor="List")
+        results2 = [p["var1"] for p in table2
+                    if il <= p["var1"] <= sl]
+        results1.sort(); results2.sort()
+        if verbose:
+#             print "Superior & inferior limits:", il, sl
+#             print "Selection results (index):", results1
+#             print "Should look like:", results2
+            print "Length results:", len(results1)
+            print "Should be:", len(results2)
+        assert len(results1) == len(results2)
+        assert results1 == results2
+
 
 class SV1aTestCase(SelectValuesTestCase):
     minRowIndex = 10
@@ -2071,10 +2266,6 @@ def suite():
     niter = 1
     #heavy = 1  # Uncomment this only for testing purposes!
 
-    #theSuite.addTest(unittest.makeSuite(SV14aTestCase))
-    #theSuite.addTest(unittest.makeSuite(SV9aTestCase))
-#     for i in range(100):
-#         theSuite.addTest(unittest.makeSuite(SV9aTestCase))
     #theSuite.addTest(unittest.makeSuite(SV11aTestCase))
     #theSuite.addTest(unittest.makeSuite(SV4aTestCase))
     for n in range(niter):
@@ -2089,9 +2280,6 @@ def suite():
         theSuite.addTest(unittest.makeSuite(SV11bTestCase))
         theSuite.addTest(unittest.makeSuite(SV12aTestCase))
         theSuite.addTest(unittest.makeSuite(SV12bTestCase))
-        theSuite.addTest(unittest.makeSuite(SV13aTestCase))
-        theSuite.addTest(unittest.makeSuite(SV13bTestCase))
-        theSuite.addTest(unittest.makeSuite(SV14aTestCase))
     if heavy:
         theSuite.addTest(unittest.makeSuite(SV1bTestCase))
         theSuite.addTest(unittest.makeSuite(SV2bTestCase))
@@ -2103,6 +2291,9 @@ def suite():
         theSuite.addTest(unittest.makeSuite(SV8bTestCase))
         theSuite.addTest(unittest.makeSuite(SV9bTestCase))
         theSuite.addTest(unittest.makeSuite(SV10bTestCase))
+        theSuite.addTest(unittest.makeSuite(SV13aTestCase))
+        theSuite.addTest(unittest.makeSuite(SV13bTestCase))
+        theSuite.addTest(unittest.makeSuite(SV14aTestCase))
         theSuite.addTest(unittest.makeSuite(SV14bTestCase))
         # The next are too hard to be above
         theSuite.addTest(unittest.makeSuite(SV5aTestCase))
@@ -2112,11 +2303,11 @@ def suite():
         # """Warning: Encountered invalid numeric result(s)  in less_equal"""
         # series of messages and I don't want to worry normal users
         # about this (I don't think this is grave anyway)
-        # F. Alted 2004-08-12
+        # F. Altet 2004-08-12
         theSuite.addTest(unittest.makeSuite(SV7aTestCase))
         theSuite.addTest(unittest.makeSuite(SV9aTestCase))
         theSuite.addTest(unittest.makeSuite(SV10aTestCase))
-    
+
     return theSuite
 
 if __name__ == '__main__':
