@@ -4,7 +4,9 @@ import os
 import tempfile
 from tables import *
 
-from test_all import verbose
+from test_all import verbose, cleanup
+# To delete the internal attributes automagically
+unittest.TestCase.tearDown = cleanup
 
 # Check if we are using python 2.2
 if sys.version_info[0] == 2 and sys.version_info[1] == 2:
@@ -12,65 +14,64 @@ if sys.version_info[0] == 2 and sys.version_info[1] == 2:
 else:
     python22 = 0
 
-def WriteRead(testTuple):
+def WriteRead(filename, testTuple):
     if verbose:
         print '\n', '-=' * 30
         print "Running test for object %s" % \
                   (type(testTuple))
 
     # Create an instance of HDF5 Table
-    file = tempfile.mktemp(".h5")
-    fileh = openFile(file, mode = "w")
+    fileh = openFile(filename, mode = "w")
     root = fileh.root
-
-    # Create the array under root and name 'somearray'
-    a = testTuple
     try:
-        fileh.createArray(root, 'somearray', a, "Some array")
-    except ValueError:
-        fileh.close()
-        os.remove(file)
-        # re-raise ValueError exception
-        raise ValueError
-    # Close the file
-    fileh.close()
+	# Create the array under root and name 'somearray'
+	a = testTuple
+	fileh.createArray(root, 'somearray', a, "Some array")
+    finally:
+	# Close the file
+	fileh.close()
 
     # Re-open the file in read-only mode
-    fileh = openFile(file, mode = "r")
+    fileh = openFile(filename, mode = "r")
     root = fileh.root
 
     # Read the saved array
-    b = root.somearray.read()
-    # Compare them. They should be equal.
-    if not a == b and verbose:
-        print "Write and read lists/tuples differ!"
-        print "Object written:", a
-        print "Object read:", b
+    try:
+	b = root.somearray.read()
+	# Compare them. They should be equal.
+	if not a == b and verbose:
+	    print "Write and read lists/tuples differ!"
+	    print "Object written:", a
+	    print "Object read:", b
 
+	# Check strictly the array equality
+	assert a == b
+    finally:
+	# Close the file
+	fileh.close()
 
-    # Check strictly the array equality
-    assert a == b
-
-    # Close the file
-    fileh.close()
-    # Then, delete the file
-    os.remove(file)
-    return file
-    
 class BasicTestCase(unittest.TestCase):
 
     def test00_char(self):
         "Data integrity during recovery (character types)"
 
         a = self.charList
-	WriteRead(a)
-	return
+        fname = tempfile.mktemp(".h5")
+        try:
+            WriteRead(fname, a)
+        finally:
+            os.remove(fname)
+        return
 
     def test01_types(self):
         "Data integrity during recovery (numerical types)"
         
         a = self.numericalList
-        WriteRead(a)
+        fname = tempfile.mktemp(".h5")
+        try:
+            WriteRead(fname, a)
+        finally:
+            os.remove(fname)
         return
 
 class Basic0DOneTestCase(BasicTestCase):
@@ -137,14 +138,18 @@ class ExceptionTestCase(unittest.TestCase):
                   (self.title)
         a = self.charList
         try:
-            WriteRead(a)
-        except ValueError:
+            fname = tempfile.mktemp(".h5")
+            try:
+                WriteRead(fname, a)
+            finally:
+                os.remove(fname)
+        except TypeError:
             if verbose:
                 (type, value, traceback) = sys.exc_info()
                 print "\nGreat!, the next error was catched!"
                 print type, ":", value
         else:
-            self.fail("expected an ValueError")
+            self.fail("expected an TypeError")
             
 
 	return
@@ -154,7 +159,11 @@ class ExceptionTestCase(unittest.TestCase):
         
         a = self.numericalList
         try:
-            WriteRead(a)
+            fname = tempfile.mktemp(".h5")
+            try:
+                WriteRead(fname, a)
+            finally:
+                os.remove(fname)
         except ValueError:
 	    # Close the file
 	    #fileh.close()
