@@ -8,66 +8,78 @@ from numarray import *
 import numarray.records as records
 from numarray import strings
 from tables import *
-from tables.hdf5Extension import getIndices
 
-from test_all import verbose, heavy, allequal
-# If we use the test_all.allequal function, a segmentation violation appears
-# but only when the test runs *alone* and *without* verbose parameters!
-# However, if we use the allequal in this module, everything seems to work well
-# this should be further investigated!. F. Alted 2004/01/01
+from test_all import verbose, heavy, allequal, cleanup
+# To delete the internal attributes automagically
+unittest.TestCase.tearDown = cleanup
 
-# Update: That seems to work well now. Perhaps a bug in numarray that
-# has been solved? F. Alted 2004/08/06
-
-# def allequal(a,b):
-#     """Checks if two numarrays are equal"""
-
-#     if a.shape <> b.shape:
-#         return 0
-
-#     # Rank-0 case
-#     if len(a.shape) == 0:
-#         if str(equal(a,b)) == '1':
-#             return 1
-#         else:
-#             return 0
-#     # Multidimensional case
-#     result = (a == b)
-#     for i in range(len(a.shape)):
-#         result = logical_and.reduce(result)
-
-#     return result
+# It is important that columns are ordered according to their names
+# to ease the comparison with record arrays.
 
 # Test Record class
 class Record(IsDescription):
+    var0 = StringCol(4, "abcd", shape=2)        # 4-character string array
     var1 = StringCol(4, "abcd", shape=(2,2))    # 4-character string array
+    var1_= IntCol(((1,1),), shape=2)            # integer array
     var2 = IntCol(((1,1),(1,1)), shape=(2,2))   # integer array
-    var3 = Int16Col(2)                          # short integer 
+    var3 = Int16Col(2)                          # short integer
     var4 = FloatCol(3.1)                        # double (double-precision)
     var5 = Float32Col(4.2)                      # float  (single-precision)
-    var6 = UInt16Col(5)                         # unsigned short integer 
+    var6 = UInt16Col(5)                         # unsigned short integer
     var7 = StringCol(length=1, dflt="e")        # 1-character String
 
-# From 0.3 on, you can dynamically define the tables with a dictionary
+# A byte-reversed class definition
+class RecordRevOrder(IsDescription):
+    # Change the byteorder property for this table
+    _v_byteorder = {"little":"big","big":"little"}[sys.byteorder]
+    var0 = StringCol(4, "abcd", shape=2)        # 4-character string array
+    var1 = StringCol(4, "abcd", shape=(2,2))    # 4-character string array
+    var1_= IntCol(((1,1),), shape=2)            # integer array
+    var2 = IntCol(((1,1),(1,1)), shape=(2,2))   # integer array
+    var3 = Int16Col(2)                          # short integer
+    var4 = FloatCol(3.1)                        # double (double-precision)
+    var5 = Float32Col(4.2)                      # float  (single-precision)
+    var6 = UInt16Col(5)                         # unsigned short integer
+    var7 = StringCol(length=1, dflt="e")        # 1-character String
+
+#  Dictionary definition
 RecordDescriptionDict = {
+    'var0': StringCol(length=4, shape=2),         # 4-character String
     'var1': StringCol(length=4, shape=(2,2)),     # 4-character String
+    'var1_':IntCol(shape=2),                      # integer array
     'var2': IntCol(shape=(2,2)),                  # integer array
-    'var3': Int16Col(),                           # short integer 
+    'var3': Int16Col(),                           # short integer
     'var4': FloatCol(),                           # double (double-precision)
     'var5': Float32Col(),                         # float  (single-precision)
-    'var6': Int16Col(),                           # unsigned short integer 
+    'var6': Int16Col(),                           # unsigned short integer
+    'var7': StringCol(length=1),                  # 1-character String
+    }
+
+# A byte-reversed dictionary definition
+RecordDescriptionDictRevOrder = {
+    # Change the byteorder property for this table
+    '_v_byteorder': {"little":"big","big":"little"}[sys.byteorder],
+    'var0': StringCol(length=4, shape=2),         # 4-character String
+    'var1': StringCol(length=4, shape=(2,2)),     # 4-character String
+    'var1_':IntCol(shape=2),                      # integer array
+    'var2': IntCol(shape=(2,2)),                  # integer array
+    'var3': Int16Col(),                           # short integer
+    'var4': FloatCol(),                           # double (double-precision)
+    'var5': Float32Col(),                         # float  (single-precision)
+    'var6': Int16Col(),                           # unsigned short integer
     'var7': StringCol(length=1),                  # 1-character String
     }
 
 class BasicTestCase(unittest.TestCase):
     #file  = "test.h5"
-    mode  = "w" 
+    mode  = "w"
     title = "This is the table title"
     expectedrows = 100
     appendrows = 20
     compress = 0
     complib = "zlib"  # Default compression library
     record = Record
+    recordro = RecordRevOrder
     recarrayinit = 0
     maxshort = 1 << 15
 
@@ -89,10 +101,14 @@ class BasicTestCase(unittest.TestCase):
         for i in xrange(self.expectedrows+1):
             tmplist = []
             # Both forms (list or chararray) works
+            var0 = ['%04d' % (self.expectedrows - i)] * 2
+            tmplist.append(var0)
             var1 = [['%04d' % (self.expectedrows - i)] * 2] * 2
 #             var1 = strings.array([['%04d' % (self.expectedrows - i)] * 2] * 2,
 #                                  itemsize = 4, shape=(2,2))
             tmplist.append(var1)
+            var1_ = (i, 1)
+            tmplist.append(var1_)
             var2 = ((i, 1), (1,1))           # *-*
             tmplist.append(var2)
             var3 = i % self.maxshort
@@ -114,9 +130,11 @@ class BasicTestCase(unittest.TestCase):
         self.record=records.array(buflist, formats=record._formats,
                                    names=record._names,
                                    shape = self.expectedrows)
-
+        # The swapped version
+        self.recordro = self.record.copy()
+        self.recordro._byteorder = {"little":"big","big":"little"}[sys.byteorder]
         return
-		
+
     def populateFile(self):
         group = self.rootgroup
         if self.recarrayinit:
@@ -126,19 +144,26 @@ class BasicTestCase(unittest.TestCase):
             # Create a table
             filters = Filters(complevel = self.compress,
                               complib = self.complib)
-            table = self.fileh.createTable(group, 'table'+str(j), self.record,
+            if j < 2:
+                record = self.record
+            else:
+                # table2 will be byteswapped
+                record = self.recordro
+            table = self.fileh.createTable(group, 'table'+str(j), record,
                                            title = self.title,
                                            filters = filters,
                                            expectedrows = self.expectedrows)
             if not self.recarrayinit:
                 # Get the row object associated with the new table
                 row = table.row
-	    
+
                 # Fill the table
                 for i in xrange(self.expectedrows):
+                    row['var0'] = '%04d' % (self.expectedrows - i)
                     row['var1'] = '%04d' % (self.expectedrows - i)
                     row['var7'] = row['var1'][0][0][-1]
-                    row['var2'] = ((i, 1), (1,1))  # *-* 
+                    row['var1_'] = (i, 1)
+                    row['var2'] = ((i, 1), (1,1))  # *-*
                     row['var3'] = i % self.maxshort
                     if isinstance(row['var4'], NumArray):
                         row['var4'] = [float(i), float(i*i)]
@@ -153,7 +178,7 @@ class BasicTestCase(unittest.TestCase):
                                   ((row['var3']<<8) & 0xff00)
                     #print("Saving -->", row)
                     row.append()
-		
+
             # Flush the buffer for this table
             table.flush()
             # Create a new group (descendant of group)
@@ -166,7 +191,8 @@ class BasicTestCase(unittest.TestCase):
         self.fileh.close()
         #del self.fileh, self.rootgroup
         os.remove(self.file)
-        
+        cleanup(self)
+
     #----------------------------------------
 
     def test01_readTable(self):
@@ -186,20 +212,31 @@ class BasicTestCase(unittest.TestCase):
         # Read the records and select those with "var2" file less than 20
         result = [ rec['var2'][0][0] for rec in table.iterrows()
                    if rec['var2'][0][0] < 20 ]
+
         if verbose:
             print "Table:", repr(table)
             print "Nrows in", table._v_pathname, ":", table.nrows
             print "Last record in table ==>", rec
             print "Total selected records in table ==> ", len(result)
         nrows = self.expectedrows - 1
-        assert (rec['var1'][0][0], rec['var2'][0][0], rec['var7']) == \
-               ("0001",            nrows,             "1")
+        assert ((
+            rec['var0'][0],
+            rec['var1'][0][0],
+            rec['var1_'][0],
+            rec['var2'][0][0],
+            rec['var7']
+            ) == (
+            "0001",
+            "0001",
+            nrows,
+            nrows,
+            "1"))
         if isinstance(rec['var5'], NumArray):
             assert allequal(rec['var5'], array((float(nrows),)*4, Float32))
         else:
             assert rec['var5'] == float(nrows)
         assert len(result) == 20
-        
+
     def test01b_readTable(self):
         """Checking table read and cuts (multidimensional columns case)"""
 
@@ -221,16 +258,16 @@ class BasicTestCase(unittest.TestCase):
             print "Nrows in", table._v_pathname, ":", table.nrows
             print "Last record in table ==>", rec
             print "Total selected records in table ==> ", len(result)
-        nrows = table.row.nrow()
+        nrows = table.nrows
         if isinstance(rec['var5'], NumArray):
             assert allequal(result[0], array((float(0),)*4, Float32))
             assert allequal(result[1], array((float(1),)*4, Float32))
             assert allequal(result[2], array((float(2),)*4, Float32))
             assert allequal(result[3], array((float(3),)*4, Float32))
             assert allequal(result[10], array((float(10),)*4, Float32))
-            assert allequal(rec['var5'], array((float(nrows),)*4, Float32))
+            assert allequal(rec['var5'], array((float(nrows-1),)*4, Float32))
         else:
-            assert rec['var5'] == float(nrows)
+            assert rec['var5'] == float(nrows-1)
         assert len(result) == 20
 
         # Read the records and select those with "var2" file less than 20
@@ -253,7 +290,7 @@ class BasicTestCase(unittest.TestCase):
         else:
             assert rec['var1'] == "0001"
         assert len(result) == 20
-                
+
     def test02_AppendRows(self):
         """Checking whether appending record rows works or not"""
 
@@ -270,12 +307,14 @@ class BasicTestCase(unittest.TestCase):
         row = table.row
         if verbose:
             print "Nrows in old", table._v_pathname, ":", table.nrows
-            print "Record Format ==>", table._v_fmt
+            print "Record Format ==>", table.description._v_nestedFormats
             print "Record Size ==>", table.rowsize
         # Append some rows
         for i in xrange(self.appendrows):
+            row['var0'] = '%04d' % (self.appendrows - i)
             row['var1'] = '%04d' % (self.appendrows - i)
             row['var7'] = row['var1'][0][0][-1]
+            row['var1_'] = (i, 1)
             row['var2'] = ((i, 1), (1,1))   # *-*
             row['var3'] = i % self.maxshort
             if isinstance(row['var4'], NumArray):
@@ -287,15 +326,25 @@ class BasicTestCase(unittest.TestCase):
             else:
                 row['var5'] = float(i)
             row.append()
-	    
+
 	# Flush the buffer for this table and read it
         table.flush()
         result = [ row['var2'][0][0] for row in table.iterrows()
                    if row['var2'][0][0] < 20 ]
-	
+
         nrows = self.appendrows - 1
-        assert (row['var1'][0][0], row['var2'][0][0], row['var7']) == \
-               ("0001", nrows, "1")
+        assert ((
+            row['var0'][0],
+            row['var1'][0][0],
+            row['var1_'][0],
+            row['var2'][0][0],
+            row['var7']
+            ) == (
+            "0001",
+            "0001",
+            nrows,
+            nrows,
+            "1"))
         if isinstance(row['var5'], NumArray):
             assert allequal(row['var5'], array((float(nrows),)*4, Float32))
         else:
@@ -318,12 +367,12 @@ class BasicTestCase(unittest.TestCase):
 
         # Create an instance of an HDF5 Table
         self.fileh = openFile(self.file, "r")
-        table = self.fileh.getNode("/table0")
+        table = self.fileh.getNode("/group0/group1/table2")
 
         # Manually change the byteorder property for this table
         table.byteorder = {"little":"big","big":"little"}[table.byteorder]
         #table.togglebyteorder()
-	
+
         # Read the records and select the ones with "var6" column less than 20
         result = [ rec['var2'] for rec in table.iterrows() if rec['var6'] < 20]
         if verbose:
@@ -333,7 +382,7 @@ class BasicTestCase(unittest.TestCase):
         nrows = self.expectedrows - 1
         assert (rec['var1'][0][0], rec['var6']) == ("0001", nrows)
         assert len(result) == 20
-        
+
 class BasicWriteTestCase(BasicTestCase):
     title = "BasicWrite"
     pass
@@ -342,6 +391,7 @@ class DictWriteTestCase(BasicTestCase):
     # This checks also unidimensional arrays as columns
     title = "DictWrite"
     record = RecordDescriptionDict
+    recordro = RecordDescriptionDictRevOrder
     nrows = 21
     maxTuples = 3  # Choose a small value for the buffer size
     start = 0
@@ -350,35 +400,43 @@ class DictWriteTestCase(BasicTestCase):
 
 class RecArrayOneWriteTestCase(BasicTestCase):
     title = "RecArrayOneWrite"
-    record=records.array(formats="(2,2)a4,(2,2)i4,i2,2f8,f4,i2,a1",
-                         names='var1,var2,var3,var4,var5,var6,var7')
+    record=records.array(
+        formats="(2,)a4,(2,2)a4,(2,)i4,(2,2)i4,i2,2f8,f4,i2,a1",
+        names='var0,var1,var1_,var2,var3,var4,var5,var6,var7')
 
 class RecArrayTwoWriteTestCase(BasicTestCase):
     title = "RecArrayTwoWrite"
     expectedrows = 100
     recarrayinit = 1
-    recordtemplate=records.array(formats="(2,2)a4,(2,2)i4,i2,f8,f4,i2,a1",
-                                 names='var1,var2,var3,var4,var5,var6,var7',
-                                 shape=1)
+    recordtemplate=records.array(
+        formats="(2,)a4,(2,2)a4,(2,)i4,(2,2)i4,i2,f8,f4,i2,a1",
+        names='var0,var1,var1_,var2,var3,var4,var5,var6,var7',
+        shape=1)
 
 class RecArrayThreeWriteTestCase(BasicTestCase):
     title = "RecArrayThreeWrite"
     expectedrows = 100
     recarrayinit = 1
-    recordtemplate=records.array(formats="(2,2)a4,(2,2)i4,i2,2f8,4f4,i2,a1",
-                                 names='var1,var2,var3,var4,var5,var6,var7',
-                                 shape=1)
+    recordtemplate=records.array(
+        formats="(2,)a4,(2,2)a4,(2,)i4,(2,2)i4,i2,2f8,4f4,i2,a1",
+        names='var0,var1,var1_,var2,var3,var4,var5,var6,var7',
+        shape=1)
 
 class CompressLZOTablesTestCase(BasicTestCase):
     title = "CompressLZOTables"
     compress = 1
     complib = "lzo"
-    
+
 class CompressUCLTablesTestCase(BasicTestCase):
     title = "CompressUCLTables"
     compress = 1
     complib = "ucl"
-    
+
+class CompressBZIP2TablesTestCase(BasicTestCase):
+    title = "CompressBZIP2Tables"
+    compress = 1
+    complib = "bzip2"
+
 class CompressZLIBTablesTestCase(BasicTestCase):
     title = "CompressOneTables"
     compress = 1
@@ -389,12 +447,13 @@ class CompressTwoTablesTestCase(BasicTestCase):
     compress = 1
     # This checks also unidimensional arrays as columns
     record = RecordDescriptionDict
+    recordro = RecordDescriptionDictRevOrder
 
 class BigTablesTestCase(BasicTestCase):
     title = "BigTables"
     # 10000 rows takes much more time than we can afford for tests
     # reducing to 1000 would be more than enough
-    # F. Alted 2004-01-19
+    # F. Altet 2004-01-19
 #     expectedrows = 10000
 #     appendrows = 1000
     expectedrows = 1000
@@ -402,7 +461,7 @@ class BigTablesTestCase(BasicTestCase):
 
 class BasicRangeTestCase(unittest.TestCase):
     #file  = "test.h5"
-    mode  = "w" 
+    mode  = "w"
     title = "This is the table title"
     record = Record
     maxshort = 1 << 15
@@ -439,7 +498,7 @@ class BasicRangeTestCase(unittest.TestCase):
             for i in xrange(self.expectedrows):
                 row['var1'] = '%04d' % (self.expectedrows - i)
                 row['var7'] = row['var1'][0][0][-1]
-                row['var2'] = i 
+                row['var2'] = i
                 row['var3'] = i % self.maxshort
                 if isinstance(row['var4'], NumArray):
                     row['var4'] = [float(i), float(i*i)]
@@ -453,7 +512,7 @@ class BasicRangeTestCase(unittest.TestCase):
                 row['var6'] = ((row['var3'] >> 8) & 0xff) + \
                               ((row['var3'] << 8) & 0xff00)
                 row.append()
-		
+
             # Flush the buffer for this table
             table.flush()
             # Create a new group (descendant of group)
@@ -467,7 +526,8 @@ class BasicRangeTestCase(unittest.TestCase):
             self.fileh.close()
         #del self.fileh, self.rootgroup
         os.remove(self.file)
-        
+        cleanup(self)
+
     #----------------------------------------
 
     def check_range(self):
@@ -479,8 +539,7 @@ class BasicRangeTestCase(unittest.TestCase):
 
         table._v_maxTuples = self.maxTuples
         r = slice(self.start, self.stop, self.step)
-        #resrange = r.indices(table.nrows)
-        resrange = getIndices(r,table.nrows)
+        resrange = r.indices(table.nrows)
         reslength = len(range(*resrange))
         if self.checkrecarray:
             recarray = table.read(self.start, self.stop, self.step)
@@ -498,14 +557,14 @@ class BasicRangeTestCase(unittest.TestCase):
             result = [ rec['var2'][0][0] for rec in
                        table.iterrows(self.start, self.stop, self.step)
                        if rec['var2'][0][0] < self.nrows ]
-        
+
         if self.start < 0:
             startr = self.expectedrows + self.start
         else:
             startr = self.start
 
         if self.stop == None:
-            stopr = startr + 1                
+            stopr = startr + 1
         elif self.stop < 0:
             stopr = self.expectedrows + self.stop
         else:
@@ -513,7 +572,7 @@ class BasicRangeTestCase(unittest.TestCase):
 
         if self.nrows < stopr:
             stopr = self.nrows
-            
+
         if verbose:
             print "Nrows in", table._v_pathname, ":", table.nrows
             if reslength:
@@ -595,7 +654,7 @@ class BasicRangeTestCase(unittest.TestCase):
             print '\n', '-=' * 30
             print "Running %s.test04_range..." % self.__class__.__name__
 
-        # Case where step == maxTuples 
+        # Case where step == maxTuples
         self.nrows = self.expectedrows
         self.maxTuples = 11  # Choose a small value for the buffer size
         self.start = 1
@@ -611,7 +670,7 @@ class BasicRangeTestCase(unittest.TestCase):
             print '\n', '-=' * 30
             print "Running %s.test05_range..." % self.__class__.__name__
 
-        # Case where step > 1.1*maxTuples 
+        # Case where step > 1.1*maxTuples
         self.nrows = 21
         self.maxTuples = 10  # Choose a small value for the buffer size
         self.start = 1
@@ -627,7 +686,7 @@ class BasicRangeTestCase(unittest.TestCase):
             print '\n', '-=' * 30
             print "Running %s.test06_range..." % self.__class__.__name__
 
-        # Case where step > 3*maxTuples 
+        # Case where step > 3*maxTuples
         self.nrows = 3
         self.maxTuples = 3  # Choose a small value for the buffer size
         self.start = 2
@@ -643,7 +702,7 @@ class BasicRangeTestCase(unittest.TestCase):
             print '\n', '-=' * 30
             print "Running %s.test07_range..." % self.__class__.__name__
 
-        # Case where start == stop 
+        # Case where start == stop
         self.nrows = 2
         self.maxTuples = 3  # Choose a small value for the buffer size
         self.start = self.nrows
@@ -659,7 +718,7 @@ class BasicRangeTestCase(unittest.TestCase):
             print '\n', '-=' * 30
             print "Running %s.test08_range..." % self.__class__.__name__
 
-        # Case where start > stop 
+        # Case where start > stop
         self.nrows = 2
         self.maxTuples = 3  # Choose a small value for the buffer size
         self.start = self.nrows + 1
@@ -745,7 +804,7 @@ class BasicRangeTestCase(unittest.TestCase):
             print '\n', '-=' * 30
             print "Running %s.test13_range..." % self.__class__.__name__
 
-        # Case where step < 0 
+        # Case where step < 0
         self.step = -11
         try:
             self.check_range()
@@ -759,7 +818,7 @@ class BasicRangeTestCase(unittest.TestCase):
             print rec
             self.fail("expected a ValueError")
 
-        # Case where step == 0 
+        # Case where step == 0
         self.step = 0
         try:
             self.check_range()
@@ -797,15 +856,15 @@ class getColRangeTestCase(BasicRangeTestCase):
 
         try:
             column = table.read(field='non-existent-column')
-        except LookupError:
+        except KeyError:
             if verbose:
                 (type, value, traceback) = sys.exc_info()
-                print "\nGreat!, the next LookupError was catched!"
+                print "\nGreat!, the next KeyError was catched!"
                 print value
             pass
         else:
             print rec
-            self.fail("expected a LookupError")
+            self.fail("expected a KeyError")
 
 
 class RecArrayIO(unittest.TestCase):
@@ -833,7 +892,7 @@ class RecArrayIO(unittest.TestCase):
         r2 = fileh.root.recarray.read()
 
         assert r.tostring() == r2.tostring()
-        
+
         fileh.close()
         os.remove(file)
 
@@ -856,7 +915,7 @@ class RecArrayIO(unittest.TestCase):
         # Get an offsetted bytearray
         r1 = r[1:]
         assert r1._byteoffset > 0
-        
+
         # Save it in a table:
         fileh.createTable(fileh.root, 'recarray', r1)
 
@@ -864,7 +923,7 @@ class RecArrayIO(unittest.TestCase):
         r2 = fileh.root.recarray.read()
 
         assert r1.tostring() == r2.tostring()
-        
+
         fileh.close()
         os.remove(file)
 
@@ -889,7 +948,7 @@ class RecArrayIO(unittest.TestCase):
         if verbose:
             print "\noffseted recarray --> ", r1
         assert r1._byteoffset > 0
-        
+
         # Save it in a table:
         fileh.createTable(fileh.root, 'recarray', r1)
 
@@ -897,7 +956,7 @@ class RecArrayIO(unittest.TestCase):
         r2 = fileh.root.recarray.read()
 
         assert r1.tostring() == r2.tostring()
-        
+
         fileh.close()
         os.remove(file)
 
@@ -932,7 +991,7 @@ class RecArrayIO(unittest.TestCase):
         r2 = fileh.root.recarray.read()
 
         assert r1.tostring() == r2.tostring()
-        
+
         fileh.close()
         os.remove(file)
 
@@ -953,7 +1012,7 @@ class DefaultValues(unittest.TestCase):
         # Fill the table with nrows records
         for i in xrange(nrows):
             if i == 3 or i == 4:
-                table.row['var2'] = ((2,2),(2,2))  #*-* 
+                table.row['var2'] = ((2,2),(2,2))  #*-*
             # This injects the row values.
             table.row.append()
 
@@ -962,14 +1021,20 @@ class DefaultValues(unittest.TestCase):
         table.flush()
 
         # Create a recarray with the same default values
-        buffer = [[[["abcd"]*2]*2, ((1,1),(1,1)), 2, 3.1, 4.2, 5, "e"]]
-        r=records.array(buffer*nrows,
-                        formats='(2,2)a4,(2,2)i4,i2,f8,f4,i2,a1')  #*-*
-        
+        buffer = [[
+            ["abcd"]*2,
+            [["abcd"]*2]*2,
+            (1,1),
+            ((1,1),(1,1)),
+            2, 3.1, 4.2, 5, "e"]]
+        r = records.array(
+            buffer*nrows,
+            formats='(2,)a4,(2,2)a4,(2,)i4,(2,2)i4,i2,f8,f4,i2,a1')  #*-*
+
         # Assign the value exceptions
-        r.field("c2")[3] = ((2,2), (2,2))  #*-*
-        r.field("c2")[4] = ((2,2), (2,2))  #*-*
-        
+        r.field("c4")[3] = ((2,2), (2,2))  #*-*
+        r.field("c4")[4] = ((2,2), (2,2))  #*-*
+
         # Read the table in another recarray
         r2 = table.read()
 
@@ -984,10 +1049,110 @@ class DefaultValues(unittest.TestCase):
             print r
 
         assert r.tostring() == r2.tostring()
-        
+
         fileh.close()
         os.remove(file)
 
+class RecordT(IsDescription):
+    var0 = IntCol(1, shape=1)   # native int
+    var1 = IntCol([1], shape=(1,))   # 1-D int (one element)
+    var2_s = IntCol([1,1], shape=2)   # 1-D int (two elements)
+    var2 = IntCol([1,1], shape=(2,))   # 1-D int (two elements)
+    var3 = IntCol([[0,0],[1,1]], shape=(2,2))   # 2-D int
+
+class ShapeTestCase(unittest.TestCase):
+
+    def setUp(self):
+
+        # Create an instance of an HDF5 Table
+        self.file = tempfile.mktemp(".h5")
+        self.fileh = openFile(self.file, "w")
+        self.populateFile()
+
+    def populateFile(self):
+        table = self.fileh.createTable(self.fileh.root, 'table', RecordT)
+        row = table.row
+        # Fill the table with some rows with default values
+        for i in xrange(1):
+            row.append()
+
+        # Flush the buffer for this table
+        table.flush()
+
+    def tearDown(self):
+        self.fileh.close()
+        os.remove(self.file)
+        cleanup(self)
+
+    #----------------------------------------
+
+    def test00(self):
+        "Checking scalar shapes"
+
+        if self.reopen:
+            self.fileh.close()
+            self.fileh = openFile(self.file)
+        table = self.fileh.root.table
+
+        if verbose:
+            print "The values look like:", table.cols.var0[:]
+            print "They should look like:", [1]
+
+        # The real check
+        assert table.cols.var0[:].tolist() == [1]
+
+    def test01(self):
+        "Checking undimensional (one element) shapes"
+
+        if self.reopen:
+            self.fileh.close()
+            self.fileh = openFile(self.file)
+        table = self.fileh.root.table
+
+        if verbose:
+            print "The values look like:", table.cols.var1[:]
+            print "They should look like:", [[1]]
+
+        # The real check
+        assert table.cols.var1[:].tolist() == [[1]]
+
+    def test02(self):
+        "Checking undimensional (two elements) shapes"
+
+        if self.reopen:
+            self.fileh.close()
+            self.fileh = openFile(self.file)
+        table = self.fileh.root.table
+
+        if verbose:
+            print "The values look like:", table.cols.var2[:]
+            print "They should look like:", [[1,1]]
+
+        # The real check
+        assert table.cols.var2[:].tolist() == [[1,1]]
+        assert table.cols.var2_s[:].tolist() == [[1,1]]
+
+    def test03(self):
+        "Checking bidimensional shapes"
+
+        if self.reopen:
+            self.fileh.close()
+            self.fileh = openFile(self.file)
+        table = self.fileh.root.table
+
+        if verbose:
+            print "The values look like:", table.cols.var3[:]
+            print "They should look like:", [[[0,0],[1,1]]]
+
+        # The real check
+        assert table.cols.var3[:].tolist() == [[[0,0],[1,1]]]
+
+
+class ShapeTestCase1(ShapeTestCase):
+    reopen = 0
+
+class ShapeTestCase2(ShapeTestCase):
+    reopen = 1
 
 #----------------------------------------------------------------------
 
@@ -996,23 +1161,12 @@ def suite():
     niter = 1
     #heavy = 1  # Uncomment this only for testing purposes
 
-    #theSuite.addTest(unittest.makeSuite(CompressUCLTablesTestCase))
-    #theSuite.addTest(unittest.makeSuite(BasicWriteTestCase))
-    #theSuite.addTest(unittest.makeSuite(RecArrayOneWriteTestCase))
-    #theSuite.addTest(unittest.makeSuite(RecArrayTwoWriteTestCase))
-    #theSuite.addTest(unittest.makeSuite(RecArrayThreeWriteTestCase))
-    #theSuite.addTest(unittest.makeSuite(DefaultValues))
-    #theSuite.addTest(unittest.makeSuite(RecArrayIO))
-    #theSuite.addTest(unittest.makeSuite(BigTablesTestCase))
-
     for n in range(niter):
         theSuite.addTest(unittest.makeSuite(BasicWriteTestCase))
         theSuite.addTest(unittest.makeSuite(DictWriteTestCase))
         theSuite.addTest(unittest.makeSuite(RecArrayOneWriteTestCase))
         theSuite.addTest(unittest.makeSuite(RecArrayTwoWriteTestCase))
         theSuite.addTest(unittest.makeSuite(RecArrayThreeWriteTestCase))
-        theSuite.addTest(unittest.makeSuite(CompressLZOTablesTestCase))
-        theSuite.addTest(unittest.makeSuite(CompressUCLTablesTestCase))
         theSuite.addTest(unittest.makeSuite(CompressZLIBTablesTestCase))
         theSuite.addTest(unittest.makeSuite(CompressTwoTablesTestCase))
         theSuite.addTest(unittest.makeSuite(IterRangeTestCase))
@@ -1020,9 +1174,14 @@ def suite():
         theSuite.addTest(unittest.makeSuite(getColRangeTestCase))
         theSuite.addTest(unittest.makeSuite(DefaultValues))
         theSuite.addTest(unittest.makeSuite(RecArrayIO))
+        theSuite.addTest(unittest.makeSuite(ShapeTestCase1))
+        theSuite.addTest(unittest.makeSuite(ShapeTestCase2))
     if heavy:
+        theSuite.addTest(unittest.makeSuite(CompressLZOTablesTestCase))
+        theSuite.addTest(unittest.makeSuite(CompressUCLTablesTestCase))
+        theSuite.addTest(unittest.makeSuite(CompressBZIP2TablesTestCase))
         theSuite.addTest(unittest.makeSuite(BigTablesTestCase))
-            
+
     return theSuite
 
 
