@@ -6,11 +6,12 @@ import tempfile
 
 from tables import *
 # Next imports are only necessary for this test suite
-from tables import Group, Leaf, Table, Array
+from tables import Group, Leaf, Table, Array, File
+from tables.Node import HiddenNodeMixin
 
-from test_all import verbose, heavy
-
-#heavy = 1  # Uncomment this only for test purposes!
+from test_all import verbose, heavy, cleanup
+# To delete the internal attributes automagically
+unittest.TestCase.tearDown = cleanup
 
 # Test Record class
 class Record(IsDescription):
@@ -21,7 +22,7 @@ class Record(IsDescription):
     var5 = Float32Col()           # float  (single-precision)
 
 class TreeTestCase(unittest.TestCase):
-    mode  = "w" 
+    mode  = "w"
     title = "This is the table title"
     expectedrows = 10
     appendrows = 5
@@ -33,7 +34,7 @@ class TreeTestCase(unittest.TestCase):
         self.h5file = openFile(self.file, self.mode, self.title)
         self.populateFile()
         self.h5file.close()
-            
+
     def populateFile(self):
         group = self.h5file.root
         maxshort = 1 << 15
@@ -45,42 +46,43 @@ class TreeTestCase(unittest.TestCase):
                                         filters = None,
                                         expectedrows = self.expectedrows)
             # Get the record object associated with the new table
-            d = table.row 
+            d = table.row
             # Fill the table
             for i in xrange(self.expectedrows):
                 d['var1'] = '%04d' % (self.expectedrows - i)
-                d['var2'] = i 
+                d['var2'] = i
                 d['var3'] = i % maxshort
                 d['var4'] = float(i)
                 d['var5'] = float(i)
                 d.append()      # This injects the Record values
             # Flush the buffer for this table
             table.flush()
-            
+
             # Create a couple of arrays in each group
             var1List = [ x['var1'] for x in table.iterrows() ]
             var4List = [ x['var4'] for x in table.iterrows() ]
 
             self.h5file.createArray(group, 'var1', var1List, "1")
             self.h5file.createArray(group, 'var4', var4List, "4")
-            
+
             # Create a new group (descendant of group)
             group2 = self.h5file.createGroup(group, 'group'+str(j))
             # Iterate over this new group (group2)
             group = group2
-    
+
     def tearDown(self):
         # Close the file
         if self.h5file.isopen:
             self.h5file.close()
 
         os.remove(self.file)
+        cleanup(self)
 
     #----------------------------------------
 
     def test00_getNode(self):
         "Checking the File.getNode() with string node names"
-        
+
         if verbose:
             print '\n', '-=' * 30
             print "Running %s.test00_getNode..." % self.__class__.__name__
@@ -93,10 +95,8 @@ class TreeTestCase(unittest.TestCase):
             nodenames.append(object._v_pathname)
 
         assert nodenames == nodelist
-        
         if verbose:
             print "getNode(pathname) test passed"
-            
         nodegroups = ['/', '/group0', '/group0/group1', '/group0/group1/group2']
         nodenames = ['var1', 'var4']
         nodepaths = []
@@ -116,8 +116,6 @@ class TreeTestCase(unittest.TestCase):
 
         if verbose:
             print "getNode(groupname, name) test passed"
-            
-        
         nodelist = ['/', '/group0', '/group0/group1', '/group0/group1/group2',
                     '/table0']
         nodenames = []
@@ -138,7 +136,6 @@ class TreeTestCase(unittest.TestCase):
         assert nodenames == ['/', '/group0', '/group0/group1',
                              '/group0/group1/group2',
                              ]
-        
         if verbose:
             print "getNode(groupname, classname='Group') test passed"
 
@@ -160,11 +157,9 @@ class TreeTestCase(unittest.TestCase):
                              '/group0/var1', '/group0/var4',
                              '/group0/group1/var1', '/group0/group1/var4',
                              ]
-        
         if verbose:
             print "getNode(groupobject, name, classname='Array') test passed"
 
-            
     def test01_getNodeClass(self):
         "Checking the File.getNode() with instances"
 
@@ -180,17 +175,16 @@ class TreeTestCase(unittest.TestCase):
         assert isinstance(table, Table)
         table = self.h5file.getNode(self.h5file.root.group0, "table1")
         assert isinstance(table, Table)
-        
+
         # This should return an array instance
         arr = self.h5file.getNode("/group0/var1")
         assert isinstance(arr, Array)
         assert isinstance(arr, Leaf)
-        
+
         # And this a Group
         group = self.h5file.getNode("/group0", "group1", "Group")
         assert isinstance(group, Group)
 
-        
     def test02_listNodes(self):
         "Checking the File.listNodes() method"
 
@@ -219,7 +213,6 @@ class TreeTestCase(unittest.TestCase):
                              '/group0/group1', '/group0/table1',
                              '/group0/var1', '/group0/var4',
                              ]
-        
         if verbose:
             print "listNodes(pathname) test passed"
 
@@ -233,12 +226,12 @@ class TreeTestCase(unittest.TestCase):
                 for object in objectlist:
                     nodenames.append(object._v_pathname)
 
-        assert nodenames == ['/group0/group1', '/group0/table1', 
+        assert nodenames == ['/group0/group1', '/group0/table1',
                              '/group0/var1', '/group0/var4',
                              '/group0/group1/group2', '/group0/group1/table2',
                              '/group0/group1/var1', '/group0/group1/var4',
                              ]
-        
+
         if verbose:
             print "listNodes(groupobject) test passed"
 
@@ -246,21 +239,21 @@ class TreeTestCase(unittest.TestCase):
         for node in objects:
             try:
                 objectlist = self.h5file.listNodes(node, 'Leaf')
-            except LookupError:
+            except TypeError:
                 if verbose:
                     (type, value, traceback) = sys.exc_info()
-                    print "\nGreat!, the next LookupError was catched!"
+                    print "\nGreat!, the next TypeError was catched!"
                     print value
             else:
                 for object in objectlist:
                     nodenames.append(object._v_pathname)
 
-        assert nodenames == ['/group0/table1', 
+        assert nodenames == ['/group0/table1',
                              '/group0/var1', '/group0/var4',
                              '/group0/group1/table2',
                              '/group0/group1/var1', '/group0/group1/var4',
                              ]
-        
+
         if verbose:
             print "listNodes(groupobject, classname = 'Leaf') test passed"
 
@@ -268,28 +261,28 @@ class TreeTestCase(unittest.TestCase):
         for node in objects:
             try:
                 objectlist = self.h5file.listNodes(node, 'Table')
-            except LookupError:
+            except TypeError:
                 if verbose:
                     (type, value, traceback) = sys.exc_info()
-                    print "\nGreat!, the next LookupError was catched!"
+                    print "\nGreat!, the next TypeError was catched!"
                     print value
             else:
                 for object in objectlist:
                     nodenames.append(object._v_pathname)
 
-        assert nodenames == ['/group0/table1', 
+        assert nodenames == ['/group0/table1',
                              '/group0/group1/table2',
                              ]
-        
+
         if verbose:
             print "listNodes(groupobject, classname = 'Table') test passed"
 
         # Reset the warning
         #warnings.filterwarnings("default", category=UserWarning)
-            
+
     def test03_TraverseTree(self):
         "Checking the File.walkGroups() method"
-        
+
         if verbose:
             print '\n', '-=' * 30
             print "Running %s.test03_TraverseTree..." % self.__class__.__name__
@@ -307,13 +300,12 @@ class TreeTestCase(unittest.TestCase):
 
         assert groups == ["/", "/group0", "/group0/group1",
                           "/group0/group1/group2"]
-                          
+
         assert tables == ["/table0", "/group0/table1", "/group0/group1/table2"]
 
         assert arrays == ['/var1', '/var4',
                           '/group0/var1', '/group0/var4',
                           '/group0/group1/var1', '/group0/group1/var4']
-                          
         if verbose:
             print "walkGroups() test passed"
 
@@ -329,17 +321,17 @@ class TreeTestCase(unittest.TestCase):
 
         assert groups == ["/group0/group1",
                           "/group0/group1/group2"]
-                          
+
         assert tables == ["/group0/group1/table2"]
 
         assert arrays == ['/group0/group1/var1', '/group0/group1/var4']
-        
+
         if verbose:
             print "walkGroups(pathname) test passed"
 
     def test04_walkNodes(self):
         "Checking File.walkNodes"
-        
+
         if verbose:
             print '\n', '-=' * 30
             print "Running %s.test04_walkNodes..." % self.__class__.__name__
@@ -351,10 +343,9 @@ class TreeTestCase(unittest.TestCase):
         arrays = []
         for group in self.h5file.walkNodes(classname="Group"):
             groups.append(group._v_pathname)
-            for table in group._f_walkNodes(classname='Table'):
+            for table in group._f_walkNodes(classname='Table', recursive=0):
                 tables.append(table._v_pathname)
-                
-        # Test the recursivity    
+        # Test the recursivity
         for table in self.h5file.root._f_walkNodes('Table', recursive=1):
             tables2.append(table._v_pathname)
 
@@ -370,7 +361,7 @@ class TreeTestCase(unittest.TestCase):
         assert arrays == ['/var1', '/var4',
                           '/group0/var1', '/group0/var4',
                           '/group0/group1/var1', '/group0/group1/var4']
-                          
+
         if verbose:
             print "File.__iter__() and Group.__iter__ test passed"
 
@@ -386,26 +377,26 @@ class TreeTestCase(unittest.TestCase):
 
         assert groups == ["/group0/group1",
                           "/group0/group1/group2"]
-                          
+
         assert tables == ["/group0/group1/table2"]
 
         assert arrays == ['/group0/group1/var1', '/group0/group1/var4']
-        
+
         if verbose:
             print "walkNodes(pathname, classname) test passed"
 
 
 class DeepTreeTestCase(unittest.TestCase):
     """Checks for maximum deepest level in PyTables trees.
-    
+
     Right now, the maximum depth for object tree is determined by the
     maximum recursion level offered by Python (which for my platform
     is a number between 768 and 1024).
-    
+
     """
     def test00_deepTree(self):
         "Checking creation of large depth object tree Variable"
-        
+
         # Here we put a more conservative limit to deal with more platforms
         # With maxdepth = 512 this test would take less than 20 MB
         # of main memory to run, which is quite reasonable nowadays.
@@ -414,13 +405,13 @@ class DeepTreeTestCase(unittest.TestCase):
             maxdepth = 1024  # Only for big machines!
         else:
             maxdepth = 256  # This should be safe for most machines
-        
+
         if verbose:
             print '\n', '-=' * 30
             print "Running %s.test00_deepTree..." % \
                   self.__class__.__name__
             print "Maximum depth tested :", maxdepth
-                  
+
         # Open a new empty HDF5 file
         file = tempfile.mktemp(".h5")
         #file = "deep.h5"
@@ -442,7 +433,7 @@ class DeepTreeTestCase(unittest.TestCase):
             pathname = group._v_pathname
         # Close the file
         fileh.close()
-        
+
         # Open the previous HDF5 file in read-only mode
         fileh = openFile(file, mode = "r")
         group = fileh.root
@@ -467,31 +458,30 @@ class DeepTreeTestCase(unittest.TestCase):
             print # This flush the stdout buffer
         # Close the file
         fileh.close()
-        
+
         # Then, delete the file
         os.remove(file)
-        
+
+
 class WideTreeTestCase(unittest.TestCase):
     """Checks for maximum number of children for a Group.
-    
-    
     """
+
     def test00_Leafs(self):
-        """Checking creation of large number of leafs (1024) per group 
-        
+        """Checking creation of large number of leafs (1024) per group
+
         Variable 'maxchildren' controls this check. PyTables support
         up to 4096 children per group, but this would take too much
         memory (up to 64 MB) for testing purposes (may be we can add a
         test for big platforms). A 1024 children run takes up to 30 MB.
         A 512 children test takes around 25 MB.
-        
         """
 
         import time
         if heavy:
             maxchildren = 4096
         else:
-            maxchildren = 256            
+            maxchildren = 256
         if verbose:
             print '\n', '-=' * 30
             print "Running %s.test00_wideTree..." % \
@@ -539,16 +529,16 @@ class WideTreeTestCase(unittest.TestCase):
         fileh.close()
         # Then, delete the file
         os.remove(file)
-        
+
+
     def test01_wideTree(self):
-        """Checking creation of large number of groups (1024) per group 
-        
+        """Checking creation of large number of groups (1024) per group
+
         Variable 'maxchildren' controls this check. PyTables support
         up to 4096 children per group, but this would take too much
         memory (up to 64 MB) for testing purposes (may be we can add a
         test for big platforms). A 1024 children run takes up to 30 MB.
         A 512 children test takes around 25 MB.
-        
         """
 
         import time
@@ -601,21 +591,196 @@ class WideTreeTestCase(unittest.TestCase):
         fileh.close()
         # Then, delete the file
         os.remove(file)
-        
+
+
+
+class MyHiddenGroup(Group, HiddenNodeMixin):
+    _c_classId = 'HGROUP'
+
+class MyHiddenArray(Array, HiddenNodeMixin):
+    _c_classId = 'HARRAY'
+
+class MyFile(File):
+    def createHGroup(self, where, name, title="", filters=None, _log=True):
+        def doCreateNode():
+            return MyHiddenGroup(title, new=True, filters=filters)
+        return self._createNode(
+            where, name, MyHiddenGroup, doCreateNode, _log)
+
+    def createHArray(self, where, name, object, title="", _log=True):
+        def doCreateNode():
+            return MyHiddenArray(object, title)
+        return self._createNode(
+            where, name, MyHiddenArray, doCreateNode, _log)
+
+
+
+class HiddenTreeTestCase(unittest.TestCase):
+
+    """Check for hidden groups, leaves and hierarchies."""
+
+    def setUp(self):
+        self.h5fname = tempfile.mktemp('.h5')
+        self.h5file = openFile(
+            self.h5fname, 'w', title = "Test for hidden nodes")
+        self.h5file.__class__ = MyFile  # what a hack!
+
+        self.visible = []  # list of visible object paths
+        self.hidden = []  # list of hidden object paths
+
+        # Create some visible nodes: a, g, g/a1, g/a2, g/g, g/g/a.
+        h5f = self.h5file
+        a = h5f.createArray('/', 'a', [0]);
+        g = h5f.createGroup('/', 'g');
+        g_a1 = h5f.createArray(g, 'a1', [0]);
+        g_a2 = h5f.createArray(g, 'a2', [0]);
+        g_g = h5f.createGroup(g, 'g');
+        g_g_a = h5f.createArray(g_g, 'a', [0]);
+
+        self.visible.extend(['/a', '/g', '/g/a1', '/g/a2', '/g/g', '/g/g/a'])
+
+        # Create some hidden nodes: ha, hg, hg/a, hg/ha, g/ha.
+        ha = h5f.createHArray('/', 'ha', [0]);
+        hg = h5f.createHGroup('/', 'hg');
+        hg_a = h5f.createArray(hg, 'a', [0]);
+        hg_ha = h5f.createHArray(hg, 'ha', [0]);
+        g_ha = h5f.createHArray(g, 'ha', [0]);
+
+        self.hidden.extend(['/ha', '/hg', '/hg/a', '/hg/ha', '/g/ha'])
+
+
+    def tearDown(self):
+        self.h5file.close()
+        self.h5file = None
+        os.remove(self.h5fname)
+
+
+    def test00_objects(self):
+        """Absence of hidden nodes in `File.objects`."""
+
+        objects = self.h5file.objects
+
+        for vpath in self.visible:
+            self.assert_(vpath in objects,
+                         "Missing visible node ``%s`` from ``File.objects``." % vpath)
+        for hpath in self.hidden:
+            self.assert_(hpath not in objects,
+                         "Found hidden node ``%s`` in ``File.objects``." % hpath)
+
+
+    def test01_getNode(self):
+        """Node availability via `File.getNode()`."""
+
+        h5f = self.h5file
+
+        for vpath in self.visible:
+            node = h5f.getNode(vpath)
+        for hpath in self.hidden:
+            node = h5f.getNode(hpath)
+
+
+    def test02_walkGroups(self):
+        """Hidden group absence in `File.walkGroups()`."""
+
+        hidden = self.hidden
+
+        for group in self.h5file.walkGroups('/'):
+            pathname = group._v_pathname
+            self.assert_(not isinstance(group, HiddenNodeMixin),
+                         "Walked across hidden group ``%s``." % pathname)
+            self.assert_(pathname not in hidden,
+                         "Walked across hidden ordinary group ``%s``." % pathname)
+
+
+    def test03_walkNodes(self):
+        """Hidden node absence in `File.walkNodes()`."""
+
+        hidden = self.hidden
+
+        for node in self.h5file.walkNodes('/'):
+            pathname = node._v_pathname
+            self.assert_(not isinstance(node, HiddenNodeMixin),
+                         "Walked across hidden node ``%s``." % pathname)
+            self.assert_(pathname not in hidden,
+                         "Walked across hidden ordinary node ``%s``." % pathname)
+
+
+    def test04_listNodesVisible(self):
+        """Listing visible nodes under a visible group."""
+
+        hidden = self.hidden
+
+        for node in self.h5file.listNodes('/g'):
+            pathname = node._v_pathname
+            self.assert_(not isinstance(node, HiddenNodeMixin),
+                         "Listed hidden node ``%s``." % pathname)
+            self.assert_(pathname not in hidden,
+                         "Listed hidden ordinary node ``%s``." % pathname)
+
+
+    def test05_listNodesHidden(self):
+        """Listing visible nodes under a hidden group."""
+
+        hidden = self.hidden
+
+        node_to_find = '/hg/a'
+        found_node = False
+        for node in self.h5file.listNodes('/hg'):
+            pathname = node._v_pathname
+            if pathname == node_to_find:
+                found_node = True
+            self.assert_(not isinstance(node, HiddenNodeMixin),
+                         "Listed hidden node ``%s``." % pathname)
+            self.assert_(pathname in hidden,
+                         "Listed hidden ordinary node ``%s``." % pathname)
+
+        self.assert_(found_node,
+                     "Hidden ordinary node ``%s`` was not listed." % node_to_find)
+
+
+    def test06_reopen(self):
+        """Reopening a file with hidden nodes."""
+
+        self.h5file.close()
+        self.h5file = openFile(self.h5fname)
+        self.test00_objects()
+
+
+    def test07_move(self):
+        """Moving a node between hidden and visible groups."""
+
+        objects = self.h5file.objects
+
+        self.assert_('/hg/a' not in objects)
+        self.h5file.moveNode('/hg/a', '/g', 'a')
+        self.assert_('/g/a' in objects)
+        self.h5file.moveNode('/g/a', '/hg', 'a')
+        self.assert_('/hg/a' not in objects)
+
+
+    def test08_remove(self):
+        """Removing a visible group with hidden children."""
+
+        self.assert_('/g/ha' in self.h5file)
+        self.h5file.root.g._f_remove(recursive=True)
+        self.assert_('/g/ha' not in self.h5file)
+
+
+
 #----------------------------------------------------------------------
 
 def suite():
     theSuite = unittest.TestSuite()
     # This counter is useful when detecting memory leaks
     niter = 1
-    #heavy=1  # see at the beginning
+    #heavy=1
 
     #theSuite.addTest(unittest.makeSuite(DeepTreeTestCase))
-    
     for i in range(niter):
         theSuite.addTest(unittest.makeSuite(TreeTestCase))
         theSuite.addTest(unittest.makeSuite(DeepTreeTestCase))
         theSuite.addTest(unittest.makeSuite(WideTreeTestCase))
+        theSuite.addTest(unittest.makeSuite(HiddenTreeTestCase))
 
     return theSuite
 
