@@ -5,7 +5,9 @@ import tempfile
 
 from tables import *
 
-from test_all import verbose
+from test_all import verbose, cleanup
+# To delete the internal attributes automagically
+unittest.TestCase.tearDown = cleanup
 
 # Test Record class
 class Record(IsDescription):
@@ -35,8 +37,8 @@ class RangeTestCase(unittest.TestCase):
 
     def tearDown(self):
         self.fileh.close()
-
         os.remove(self.file)
+        cleanup(self)
 
     #----------------------------------------
 
@@ -46,7 +48,7 @@ class RangeTestCase(unittest.TestCase):
         # Save a record
         i = self.maxshort
         rec['var1'] = '%04d' % (i)
-        rec['var2'] = i 
+        rec['var2'] = i
         rec['var3'] = i
         rec['var4'] = float(i)
         rec['var5'] = float(i)
@@ -68,12 +70,12 @@ class RangeTestCase(unittest.TestCase):
         # Save a record
         i = self.maxshort
         rec['var1'] = '%04d' % (i)
-        rec['var2'] = i 
+        rec['var2'] = i
         rec['var3'] = i % self.maxshort
         rec['var5'] = float(i)
         try:
             rec['var4'] = "124"
-        except KeyError:
+        except ValueError:
             if verbose:
                 (type, value, traceback) = sys.exc_info()
                 print "\nGreat!, the next TypeError was catched!"
